/*
 * Flickr.java
 *
 *  Copyright (c) 2006, James G. Driscoll
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr;

import java.util.logging.Level;
import java.util.logging.Logger;
import java.io.IOException;
import java.util.prefs.Preferences;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

/**
 * Flickr access information.  Also includes some static methods to perform
 * certain generic tasks.
 *
 * @author Jim Driscoll <a href="mailto:jgd@jamesgdriscoll.com">jgd@jamesgdriscoll.com</a>
 */
public class Flickr {
    private static String apiKey;
    private static String sharedSecret;
    
    private static final String baseURL = "http://api.flickr.com/services/rest/";
    
    // No constructor - this class is static only.
    private Flickr() {
    }
    
    
    /**
     * Set the API key for use in other parts of the library.  Note:  This should
     * be the only place where you have to know your API key.  You'll need to get
     * your key from the <a href="http://www.flickr.com/services/api/keys/">Flickr API site</a>.
     * <P>
     * Note:  This call has the side effect of setting java.net.useSystemProxies to true.
     * If you don't know what that means, don't worry about it.
     * 
     * @param newApiKey Flickr API key
     */
    public static void setApiKey(String newApiKey) {
        apiKey = newApiKey;
        
        // I'm unaware of any time I WOULDN'T want to set this
        // I'm stumped why it's not the default.
        // It's certainly a poor design choice to put this here, though.
        // TODO: Figure out a better place to put this, or take it out.
        System.setProperty("java.net.useSystemProxies","true");
    }
    
    /**
     * Get the API key, after having stored it via setApiKey.
     * This call should be made after <code>Flickr.setApiKey()</code>, or an
     * exception will be thrown.
     * @see Flickr#setApiKey(String)
     * @return apiKey  The Flickr API key.
     */
    public static String getApiKey() {
        if (apiKey == null) throw new FlickrRuntimeException("API Key not set");
        return apiKey;
    }
    
    /**
     * Set the shared secret for use in other parts of the library.  Note:  This should
     * be the only place where you have to know your Shared Secret.  You'll need to get
     * the Shared Secret from Flickr at the same place you get your API Key.  Setting the
     * Shared Secret also signs all requests automatically.
     *
     * @param newSharedSecret Flickr Shared Secret
     */
    public static void setSharedSecret(String newSharedSecret) {
        sharedSecret = newSharedSecret;
    }
    
    /**
     * Get the Shared Secret for this app.  Attempts to call this method before 
     * <code>Flickr.setSharedSecret()</code> will result in an exception.
     * @see Flickr#setSharedSecret(String)
     * @return The shared secret.
     */
    public static String getSharedSecret() {
        if (sharedSecret == null) throw new FlickrRuntimeException("Shared Secret not set");
        return sharedSecret;
    }
    
    /**
     * Get the base URL for all requests.
     * @return the base URL for all requests to Flickr.
     */
    public static String getBase() {
        return baseURL;
    }
}
