/*
 * License.java
 *
 *  Copyright (c) 2006, James G. Driscoll
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import org.jdom.Element;

/**
 *
 * Encapsulates all the information on a License option for Flickr.  To get a list
 * of valid license options from Flickr, use the <code>License.getLicenses()</code>
 * method.
 *
 * @see License#getLicenses()
 * @author Jim Driscoll <a href="mailto:jgd@jamesgdriscoll.com">jgd@jamesgdriscoll.com</a>
 *
 */
public class License {
    
    private static List<License> licenses = null;
    
    private String id = null;
    private String name = null;
    private URL url = null;
    
    /**
     * Creates a new instance of License
     */
    private License(Element license) throws FlickrException {
        if (license == null) throw new NullPointerException("license may not be null");
        this.id = license.getAttributeValue("id");
        if (id == null) throw new FlickrException("ID for license not found");
        this.name = license.getAttributeValue("name");
        if (name == null) throw new FlickrException("Name for license not found");
        try {
            // Add special casing for default license
            String urlStr = license.getAttributeValue("url");
            if (urlStr == null) throw new FlickrException("URL for license not found");
            if (this.id.equals("0") && urlStr.equals("")) {
                this.url = new URL("http://www.flickr.com/account/prefs/license/");
            } else {
                this.url = new URL(urlStr);
            }
        } catch (MalformedURLException mue) {
            throw new FlickrException("Malformed URL for id: "+this.id,mue);
        }
    }
    
    /**
     * Hardcoded creation of license.  Used for the "default license"
     */
    private License(String id, String name, URL url) {
        this.id = id;
        this.name = name;
        this.url = url;
    }
    
    /**
     * Get a list of license options available.  This response is cached - it's
     * only checked once.
     * @return A list of License options available.
     * @throws FlickrException on any error.
     */
    public static List<License> getLicenses() throws FlickrException {
        if (licenses != null)  return licenses;
        licenses = new ArrayList<License>();
        Request req = new Request();
        req.setParameter("method","flickr.photos.licenses.getInfo");
        List<Element> elist = req.getResponse().getRootElement().getChild("licenses").getChildren("license");
        for (Element l : elist) {
            License license = new License(l);
            licenses.add(license);
        }
        return new ArrayList(licenses);
    }
    
    /**
     * Gets the License option that matches the supplied ID.
     * @param id The License Option ID to search for.  Must not be null.
     * @return The license option corresponding to the supplied ID.
     * @throws FlickrException on any error, including id not found.
     */
    public static License findByID(String id) throws FlickrException {
        if (id == null) throw new NullPointerException("ID must not be null");
        if (licenses == null) getLicenses();
        License license = null;
        for (License l : licenses) {
            if (l.getID().equals(id)) license = l;
        }
        if (license == null) throw new FlickrException("ID not found, id: "+id);
        return license;
    }
    
    /**
     * Get the ID for this license option.
     * @return ID of the license option.
     */
    public String getID() {
        return id;
    }
    
    /**
     * Get the name of this license option.
     * @return Name of the license option.
     */
    public String getName() {
        return name;
    }
    
    /**
     * Get the URL for this license option.
     * @return URL of the license.
     */
    public URL getURL() {
        return url;
    }
    
    /**
     * Equals function for License options.  Compares ID values.
     */
    public boolean equals(Object o) {
        if (o == null) return false;
        if (!(o instanceof License)) return false;
        License l = (License)o;
        return this.getID().equals(l.getID());
    }
        
    /**
     * Return the HashCode of this License, based on ID.
     * @return int value representing a fairly unique hashcode.
     */
    public int hashCode() {
        return this.getID().hashCode();
    }
}
