/*
 * Note.java
 *
 *  Copyright (c) 2006, James G. Driscoll 
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr;

import java.awt.Rectangle;
import org.jdom.Element;

/**
 * <P>All the infomation about a Note attached to a Photo at Flickr. Notes in Flickr
 * are areas of a photo associated with text.</P>
 * 
 * <P>Coordinates 
 * for the points returned are in pixels, based on the 500px image size shown 
 * on individual photo pages. </P>
 *
 * @see Photo#getNotes()
 *
 * @author Jim Driscoll <a href="mailto:jgd@jamesgdriscoll.com">jgd@jamesgdriscoll.com</a>
 */
public class Note {
    
    private String id;
    private User author;
    private Rectangle area;
    private String text;
    
    private boolean deleted = false;
    
    /** 
     * Creates a new instance of a Note.
     */
    Note(Element e) throws FlickrException {
        if (e == null) throw new FlickrException("Null Note not allowed");
        this.id = e.getAttributeValue("id");
        if (id == null) throw new FlickrException("Note ID not set");
        String auth = e.getAttributeValue("author");
        if (auth == null) throw new FlickrException("Note author not set");
        this.author = new User(auth);
        String x = e.getAttributeValue("x");
        String y = e.getAttributeValue("y");
        String w = e.getAttributeValue("w");
        String h = e.getAttributeValue("h");
        if (x == null || y == null || w == null || h == null) throw new FlickrException("Note rectangle not set");
        try {
            this.area = new Rectangle(Integer.parseInt(x),Integer.parseInt(y),Integer.parseInt(w),Integer.parseInt(h));
        } catch (NumberFormatException nfe) {
            throw new FlickrException("Error parsing Note rectangle: "+x+" "+y+" "+w+" "+h,nfe);
        }
        this.text = e.getText();
    }
    
    /**
     * Get the unique identifier for this note.
     * @return The ID for this note.
     */
    public String getID() {
        if (deleted) throw new FlickrRuntimeException("Cannot perform action on deleted object");
        return id;
    }
    
    /**
     * Get the Author of this note.
     * @return The user who authored the note.
     */
    public User getAuthor() {
        if (deleted) throw new FlickrRuntimeException("Cannot perform action on deleted object");
        return author;
    }
    
    /**
     * Get the X origin point for the note.
     * @return The X coordinate of the note.
     */
    public int getX() {
        if (deleted) throw new FlickrRuntimeException("Cannot perform action on deleted object");
        return (int)area.getX();
    }
    
    /**
     * Get the Y origin point for the note.
     * @return The Y coordinate of the note.
     */
    public int getY() {
        if (deleted) throw new FlickrRuntimeException("Cannot perform action on deleted object");
        return (int)area.getY();
    }
    
    /**
     * Get the width of the note.
     * @return The width of the note.
     */
    public int getWidth() {
        if (deleted) throw new FlickrRuntimeException("Cannot perform action on deleted object");
        return (int)area.getWidth();
    }
    
    /**
     * Get the height of the note.
     * @return The Height of the note.
     */
    public int getHeight() {
        if (deleted) throw new FlickrRuntimeException("Cannot perform action on deleted object");
        return (int)area.getHeight();
    }
    
    /**
     * Get the area covered by the note.  Coordinates for the area are in pixels, 
     * based on the 500px image size shown on individual photo pages.
     * @return A Rectangle representing the area covered by the note.
     */
    public Rectangle getArea() {
        if (deleted) throw new FlickrRuntimeException("Cannot perform action on deleted object");
        return area;
    }
    
    /** 
     * Get the text of the note.
     * @return The text associated with this note.
     */
    public String getText() {
        if (deleted) throw new FlickrRuntimeException("Cannot perform action on deleted object");
        return text;
    }
    
    /**
     * <P>Modify the content and area of a Note.  Coordinates for the area are in pixels, 
     * based on the 500px image size shown on individual photo pages.</P>  
     * <P>Requires authentication with WRITE permission.</P>
     * @param area Rectangle describing the area covered by the note. Must not be null.
     * @param text Text of the note. Must not be null.
     * @throws FlickrException on any error.
     */
    public void update(Rectangle area, String text) throws FlickrException {
        if (deleted) throw new FlickrRuntimeException("Cannot perform action on deleted object");
        if (area == null) throw new NullPointerException("Area cannot be null");
        if (text == null) throw new NullPointerException("Text cannot be null");
        
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photos.notes.edit");
        req.setParameter("note_id",id);
        req.setParameter("note_x",""+area.x);
        req.setParameter("note_y",""+area.y);
        req.setParameter("note_w",""+area.width);
        req.setParameter("note_h",""+area.height);
        req.setParameter("note_text",text);
        req.getResponse();
        
        this.area = new Rectangle(area);
        this.text = text;
    }
    
    /** 
     * <P>Delete this note.  Further method calls on this object are invalid 
     * after this call.</P>
     * <P>Requires authentication with WRITE privleges.</P>
     * @throws FlickrException on any error.
     */
    public void delete() throws FlickrException {
        if (deleted) throw new FlickrRuntimeException("Cannot perform action on deleted object");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photos.notes.delete");
        req.setParameter("note_id",id);
        req.getResponse();
        deleted = true;
    }
   
    /**
     * Check if this Note is equal to the object supplied.  Uses id for comparison.
     * @param o Object to compare.
     * @return If the object to equal to this object.
     */
    public boolean equals(Object o) {
        if (o == null) return false;
        if (! (o instanceof Note)) return false;
        return this.getID().equals(((Note)o).getID());
    }
    
    /**
     * Return the HashCode of this Note, based on ID.
     * @return int value representing a fairly unique hashcode.
     */
    public int hashCode() {
        return this.getID().hashCode();
    }
}
