/*
 * PhotoSearch.java
 *
 *  Copyright (c) 2006, James G. Driscoll 
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Class used for complex searches on Photo objects.  This class should
 * be populated with search terms, and then passed to
 * <code>Photo.search(PhotoSearch)</code>.
 *
 * @see org.jickr.Photo#search(PhotoSearch)
 * @author Jim Driscoll <a href="mailto:jgd@jamesgdriscoll.com">jgd@jamesgdriscoll.com</a>
 */
public class PhotoSearch {
    
    private User user = null;
    private String tags = null;
    private Tagmode tagmode = null;
    private String text = null;
    private Privacy priv = null;
    private Date minUpload = null;
    private Date maxUpload = null;
    private Date minTaken = null;
    private Date maxTaken = null;
    private List<License> licenses = null;
    private Sort sort = null;
    
    /** 
     * Creates a new instance of PhotoSearch 
     */
    public PhotoSearch() {
    }
    
    /**
     * Search for Photos owned by the specified user.
     * @param user The user to search for.  Null to search for all users.
     */
    public void setUser(User user) {
        this.user = user;
    }
    
    /**
     * Get the user in the search term.
     * @return The owner of the Photos you want to find.  Null if unset.
     */
    public User getUser() {
        return user;
    }
    
    /**
     * Search for Photos with the following tags.
     * @param tags Comma separated list of tags. Null to search for all.
     */
    public void setTags(String tags) {
        this.tags = tags;
    }
    
    /**
     * Get the comma separated list of tags to search for.
     * @return The list of tags. Null if unset.
     */
    public String getTags() {
        return tags;
    }
    
    /**
     * Set the join mode for multi-tag searches.  ANY looks for any tag in the list (OR).
     * ALL looks for every tag in the list (AND).  Defaults to ANY if unset.  May be
     * set to null to indicate default behavior.
     *
     * @param tagmode Join mode for multi-tag searches.
     */
    public void setTagMode(Tagmode tagmode) {
        this.tagmode = tagmode;
    }
    
    /**
     * Get the join mode for multi-tag searches.
     * @return The join mode for tag searches, null if unset.
     */
    public Tagmode getTagMode() {
        return tagmode;
    }
    
    /**
     *  Full text search string.  Flickr will look for this string in
     *  descriptions, title and tags.  Set to null to clear.
     * @param searchText The string to use to search Flickr.
     */
    public void setSearchText(String searchText) {
        this.text = searchText;
    }
    
    /**
     * Get the text search string.
     * @return The search string for full text search, null if unset.
     */
    public String getSearchText() {
        return text;
    }
    
    /**
     * Set the privacy level for the search.
     * @param privacy Privacy level.
     */
    public void setPrivacy(Privacy privacy) {
        this.priv = privacy;
    }
    
    /**
     * Get the privacy level for the search.
     * @return The Privacy level for the search.
     */
    public Privacy getPrivacy() {
        return priv;
    }
    
    /** 
     * Set the sort order for Photos returned by the search.
     * @param sort Sort order
     */
    public void setSort(Sort sort) {
        this.sort = sort;
    }
    
    /** 
     * Get the sort order for Photos returned by the search.
     * @return The sort order requested.
     */
    public Sort getSort() {
        return sort;
    }
    
    /**
     * Set the minimum upload date for the search.
     * @param minUpload The date which is the minimum value for an upload date on Flickr.
     */
    public void setMinUpload(Date minUpload) {
        this.minUpload = minUpload;
    }
    
    /**
     * Get the minimum upload date for the search.
     * @return The minimum upload date for each Photo returned.
     */
    public Date getMinUpload() {
        return minUpload;
    }
    
    /**
     * Set the maximum upload date for the search.
     * @param maxUpload The maximum upload date for each Photo returned.
     */
    public void setMaxUpload(Date maxUpload) {
        this.maxUpload = maxUpload;
    }
    
    /**
     * Get the maximum upload date for the search.
     * @return The maximum upload date for each Photo returned.
     */
    public Date getMaxUpload() {
        return maxUpload;
    }

    /**
     * Set the minimum date taken for the search.
     * @param minTaken The minimum date taken for each Photo returned.
     */
    public void setMinTaken(Date minTaken) {
        this.minTaken = minTaken;
    }
    
    /**
     * Get the minimum date taken for the search.
     * @return The minimum date taken for each Photo returned.
     */
    public Date getMinTaken() {
        return minTaken;
    }
    
    /**
     * Set the maximum date taken for the search.
     * @param maxTaken The maximum date taken for each Photo returned.
     */
    public void setMaxTaken(Date maxTaken) {
        this.maxTaken = maxTaken;
    }
    
    /**
     * Get the maximum date taken for the search.
     * @return The maximum date taken for each Photo returned.
     */
    public Date getMaxTaken() {
        return maxTaken;
    }
    
    /**
     * Set the list of licenses to use in the search.  Set to null or empty 
     * list to clear.
     * @see License#getLicenses()
     * @param licenses The list of licenses that the Photos should have.
     */
    public void setLicenses(List<License> licenses) {
        this.licenses = new ArrayList<License>(licenses);
    }
    
    /**
     * Get the list of licenses to use in the search.
     * @see License#getLicenses()
     * @return A list of licenses to use.
     */
    public List<License> getLicenses() {
        return this.licenses;
    }
            
    /**
     * Privacy value associated with an object on Flickr.  Used to set search
     * conditions via <code>PhotoSearch.setPrivacy(Privacy)</code>
     * @see PhotoSearch#setPrivacy(Privacy)
     */
    public enum Privacy {
        /**
         * The object is visible to everyone.
         */
        PUBLIC,
        /**
         * The object is visible only to people marked as a friend on Flickr.
         */
        FRIENDS,
        /**
         * The object is visible only to people marked as family on Flickr.
         */
        FAMILY,
        /**
         * The object is visible only to people marked either a friend or family on Flickr.
         */
        FRIENDSANDFAMILY,
        /**
         * The object is visible only to the authenticated user, it is completely private.
         */
        PRIVATE }

    /**
     * The join mode for multitag searches.  Used to set search conditions via 
     * <code>PhotoSearch.setTagMode(Tagmode)</code>
     * @see PhotoSearch#setTagMode(Tagmode)
     */
    public enum Tagmode {
        /**
         * Search for any tag in the taglist - i.e., this is an OR operation.
         */
        ANY,
        /**
         * Search for every tag in the taglist - i.e., this is an AND operation.
         */
        ALL }
    
    /**
     * The order to sort returned Photos for a PhotoSearch.  Used to create search
     * conditions via <code>PhotoSearch.setSort(Sort)</code>
     * @see PhotoSearch#setSort(Sort)
     */
    public enum Sort {
        /**
         * Ordered by Date Posted, ascending order.
         */
        POST_ASC,
        /**
         * Ordered by Date Posted, descending order.  This is the default if unset.
         */
        POST_DESC,
        /**
         * Ordered by Date Taken, ascending order.
         */
        TAKEN_ASC,
        /**
         * Ordered by Date Taken, descending order.
         */
        TAKEN_DESC,
        /** 
         * Ordered by Interestingness, descending.
         */
        INTERESTING_DESC,
        /**
         * Ordered by Interestingness, ascending.
         */
        INTERESTING_ASC,
        /**
         * Ordered in the way that Flickr thinks is most relevant to the search.
         */
        RELEVANCE
    }
    
}