/*
 * PhotoSet.java
 *
 *  Copyright (c) 2006, James G. Driscoll 
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr;

import java.util.ArrayList;
import java.util.List;
import org.jdom.Document;
import org.jdom.Element;

/**
 * A Flickr PhotoSet.  Created by calls that get a <code>List&lt;PhotoSet&gt;</code>,
 * such as <code>User.getPhotoSets()</code> and <code>PhotoSet.findByID(id)</code>
 * @see User#getPhotoSets()
 * @see PhotoSet#findByID(String)
 * @author Jim Driscoll <a href="mailto:jgd@jamesgdriscoll.com">jgd@jamesgdriscoll.com</a>
 */
public class PhotoSet {
    
    private String description = null;
    private String title = null;
    private String id = null;
    private int numPhotos = 0;
    private Photo primaryPhoto = null;
    private boolean deleted = false;
    private boolean checked = false;
    
    /**
     * Creates a new instance of PhotoSet, using the Element returned by Flickr.
     * <code>PhotoSet</code> is created in lists via FlickUser.getPhotoSets(), or
     * via the <code>PhotoSet.findByID(id)</code> call.
     *
     *
     * @param photoset The Photoset information.  Must not be null.
     * @throws FlickrException On any error.
     */
    PhotoSet(Element photoset) throws FlickrException {
        if (photoset == null) throw new NullPointerException("Photoset cannot be null");
        this.id = photoset.getAttributeValue("id");
        this.primaryPhoto = new Photo(photoset.getAttributeValue("primary"));
        String photoCountStr = photoset.getAttributeValue("photos");
        if (photoCountStr == null)
            throw new FlickrException("Read invalid value for PhotoCount from Flickr");
        this.numPhotos = Integer.parseInt(photoCountStr);
        this.title = photoset.getChildText("title");
        this.description = photoset.getChildText("description");
    }
    
    /**
     * Created a PhotoSet given it's ID string. Note: This call will not look up
     * the PhotoSet.
     * @param id Unique String Identifier.
     */
    PhotoSet(String id) throws FlickrException {
        if (id == null) throw new FlickrRuntimeException("id cannot be null");
        this.id = id;
    }
    
    /**
     * Get a new PhotoSet identified by the supplied ID.
     * @param id - The unique ID of the PhotoSet in Flickr.
     * @return A PhotoSet that is identified by the supplied ID.
     */
    public static PhotoSet findByID(String id) throws FlickrException {
        PhotoSet set = new PhotoSet(id);
        set.checkInfo();
        return set;
    }
    
    /**
     * Create a new PhotoSet.  Returns PhotoSet created.  Requires WRITE authentication.
     * <P>NOTE: Flickr may take a few moments to register the new PhotoSet, and it may not 
     * immediately be found, though it will eventually be there.</P>
     * @param title Title of the PhotoSet. May not be null.
     * @param description Description of the PhotoSet. If left null, this will be left blank.
     * @param primaryPhoto The primary photo associated with this PhotoSet.  May not be null.
     * @return The newly created PhotoSet.
     * @throws FlickrException on any error.
     */
    public static PhotoSet create(String title, String description, Photo primaryPhoto) throws FlickrException {
        if (title == null) throw new NullPointerException("Title may not be null");
        if (primaryPhoto == null) throw new NullPointerException("Primary Photo may not be null");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photosets.create");
        req.setParameter("title",title);
        if (description != null) req.setParameter("description",description);
        req.setParameter("primary_photo_id",primaryPhoto.getID());
        Document doc = req.getResponse();
        String id;
        try {
            id = doc.getRootElement().getChild("photoset").getAttributeValue("id");
        } catch (NullPointerException npe) {
            throw new FlickrException("Malformed XML error",npe);
        }
        return new PhotoSet(id);
    }
    
    /**
     * Order the PhotoSets for display on Flickr.  A list of PhotoSets are passed 
     * to this method, and the authenticated user's PhotoSets are ordered via that
     * list.  PhotoSets that are not on the list are put at the end of the list in 
     * ID order.  Request WRITE level authentication and permissions.
     * @param setlist List of PhotoSets, in the order that you wish them to appear on Flickr.  Must not be null.
     * @throws FlickrException on any error.
     */
    public static void orderSets(List<PhotoSet> setlist) throws FlickrException {
        if (setlist == null) throw new NullPointerException("Set List must not be null");
        if (setlist.size() == 0) throw new IllegalArgumentException("Set List must have at least one PhotoSet");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photosets.orderSets");
        boolean first = true;
        String psid = "";
        for (PhotoSet ps : setlist) {
            if (!first) psid += ",";
            psid += ps.getID();
            first = false;
        }
        req.setParameter("photoset_ids",psid);
        req.getResponse();
    }
    
    /**
     * Sets up all the photos in a PhotoSet.  Requires WRITE level 
     * authentication and permissions.
     * @param primaryPhoto The main photo to associate with this PhotoSet.
     * @param photolist The list of Photos that the PhotoSet should contain.
     * @throws FlickrException on any error.
     */
    public void setPhotos(Photo primaryPhoto, List<Photo> photolist) throws FlickrException {
        if (primaryPhoto == null) throw new NullPointerException("Primary Photo must not be null");
        if (photolist == null) throw new NullPointerException("Photo List must not be null");
        if (photolist.size() == 0) throw new IllegalArgumentException("Photo List must contain photos");
        if (!photolist.contains(primaryPhoto)) throw new IllegalArgumentException("Photo List much contain Primary Photo");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photosets.editPhotos");
        req.setParameter("photoset_id",id);
        req.setParameter("primary_photo_id",primaryPhoto.getID());
        boolean first = true;
        String phid = "";
        for (Photo photo : photolist) {
            if (!first) phid += ",";
            phid += photo.getID();
            first = false;
        }
        req.setParameter("photo_ids",phid);
        req.getResponse();
    }

    /**
     * Delete the current PhotoSet.  Requires WRITE authentication.
     * <P>NOTE: Flickr may take a few moments to register the deletion, and it 
     * may still be found momentarily in a search, though it will eventually be 
     * gone.</P>
     * <P>No other calls may be made on this object after deletion.</P>
     * 
     */
    public void delete() throws FlickrException {
        if (deleted) throw new IllegalStateException("PhotoSet already Deleted");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photosets.delete");
        req.setParameter("photoset_id",id);
        req.getResponse();
        deleted = true;
        return;
    }

    /**
     * Get the Title of the PhotoSet.
     * @return Title of the PhotoSet.
     */
    public String getTitle() throws FlickrException {
        if (deleted) throw new IllegalStateException("PhotoSet already Deleted");
        if (title == null && !checked) checkInfo();
        return title;
    }
    
    /**
     * Get the Description of the PhotoSet.
     * @return The description of the PhotoSet.
     */
    public String getDescription() throws FlickrException {
        if (deleted) throw new IllegalStateException("PhotoSet already Deleted");
        if (description == null && !checked) checkInfo();
        return description;
    }
    
    /**
     * Get the number of Photos in the PhotoSet. Note: this value is cached, and
     * is not updated once checked.  Use PhotoSet.getPhotos().size() to get an
     * up to date value, though note that this is an expensive call.
     * @return The number of photos in the PhotoSet.
     */
    public int getNumPhotos() throws FlickrException {
        if (deleted) throw new IllegalStateException("PhotoSet already Deleted");
        if (numPhotos == 0 && !checked) checkInfo();
        return numPhotos;
    }
    
    /**
     * Get the ID of the PhotoSet.  This value is unique.
     * @return The ID of this PhotoSet on Flickr.
     */
    public String getID() {
        if (deleted) throw new IllegalStateException("PhotoSet already Deleted");
        return id;
    }
    
    /** 
     * Get the primary Photo associated with this PhotoSet.
     * @return Primary Photo associated with this PhotoSet.
     */
    public Photo getPrimaryPhoto() throws FlickrException {
        if (deleted) throw new IllegalStateException("PhotoSet already Deleted");
        if (primaryPhoto == null && !checked) checkInfo();
        return primaryPhoto;
    }
    
    /**
     * Retrieve the list of photos associated with this PhotoSet.  This
     * value is not cached, and may be expensive to execute, depending on
     * how many Photos are in the PhotoSet.
     * @return photoset A list of photos for this Photoset.
     * @throws FlickrException for almost any error.
     */
    public PhotoList getPhotos() throws FlickrException {
        if (deleted) throw new IllegalStateException("PhotoSet already Deleted");
        Request req = new Request();
        req.setParameter("method","flickr.photosets.getPhotos");
        req.setParameter("photoset_id",id);
        return new PhotoList(req);
    }
    
    /**
     * Get a list of comments for this PhotoSet.
     * @return A list of comments for this PhotoSet.
     * @throws FlickrException in the event of any error.
     */
    public List<Comment> getComments() throws FlickrException {
        if (deleted) throw new IllegalStateException("PhotoSet already Deleted");
        List<Comment> commentList = new ArrayList();
        Request req = new Request();
        req.setParameter("method","flickr.photosets.comments.getList");
        req.setParameter("photoset_id",id);
        
        Document doc = req.getResponse();
        Element root = doc.getRootElement();
        
        List<Element> comments = root.getChild("comments").getChildren("comment");
        for (Element comment: comments) {
            commentList.add(new Comment(Comment.Type.PHOTOSETCOMMENT,comment));
        }
        return commentList;
    }
    
    
    /**
     * <P>Adds a comment to this Photo, returns the new comment ID.  Since Flickr
     * doesn't offer a way to search for a comment via ID, the user will
     * have to do a <code>getComments()</code> and search for the comment with
     * <code>Comment.getID()</code> if they want a Comment object representing this
     * added comment.</P>
     *
     * <P>This call requires that the user be authenticated at the WRITE level.</P>
     *
     * @see org.jickr.Photo#getComments()
     * @see org.jickr.Comment#getID()
     * @return The comment's ID string
     * @throws FlickrException in the event of a failure.  Note that if an exception is thrown, it's possible (thought unlikely) that a comment was added anyway.
     */
    public String addComment(String commentText) throws FlickrException {
        if (deleted) throw new IllegalStateException("PhotoSet already Deleted");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photosets.comments.addComment");
        req.setParameter("photoset_id",id);
        req.setParameter("comment_text",commentText);
        
        Element root = req.getResponse().getRootElement();
        
        try {
            String commentID = root.getChild("comment").getAttributeValue("id");
            // Sigh.  Flickr doesn't let you get a comment via commentID
            // So instead, we'll have just return the ID.
            // The developer can search for it if they wanna.
            return commentID;
        } catch (NullPointerException npe) {
            throw new FlickrException("Oddly formed XML in response",npe);
        }
    }
    
    /**
     * Update the Title and Description of the PhotoSet.  Requires WRITE
     * authentication.
     * @param titleText New title of the PhotoSet.  May not be null.
     * @param descText New description of the PhotoSet.  If null, description is not updated.
     * @throws FlickrException on any error.
     */
    public void updateTitleDesc(String titleText,String descText) throws FlickrException {
        if (deleted) throw new IllegalStateException("PhotoSet already Deleted");
        if (titleText == null) throw new NullPointerException("Title may not be null");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photosets.editMeta");
        req.setParameter("photoset_id",id);
        req.setParameter("title",titleText);
        if (descText != null) req.setParameter("description",descText);
        req.getResponse();
        title = titleText;
        if (descText != null) description = descText;
        return;
    }
    
    /**
     * Adds a photo to a PhotoSet.  Requires WRITE authentication.
     * @param photo Photo to add to the PhotoSet.
     */
    public void add(Photo photo) throws FlickrException {
        if (deleted) throw new IllegalStateException("PhotoSet already Deleted");
        if (photo == null) throw new NullPointerException("Photo cannot be null");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photosets.addPhoto");
        req.setParameter("photoset_id",id);
        req.setParameter("photo_id",photo.getID());
        req.getResponse();
        return;
    }
    
    /**
     * Removes a photo to a PhotoSet.  Requires WRITE authentication.
     * @param photo Photo to remove from the PhotoSet.
     */
    public void remove(Photo photo) throws FlickrException {
        if (deleted) throw new IllegalStateException("PhotoSet already Deleted");
        if (photo == null) throw new NullPointerException("Photo cannot be null");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photosets.removePhoto");
        req.setParameter("photoset_id",id);
        req.setParameter("photo_id",photo.getID());
        req.getResponse();
        return;
    }
    
    private void checkInfo() throws FlickrException {
        Request req = new Request();
        req.setParameter("method","flickr.photosets.getInfo");
        req.setParameter("photoset_id",id);
        
        Document doc = req.getResponse();
        Element root = doc.getRootElement();
        Element photoset = root.getChild("photoset");
        this.id = photoset.getAttributeValue("id");
        this.primaryPhoto = new Photo(photoset.getAttributeValue("primary"));
        String photoCountStr = photoset.getAttributeValue("photos");
        if (photoCountStr == null)
            throw new FlickrException("Read Invalid value for PhotoCount from Flickr");
        this.numPhotos = Integer.parseInt(photoCountStr);
        this.title = photoset.getChildText("title");
        this.description = photoset.getChildText("description");
        checked = true;
    }
    
    /**
     * Convert this PhotoSet to a String.
     * @return The photoset title.
     */
    public String toString() {
        if (deleted) throw new IllegalStateException("PhotoSet already Deleted");
        return title;
    }
    
    /**
     * Check equality.  Uniqueness based on ID.
     * @param o - Object to compare.
     * @return true if these objects have the same ID string.
     */
    public boolean equals(Object o) {
        if (o == null) return false;
        if (! (o instanceof PhotoSet)) return false;
        return this.getID().equals(((PhotoSet)o).getID());
    }
    
    /**
     * Return the HashCode of this PhotoSet, based on ID.
     * @return int value representing a fairly unique hashcode.
     */
    public int hashCode() {
        return this.getID().hashCode();
    }
}
