/*
 * Created on May 27, 2004
 *
 */
package com.arellano.ramon.atacama.config;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Frame;
import java.awt.TextArea;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import com.arellano.ramon.atacama.entity.Album;
import com.arellano.ramon.atacama.log.Logger;
import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * @author ramon.arellano
 *
 * Loads properties file and retrieves values for Atacama
 */
public class Config {

	private static final String DEFAULT_PROPERTIES_PATH = "properties\\atacama.properties";
	
	//A handle to the unique instance.
	static private Config theConfig = null;

    //Properties
	static private Properties props = null;

	//Path to the properties file
	static private String propertiesFilePath = null;

	//Path to the application root directory
	static private String applicationDir = null;

	/**
	 * Constructor
	 */
	private Config(String configPath) {		
		
		//Set the properties file path and the application root path
		Config.propertiesFilePath = configPath;
		applicationDir = determineApplicationDir();
		
		props = new Properties();
		
		//Loads the given properties file
		try {
			System.out.println("Using properties file: " + configPath);
		 	FileInputStream inputStream = new FileInputStream(new File(configPath));
			props.load(inputStream);
		} catch(FileNotFoundException exception) {
			throwException("Properties file was not found. " + configPath, exception);
		} catch(Exception exception) {
			throwException("Failed while loading properties. " + configPath, exception);
		}
	}
 
 	/**
 	 * Instantiates Config with the specified properties from file.
 	 * Will keep using default properties file if Config has allready been instantiated.
 	 * 
 	 * @param propertiesPath The properties path
 	 * @return The config instance
 	 */
	static public Config getInstance(String propertiesPath) {
		if(theConfig == null) {
			theConfig = new Config(propertiesPath);
		}
		
		return theConfig;
	}

	
	/**
	 * Determines the application root directory
	 * 
	 * @return The application root directory
	 */
	private static String determineApplicationDir() {
		//Determine if application is running from root application path (WSAD)
		//or from the bin directory (standalone)
		File currentDirectory = null;
		try {
			currentDirectory = new File(new File(".").getCanonicalPath());
		} catch(IOException exception) {
			throwException("Failed to determine application root directory.", exception);
		}

		if(currentDirectory.getName().equals("bin")) {
			currentDirectory = currentDirectory.getParentFile();
		} else if(currentDirectory.getName().equals("lib")) {
			currentDirectory = currentDirectory.getParentFile();
		}
			
		//Set the app root dir varialble
		return currentDirectory.getPath() + File.separatorChar;	
	}

 
	/**
	 * Returns the unique instance of the logger
	 * 
	 * @return The unique instance of this class.
	 */
	public static Config getInstance() {
	   if(theConfig == null) {

			String applicationRootDir = determineApplicationDir();

	   		//Get path to default properties
			File configFile = new File(applicationRootDir + DEFAULT_PROPERTIES_PATH);
			
			if(!configFile.exists()) {
				throwException("Failed to find the default properties file: \n\n '" + configFile + "'. \n\nExiting...");
			}
			
			//Found the properties file, load props
			theConfig = new Config(applicationRootDir + DEFAULT_PROPERTIES_PATH);
	   }
	   
	   return theConfig;
	}


	/**
	 * Stores the current properties to file
	 */
	public void storePropertiesFile() throws AtacamaException {
		//Stores the given properties file
		try {
			FileOutputStream outStream = new FileOutputStream(new File(propertiesFilePath));
			props.store(outStream, "ATACAMA PROPERTIES");
		} catch(FileNotFoundException exception) {
			throw new AtacamaException("Properties file was not found. " + propertiesFilePath, exception);
		} catch(Exception exception) {
			throw new AtacamaException("Failed while loading properties. " + propertiesFilePath, exception);
		}
	}


	/**
	 * Returns the path to the properties file
	 * 
	 * @return The proerties file path
	 */
	public String getPropertiesFilePath() {
		return propertiesFilePath;
	}


	/**
	 * Puts together the directory name for a specific album
	 * 
	 * @param album The album to create directory for
	 * @return The albums directory
	 */
	public String getAlbumDir(Album album, boolean relativeToGallery) {
		String albumPath = "";
		
		if(!relativeToGallery) {
			albumPath += theConfig.getString("common.dir.target.gallery");
		}
		
		albumPath += 
			theConfig.getString("common.dir.target.albums")
			+ album.getYear()
			+ "-"
			+ album.getCountry()
			+ "-"
			+ album.getMonth()
			+ " "
			+ album.getAlbumName()
			+ File.separator;
					
		return albumPath;
	}

	/**
	 * Creates the directory path html for a specific album
	 * 
	 * @param album The album to return photos directory for
	 * @param relativeToGallery True if the returned directory should be relative to gallery,
	 * false if it should be absolute.
	 * @return Thew albums photo directory
	 */
	public String getAlbumPhotosDir(Album album, boolean relativeToGallery) {
		return getAlbumDir(album, relativeToGallery)
			+ getString("photo.dir.photos");
	}
	
	/**
	 * Returns the root directory to the application
	 * @return The application directory
	 */
	public String getApplicationDir() {
		return applicationDir;
	}
	
	
	/**
	 * Checks that a property is not empty
	 * 
	 * @param propertyName The property name (key)
	 * @param stringValue The value to be checked
	 */
	private void assertNotEmpty(String propertyName, String stringValue) {
		if("".equals(stringValue) || stringValue == null) {
			throwException("Property '"
				+ propertyName
				+ "' is empty or does not exist in properties file "
				+ propertiesFilePath); 
		}		
	}

	/**
	 * Returns a property string value
	 * @return The property string value
	 */
	public String getString(String propertyName) {
		String stringValue = props.getProperty(propertyName);
		
		assertNotEmpty(propertyName, stringValue);
				
		stringValue.trim();
		
		//If key contains substring ".dir." it is a directory.
		//Ensure all directories are supplied with and backslash character at the end
		if(propertyName.indexOf(".dir.") != -1) {
			if(stringValue.lastIndexOf(File.separatorChar) != stringValue.length() - 1) {
				stringValue += File.separatorChar;
			}
		}
		
		return stringValue;
	}

	/**
	 * Returns a property as a boolean value
	 * @param propertyName Property name
	 * @return The property boolean value
	 */
	public boolean getBool(String propertyName) {
		String stringValue = getString(propertyName);

		assertNotEmpty(propertyName, stringValue);
		
		if("true".equals(stringValue.trim())) {
			return true;
		} else if("false".equals(stringValue.trim())) {
			return false;
		} else {
			throwException("Expected value true or false for property: " + propertyName);
			return false;
		}
		
	}

	/**
	 * Returns a property as an integer value
	 * @param propertyName The property name
	 * @return The property integer value
	 */
	public int getInt(String propertyName) {
		String stringValue = getString(propertyName);

		assertNotEmpty(propertyName, stringValue);
		
		try {
			return Integer.parseInt(stringValue.trim());
		} catch(NumberFormatException exception) {
			throwException("Expected value to be an integer for property: " + propertyName);
			return -1;
		}		
	}


	/**
	 * Returns a property as an float value
	 * @param propertyName The property name
	 * @return The property integer value
	 */
	public float getFloat(String propertyName) {
		String stringValue = getString(propertyName);

		assertNotEmpty(propertyName, stringValue);
		
		try {
			return Float.parseFloat(stringValue.trim());
		} catch(NumberFormatException exception) {
			throwException("Expected value to be an integer for property: " + propertyName);
			return -1;
		}		
	}
	
	/**
	 * Sets a property
	 * 
	 * @param propertyName The property to set
	 * @param value The value to set
	 */
	public void setProperty(String propertyName, String value) {
		props.setProperty(propertyName, value);
	}
	
	/**
	 * Combines a message from the inputs a sends to trowMessage
	 * @param message The exception message to throw
	 * @param exception An previous exception to add
	 */
	private static void throwException(String message, Exception exception) {
		throwException(message + "\n" + exception.toString());
	}
	
	/**
	 * Throws a runtime exception with a describing message
	 * @param message The exception message
	 */
	private static void throwException(String message) {

		/** 
		 * Inner class for handling the events of the config window
		 */		
		class FatalDialog extends Frame implements WindowListener, ActionListener {
			/**
			 * FatalDialog Constructor
			 */
			public FatalDialog(String message) {
				addWindowListener(this);
				setLayout(new BorderLayout());

				TextArea messageArea =
					new TextArea(message, 3, 3, TextArea.SCROLLBARS_NONE);
				messageArea.setEditable(false);
				
				Button okButton = new Button("OK");
				okButton.addActionListener(this);
				
				add(messageArea, "Center");
				add(okButton, "South");
				setLocation(300,150);
				setSize(300, 200);
				show();				
			}

			/**
			 * Handles all actions performed in this frame
			 * @param event The action event
			 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
			 */
			public void actionPerformed(ActionEvent event) {
				System.exit(0);
			}

			//Implementation of  all window listener interface methods
			public void windowClosing(WindowEvent event) {
				System.exit(0);
			}
			public void windowClosed(WindowEvent event) {
			} 
			public void windowDeiconified(WindowEvent event) {
			}
			public void windowIconified(WindowEvent event) {
			}
			public void windowActivated(WindowEvent event) {
			}
			public void windowDeactivated(WindowEvent event) {
			}
			public void windowOpened(WindowEvent event) {
			}
		}
		
		new FatalDialog(message);
		
		String logLine = Logger.createLogLine(message, Logger.ERROR);
		
		throw new RuntimeException(logLine);	
	}	
}
