/*
 * Created on May 27, 2004
 *
 */
package com.arellano.ramon.atacama.descriptor;

import java.io.File;
import java.util.Vector;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.entity.Photo;
import com.arellano.ramon.atacama.file.FileManager;
import com.arellano.ramon.atacama.log.Logger;
import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * @author ramon.arellano
 *
 * Generates photo descriptors from media files. Can also validate existing
 * descriptors.
 * 
 */
public class DescriptorGenerator {
	
	//Logger and Config and FileManager
	static private Config theConfig = null;
	static private Logger theLogger = null;
	static private FileManager theFileManager = null;	
	
	/**
	 * Constructor
	 */
	public DescriptorGenerator() {
		theConfig = Config.getInstance();
		theLogger = Logger.getInstance();
		theFileManager = FileManager.getInstance();
	}

	/**
	 * Generates or validates all descriptors within the photos path specified
	 * by the property file. In both cases all directories and subdirectories
	 * are traversed, but if the validate flag is false, descriptors are generated
	 * and written to file.
	 * 
	 * If descriptors are found to exist already, album names, comments and publish
	 * flags will be passed from old to new, while updating file names and file sizes.
	 * 
	 * If the validate flag is set to true the log will display changes to be made.
	 * 
	 * @param validate
	 * @return An integer showing how many descriptor where updated or
	 *  need to be updated
	 */
	public int generate(boolean validate) throws AtacamaException {			
		String modeString = "";
		if(validate) {
			modeString = "Validating";
		} else {
			modeString = "Generating";
		}
		
		//Loop through all directories under the photos directory
		String photosPath = theConfig.getString("common.dir.source.photos"); 
		String[] dirList = theFileManager.listSubdirs(new File(photosPath));
		theLogger.logTrace("" + modeString + " all descriptors within " + photosPath);

		Vector dirPhotos = null;
		int numDescriptorsOutOfDate = 0;
		boolean foundPhotos = false;
		for(int index=0;index < dirList.length;index++) {
			dirPhotos = readPhotoDirectory(dirList[index]);
			if(dirPhotos == null || dirPhotos.size() < 1) {
				//No photos in directory
				continue;
			} else {
				foundPhotos = true;
			}

			//Check if there is a descriptor in the directory
			DescriptorReader descReader = new DescriptorReader();
			
			String descFileName = theConfig.getString("descriptor.file.text.photo");
			if(descReader.photoDescriptorExists(dirList[index])) {
				//There is a descriptor in the directory.
				//We need to find out if it's updated.
				//If not we will pass old information to the new descriptor
				Vector descPhotos = descReader.readPhotoDescriptor(dirList[index], false);
				
				if(descReader.validatePhotoVector(dirPhotos, descPhotos, validate)) {
					//Descriptor is up to date
					continue;
				} else {
					//Descriptor differs from photos in directory
					numDescriptorsOutOfDate++;
					if(validate) {
						theLogger.logTrace("Descriptor needs updating in directory: " + dirList[index]);
					} else {
						theLogger.logTrace("Updating descriptor in directory: " + dirList[index]);
					}
					
					Vector oldPhotos = (Vector) descPhotos.clone();
					boolean passedAllInfo = passInfoToNewDesc(oldPhotos, dirPhotos);
					
					if(!passedAllInfo) {
						theLogger.logWarning("Cannot pass all info from existing descriptor. Will create backup of descriptor in directory : " + dirList[index]);

						//Some info was not passed. Make backup of descriptor
						if(!validate) {
							descReader.writePhotoDescriptor(descPhotos, dirList[index], "photos_backup.txt");
						}
					}					
				}				
			} else {
				//There is no descriptor in the directory
				numDescriptorsOutOfDate++;
				if(validate) {
					theLogger.logTrace("Missing descriptor in directory: " + dirList[index]);
				} else {
					theLogger.logTrace("Generating new descriptor in directory: " + dirList[index]);
				}
			}
			
			//Write the new or updated descriptor
			if(!validate) {
				descReader.writePhotoDescriptor(dirPhotos, dirList[index], descFileName);						
			}
		}
		
		if(!foundPhotos) {
			theLogger.logError("There are no photos in the given photo directory.");
		}
		
		return numDescriptorsOutOfDate;
	}
		
	
	/**
	 * Passes information (gallery name, publish flag and comment) from matching photos
	 * in an old vector to a new one.
	 * 
	 * Photos are mathced first by size and last modified date, then by name.
	 * If some photos are not matched the method returns false, else true.
	 * 
	 * @param oldPhotos
	 * @param newPhotos
	 * @return True if its not possible to pass info for one or more descriptors
	 */
	public boolean passInfoToNewDesc(Vector oldPhotos, Vector newPhotos) {
		//Loop through old descriptor.
		//Pass info for photos with same size and lastmodified date.
		Photo oldPhoto = null;
		Photo newPhoto = null; 
		for(int oldIndex=0;oldIndex < oldPhotos.size();oldIndex++) {
			oldPhoto = (Photo) oldPhotos.elementAt(oldIndex);
			
			for(int newIndex=0;newIndex < newPhotos.size();newIndex++) {
				newPhoto = (Photo) newPhotos.elementAt(newIndex);
					
				//Compare size and last modified date
				if(oldPhoto.getFileSize().equals(newPhoto.getFileSize())) {
					
					//Pass gallery name, publish flag and comment
					newPhoto.setAlbumName(oldPhoto.getAlbumName());
					newPhoto.setPublish(oldPhoto.getPublish());
					newPhoto.setComment(oldPhoto.getComment());

					//Info has been passed, remove photo from old vector
					oldPhotos.setElementAt(null, oldIndex);
				}				
			}			
		}

		//Loop through old elements (that are left) and pass info if name matches
		for(int oldIndex=0;oldIndex < oldPhotos.size();oldIndex++) {
			if(oldPhotos.elementAt(oldIndex) == null) {
				continue;
			} else {
				oldPhoto = (Photo) oldPhotos.elementAt(oldIndex);
			}
			
			for(int newIndex=0;newIndex < newPhotos.size();newIndex++) {
				newPhoto = (Photo) newPhotos.elementAt(newIndex);	
					
				//Compare names
				if(oldPhoto.getPhotoName().equals(newPhoto.getPhotoName())) {
					
					//Pass gallery name, publish flag and comment
					newPhoto.setAlbumName(oldPhoto.getAlbumName());
					newPhoto.setPublish(oldPhoto.getPublish());
					newPhoto.setComment(oldPhoto.getComment());

					//Info has been passed, remove photo from old vector
					oldPhotos.setElementAt(null, oldIndex);
				}				
			}			
		}
		
		//Check if any was left out
		//Loop through old elements (that are left) and pass info if name matches
		for(int oldIndex=0;oldIndex < oldPhotos.size();oldIndex++) {
			if(oldPhotos.elementAt(oldIndex) == null) {
				continue;
			} else {
				//There are still elements in old vector
				return false;
			}
		}		
		
		return true;
	}
	
	/**
	 * Creates a vector of photo objects by reading the photos in a directory
	 * 
	 * @param path The directory path
	 * @return The vector of photos
	 */
	public Vector readPhotoDirectory(String path) {
		String extensions = theConfig.getString("descriptor.filetypes");
		File directory = new File(path);
		String[] photoList = theFileManager.findFiles(directory, extensions, false);
			
		Vector dirPhotos = null;
		if(photoList.length > 0) {
			//Found photos in directory, create vector
			dirPhotos = new Vector(photoList.length);
			
			//Add each photo by parsing the file information
			for(int photoIndex=0;photoIndex < photoList.length;photoIndex++) {
				//Parse file information
				Photo photo = new Photo();

				photo.setPhotoName(new File(photoList[photoIndex]).getName());
				photo.setPublish("false");				
				photo.setPrimary("false");				
				photo.setFileSize("" + new File(photoList[photoIndex]).length());				
				//Give the photo the name of the directory as default album name
				photo.setAlbumName(directory.getName());

				//Add the photo to the vector
				dirPhotos.add(photoIndex, photo);
			}
		}
		return dirPhotos;
	}
}
