/*
 * Created on May 28, 2004
 *
 */
package com.arellano.ramon.atacama.descriptor;

import java.io.File;
import java.util.Vector;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.entity.Album;
import com.arellano.ramon.atacama.entity.Photo;
import com.arellano.ramon.atacama.file.FileManager;
import com.arellano.ramon.atacama.log.Logger;
import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * @author ramon.arellano
 *
 * Class used to both read and write photo and album descriptor files
 */
public class DescriptorReader {

	//Logger and Config
	static private Config theConfig = null;
	static private Logger theLogger = null;
	static private FileManager theFileManager = null;

	/**
	 * Constructor
	 */
	public DescriptorReader() {
		theConfig = Config.getInstance();
		theLogger = Logger.getInstance();
		theFileManager = FileManager.getInstance();
	}

	/**
	 * Reads all photo descriptors and the album descriptors and returns
	 * a vector of albums to be publish and with all corresponding photos
	 * to be published.
	 * 
	 * @return The vector of albums containing the gallery
	 * @throws AtacamException Throws when fails
	 */
	public Vector readGallery() throws AtacamaException {
		String albumDescFilePath =
			theConfig.getString("descriptor.file.text.album");

		//Get albums from descriptor file
		Vector albums = readAlbumDescriptor(albumDescFilePath);

		if (albums == null || albums.size() < 1) {
			theLogger.logError(
				"There are no albums in your album descriptor file. "
					+ "Either select a new file, or edit the existing album descriptor file.");
		}

		//Get photos from descriptor files
		String photosPath = theConfig.getString("common.dir.source.photos");
		Vector photos = readPhotoDescriptors(photosPath);

		if (photos == null || photos.size() < 1) {
			theLogger.logError(
				"There are no photo descriptors in the photos directory. "
					+ "Either generate new photo descriptors, or select another photo directory.");
		}

		Album album = null;
		Photo photo = null;
		//For each photo, find the album it belongs to, and add it
		for (int photoIndex = 0; photoIndex < photos.size(); photoIndex++) {
			photo = (Photo) photos.elementAt(photoIndex);

			for (int albumIndex = 0;
				albumIndex < albums.size();
				albumIndex++) {
				album = (Album) albums.elementAt(albumIndex);

				if (!new Boolean(album.getPublish()).booleanValue()) {
					//Remove album if publish flag is set to false
					albums.remove(album);
					continue;
				}

				if (photo
					.getAlbumName()
					.equalsIgnoreCase(album.getAlbumName())) {
					//Photo belongs to this album, add it
					album.getPhotos().add(photo);
				}
			}
		}

		return albums;
	}

	/**
	 * Reads all photo descriptor files in a given directory and its subdirectories.
	 * Returns a vector with all photos to be published.
	 * 
	 * @param path The root path to the photo descriptors
	 * @return Vector with photos to be published from all descriptors found.
	 * @throws AtacamException Throws when fails
	 */
	public Vector readPhotoDescriptors(String path) throws AtacamaException {
		theLogger.logTrace("Reading photo descriptors from " + path);

		String[] photoDescriptorFilePaths =
			theFileManager.findFiles(
				new File(path),
				theConfig.getString("descriptor.file.text.photo"),
				true);

		Vector photos = new Vector();
		for (int index = 0; index < photoDescriptorFilePaths.length; index++) {
			String[] photoDescriptor =
				theFileManager.readTextFile(
					new File(photoDescriptorFilePaths[index]));
			Photo photo = null;

			for (int photoIndex = 0;
				photoIndex < photoDescriptor.length;
				photoIndex++) {
				try {
					photo = new Photo(photoDescriptor[photoIndex]);
					if (new Boolean(photo.getPublish()).booleanValue()) {
						//Photo has publish attribute true, add to photo vector
						String photoFilePath =
							new File(photoDescriptorFilePaths[index])
								.getParent();
						photo.setPhotoFilePath(
							new File(photoFilePath, photo.getPhotoName())
								.getPath());
						photos.add(photo);
					}
				} catch (Exception exception) {
					theLogger.logError(
						"Photo descriptor has error in line "
							+ (photoIndex + 1)
							+ ": "
							+ photoDescriptorFilePaths[index],
						exception);
				}
			}
		}

		return photos;
	}

	/**
	 * Reads the photo descriptor in the given path and returns a vector of photos
	 * with the information. One photo object per descriptor line.
	 * 
	 * @param path The file path to the descriptor
	 * @return The Vector of phots
	 * @throws AtacamException Throws when fails
	 */
	public Vector readPhotoDescriptor(String path, boolean doLog) throws AtacamaException {
		if(doLog) {
			theLogger.logTrace("Reading photo descriptor from. " + path);
		}

		String fileName = theConfig.getString("descriptor.file.text.photo");
		String[] descriptor =
			theFileManager.readTextFile(new File(path, fileName));

		if (descriptor == null || descriptor.length < 1) {
			theLogger.logError("Empty descriptor file: " + path);
		}

		Vector photos = new Vector();
		for (int index = 0; index < descriptor.length; index++) {
			try {
				Photo photo = new Photo(descriptor[index]);
				photo.setPhotoFilePath(
					new File(path, photo.getPhotoName()).getPath());

				photos.add(photo);
			} catch (Exception exception) {
				theLogger.logError(
					"Error in photo descriptor line "
						+ (index + 1)
						+ " : "
						+ path,
					exception);
			}
		}

		return photos;
	}

	/**
	 * Reads the album descriptor from given file and return a vector of
	 * albums with the information. One album per descriptor line.
	 * 
	 * @param filePath The file path to the album descriptor
	 * @return The vector of albums
	 * @throws AtacamException Throws when fails
	 */
	public Vector readAlbumDescriptor(String filePath)
		throws AtacamaException {
		theLogger.logTrace("Reading album descriptor from. " + filePath);
		String[] descriptor = theFileManager.readTextFile(new File(filePath));

		Vector albums = new Vector();
		Album album = null;

		//Loop through all albums in descriptor
		for (int index = 0; index < descriptor.length; index++) {
			if (descriptor[index].trim().equals("")) {
				//Empty line skip
				continue;
			}

			try {
				album = new Album(descriptor[index]);
				albums.add(album);
			} catch (Exception exception) {
				theLogger.logError(
					"Error in album descriptor line "
						+ (index + 1)
						+ " : "
						+ filePath
						+ " : "
						+ exception.toString());
			}
		}

		return albums;
	}

	/**
	 * Returns true if the two photo have photos with the same size and name
	 * 
	 * @param dirPhotos A photo vector
	 * @param descPhotos Another photo vector
	 * @param log True if results should be logged
	 * @return True if vectors are equal
	 */
	public boolean validatePhotoVector(
		Vector dirPhotos,
		Vector descPhotos,
		boolean log) {
		boolean result = true;

		if (dirPhotos.size() != descPhotos.size()) {
			if (log) {
				theLogger.logWarning("Number of photos do not match.");
			}

			return false;
		} else {
			Photo dirPhoto = null;
			Photo descPhoto = null;

			for (int index = 0; index < dirPhotos.size(); index++) {
				dirPhoto = (Photo) dirPhotos.elementAt(index);
				descPhoto = (Photo) descPhotos.elementAt(index);
				String difference = null;

				//This test compares file size and name
				if (!dirPhoto
					.getPhotoName()
					.equals(descPhoto.getPhotoName())) {
					result = false;
					difference = "Name";
				} else if (
					!dirPhoto.getFileSize().equals(descPhoto.getFileSize())) {
					result = false;
					difference = "File size";
				}

				if (log && difference != null) {
					theLogger.logWarning(
						difference
							+ " of photo '"
							+ dirPhoto.getPhotoName()
							+ "' differs from photo in descriptor line "
							+ (index + 1));
				}
			}
		}

		return result;
	}

	/**
	 * Returns true if the two photo vectors are identical
	 * 
	 * @param oldPhotos A photo vector
	 * @param newPhotos Another photo vector
	 * @param log True if results should be logged
	 * @return True if vectors are equal
	 */
	public boolean comparePhotoVectors(
		Vector oldPhotos,
		Vector newPhotos,
		boolean log) {
		boolean result = true;

		if (oldPhotos.size() != newPhotos.size()) {
			if (log) {
				theLogger.logWarning("Number of photos do not match.");
			}

			return false;
		} else {
			Photo oldPhoto = null;
			Photo newPhoto = null;

			for (int index = 0; index < oldPhotos.size(); index++) {
				oldPhoto = (Photo) oldPhotos.elementAt(index);
				newPhoto = (Photo) newPhotos.elementAt(index);
			
				if(!oldPhoto.equals(newPhoto)) {
					result = false;
					break;
				}
			}
		}

		return result;
	}

	/**
	 * Returns true if a photo descriptor file exists in given directory
	 * 
	 * @param path The path to the directory to search
	 * @return True if descriptor exists
	 */
	public boolean photoDescriptorExists(String path) {
		return new File(
			path,
			theConfig.getString("descriptor.file.text.photo"))
			.exists();
	}

	/**
	 * Writes a photo descriptor to the specified file using a vector of photos
	 * 
	 * @param photos Vector of photos to write
	 * @param path The path to the destination directory
	 * @param descFileName The descriptor file name
	 * @throws AtacamException Throws when fails
	 * 
	 */
	public void writePhotoDescriptor(
		Vector photos,
		String path,
		String descFileName)
		throws AtacamaException {
			
		StringBuffer fileString = new StringBuffer();

		Photo photo = null;
		for (int index = 0; index < photos.size(); index++) {
			photo = (Photo) photos.elementAt(index);

			fileString.append(photo.toString() + "\n");
		}

		theFileManager.writeTextFile(
			new File(path, descFileName),
			fileString.toString(),
			true);
	}

	/**
	 * Writes an album descriptor to the specified file using a vector of albums
	 * 
	 * @param albums Vector of albums to write
	 * @param path The file path to write to
	 * @throws AtacamException Throws when fails
	 */
	public void writeAlbumDescriptor(
		Vector albums,
		String path,
		String descFileName)
		throws AtacamaException {
			
		StringBuffer fileBufferString = new StringBuffer();

		Album album = null;
		//Read album vector into file. Works for empty albums vector.
		for (int index = 0; index < albums.size(); index++) {
			album = (Album) albums.elementAt(index);

			fileBufferString.append(album.toString() + "\n");
		}

		theFileManager.writeTextFile(
			new File(path, descFileName),
			fileBufferString.toString(),
			true);
	}
}
