/*
 * Created on May 27, 2004
 *
 */
package com.arellano.ramon.atacama.file;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;

import com.arellano.ramon.atacama.log.Logger;
import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * @author ramon.arellano
 *
 * A utility class for reading, writing, manipulating
 * and searching for files and directories
 */
public class FileManager {

	//Logger
	static private Logger theLogger = null;

	//A handle to the unique instance.
	static private FileManager theInstance = null;

	/**
	 * Constructor
	 */
	protected FileManager() {
		theLogger = Logger.getInstance();
	}

	/**
	 * Gets the unique instance of this class
	 * 
	 * @return The unique instance of this class.
	 */
	static public FileManager getInstance() {
		if (null == theInstance) {
			theInstance = new FileManager();
		}
		return theInstance;
	}

	/**
	 * Read an XML file from disk and returns the DOM document
	 * @param file The file to be read
	 * @return The DOM document
	 * @throws AtacamaException Throws exception if operation fails
	 */
	public Document readXmlFile(File file) throws AtacamaException {
		if (!file.isFile()) {
			theLogger.logError("File is not valid: " + file.getPath());
		}

		Document doc = null;
		try {
			DocumentBuilderFactory docBuilderFactory =
				DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
			doc = docBuilder.parse(file);
		} catch (Exception exception) {
			theLogger.logError(
				"Failed to parse XML file :"
					+ file.getPath()
					+ " See log for details.",
				exception);
		}

		return doc;
	}

	/**
	 * Writes a XML DOM document to file
	 * 
	 * @param doc The DOM document
	 * @param file The file to write to
	 * @throws AtacamaException Throws exception if operation fails
	 */
	public void writeXmlFile(Document doc, File file) throws AtacamaException {
		try {
			// Prepare the DOM document for writing
			DOMSource source = new DOMSource(doc);

			// Prepare the output file
			Result result = new StreamResult(file);

			//Write to string
			//StringWriter outText = new StringWriter();
			//StreamResult resultStream = new StreamResult(outText);

			// Write the DOM document to the file
			Transformer transformer =
				TransformerFactory.newInstance().newTransformer();

			//Set xml output properties	for xhmlt documents
			if (doc.getElementsByTagName("html") != null
				|| doc.getElementsByTagName("html").getLength() < 1) {
				Properties properties = transformer.getOutputProperties();
				properties.setProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
				properties.setProperty(OutputKeys.METHOD, "xml");
				properties.setProperty(OutputKeys.ENCODING, "ISO-8859-1");

				//The frames page needs another DTD than all other HTML documents
				if(file.toString().indexOf("frames.html") != -1) {
					properties.setProperty(
						OutputKeys.DOCTYPE_PUBLIC,
						"-//W3C//DTD XHTML 1.0 Frameset//EN");
					properties.setProperty(
						OutputKeys.DOCTYPE_SYSTEM,
						"http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd");
				} else {
					properties.setProperty(
							OutputKeys.DOCTYPE_PUBLIC,
							"-//W3C//DTD XHTML 1.0 Transitional//EN");
					properties.setProperty(
						OutputKeys.DOCTYPE_SYSTEM,
						"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd");					
				}
				
				transformer.setOutputProperties(properties);
			}

			//Transform the document					
			transformer.transform(source, result);
		} catch (TransformerConfigurationException exception) {
			theLogger.logError(
				"Failed to write XML DOM document to file: "
					+ file.getPath()
					+ " See log for details.",
				exception);
		} catch (TransformerException exception) {
			theLogger.logError(
				"Failed to write XML DOM document to file: "
					+ file.getPath()
					+ " See log for details.",
				exception);
		}
	}

	/**
	 * Copies a directory with subdirectories and files using DOS xcopy command
	 * 
	 * @param sourceDir The source directory
	 * @param destinationDir The destination directory
	 */
	public void copyDirectory(File sourceDir, File destinationDir)
		throws AtacamaException {
		//Source must be a valid directory
		if (!sourceDir.isDirectory()) {
			theLogger.logError(
				"Cannot copy files from path. Not a valid directory: "
					+ sourceDir.getPath());
		}

		//Destination must not be a file, and parent must be a directory
		if (destinationDir.isFile()
			|| !destinationDir.getParentFile().isDirectory()) {
			theLogger.logError(
				"Cannot copy files to path. Not a valid directory: "
					+ destinationDir.getPath());
		}

		try {
			Process process = null;
			String command =
				"xcopy \""
					+ sourceDir.getPath()
					+ "\" \""
					+ destinationDir.getPath()
					+ "\" /i/e/y";
			process = Runtime.getRuntime().exec(command);

			try {
				DataInputStream inData =
					new DataInputStream(process.getInputStream());
				String processOutput = "";
				while (processOutput != null) {
					theLogger.logTrace("Xcopy command: " + processOutput);
					processOutput = inData.readLine();
				}

				if (process.waitFor() != 0) {
					theLogger.logError(
						"Failed to copy directory "
							+ sourceDir.getPath()
							+ " to "
							+ destinationDir.getPath());
				}
			} catch (InterruptedException exception) {
				theLogger.logError(
					"Directory could not be copied because "
						+ "the xcopy process was interrupted. "
						+ exception.toString());
			}

		} catch (IOException exception) {
			theLogger.logError(
				"Failed to execute xcopy command in RunTime. "
					+ "Could not copy directories."
					+ exception.toString());
		}
	}

	/**
	 * Copies a file using DOS Xcopy command
	 * 
	 * @param sourceFile Source file
	 * @param destinationFile Destination file
	 * @throws AtacamaException Throws exception if operation fails
	 */
	public void copyFile(File sourceFile, File destinationFile)
		throws AtacamaException {
		//Source must be a valid file
		if (!sourceFile.isFile()) {
			theLogger.logError(
				"Cannot copy file. Source file not valid: "
					+ sourceFile.getPath());
		}

		//Destination parent must be a directory
		if (!destinationFile.getParentFile().isDirectory()) {
			theLogger.logError(
				"Cannot copy file to path. Not a valid directory: "
					+ destinationFile.getParentFile().getPath());
		}

		//We need to create empty destination file if it doesn't.
		//This is to avoid prompts from XCopy. (You figure it out!)
		if (!destinationFile.exists()) {
			writeTextFile(destinationFile, "", true);
		}

		try {
			Process process = null;
			String command =
				"xcopy \""
					+ sourceFile.getPath()
					+ "\" \""
					+ destinationFile.getPath()
					+ "\" /y";
			process = Runtime.getRuntime().exec(command);

			try {
				DataInputStream inData =
					new DataInputStream(process.getInputStream());
				String processOutput = "";
				while (processOutput != null) {
					processOutput = inData.readLine();

					if (processOutput != null) {
						theLogger.logTrace("XCopy command: " + processOutput);
					}
				}

				if (process.waitFor() != 0) {
					theLogger.logError(
						"Failed to copy file "
							+ sourceFile.getPath()
							+ " to "
							+ destinationFile.getPath());
				}
			} catch (InterruptedException exception) {
				theLogger.logError(
					"File could not be copied because the "
						+ "xcopy process was interrupted. "
						+ exception.toString());
			}

		} catch (IOException exception) {
			theLogger.logError(
				"Failed to execute copy command in RunTime. "
					+ "Could not copy files."
					+ exception.toString());
		}
	}

	/**	 
	 * Searches for files with specified name or extensions
	 * The pattern set is semi-colon seperated (e.g. ".jpg;.gif;.swf;").
	 * Returns all files (not directories) that match any of the given
	 * extension types.
	 * 
	 * @param directory The directory to search in
	 * @param patternSet String of patterns separated by semi-colon
	 * @param searchSubdirs True for searching subdirectories
	 * @return String arrary containing path of all matching files
	 */
	public String[] findFiles(
		File directory,
		String patternSet,
		boolean searchSubdirs) {

		Vector resultSet = new Vector();
		addFiles(resultSet, directory, patternSet, searchSubdirs);

		String[] resultList = new String[0];

		if (resultSet.size() > 0) {
			//Copy results to array
			resultList = new String[resultSet.size()];
			resultSet.toArray(resultList);
		}
		return resultList;
	}

	/**
	 * Assisting method for findFiles. Adds matching files to a resultset
	 * and enters new directories recursively.
	 * 
	 * @param resultSet The growing set of results
	 * @param directory Directory to search in
	 * @param patternSet The semi-colons separated patterns to match
	 * @param searchSubdirs True for searching subdirectories
	 */
	private void addFiles(
		Vector resultSet,
		File directory,
		String patternSet,
		boolean searchSubdirs) {

		if (!directory.isDirectory()) {
			theLogger.logWarning(
				"Path is not a valid directory: " + directory.getPath());
			return;
		}

		String[] directoryList = directory.list();

		//Loop through list and add matches to vector
		for (int index = 0; index < directoryList.length; index++) {
			File file = new File(directory.getPath(), directoryList[index]);
			if (file.isDirectory()) {
				if (searchSubdirs) {
					//Search recursively
					addFiles(resultSet, file, patternSet, true);
				} else {
					//Skip this directory
					continue;
				}
			}

			//Check if file matches, and if it does add it to the result set
			if (matchFileName(file.getName(), patternSet)) {
				resultSet.add(file.getPath());
			}
		}
	}

	/**
	 * Returns a list of all directories and subdirectories within the path
	 * @param directory The directory to get list for
	 * @return An array of strings with all directories and subdirectories
	 */
	public String[] listSubdirs(File directory) {

		Vector dirVector = new Vector();
		addDir(dirVector, directory);

		String[] dirList = new String[dirVector.size()];
		dirVector.toArray(dirList);
		return dirList;
	}

	/**
	 * Recursively adds directories. Helping method for listSubdirs
	 * 
	 * @param dirVector The growing vector of directories
	 * @param directory The current directory to search in
	 */
	private void addDir(Vector dirVector, File directory) {
		if (directory.isDirectory()) {
			//Add directory to list
			dirVector.add(directory.getAbsolutePath());

			//Look for new directories recursively
			File files[] = directory.listFiles();
			for (int index = 0; index < files.length; index++) {
				addDir(dirVector, files[index]);
			}
		}
	}

	/**
	 * Matches a file name against a set of patterns/names/extensions seperated by ;
	 * 
	 * @param fileName The file name to match
	 * @param patternSet The semi-colon separated patterns to match against
	 * @return True if match
	 */
	private boolean matchFileName(String fileName, String patternSet) {

		String[] patterns = patternSet.toLowerCase().split(";");

		for (int index = 0; index < patterns.length; index++) {
			if (fileName.toLowerCase().indexOf(patterns[index]) != -1)
				return true;
		}

		return false;
	}

	/**
	 * Reads a text file into a string. 
	 * 
	 * @param file The file to be read
	 * @return The string with the file's text
	 * @throws AtacamaException Throws exception if operation fails
	 */
	public String readTextFileToString(File file) throws AtacamaException {
		String[] stringArray = readTextFile(file);
		
		StringBuffer fileString = new StringBuffer();
		for(int index = 0; index < stringArray.length; index++) {
			fileString.append(stringArray[index] + "\n");
		}
		
		return fileString.toString();
	}
	
	/**
	 * Reads a text file into an array of strings. One string per line.
	 * 
	 * @param file The file to be read
	 * @return The array of string with the file's textlines
	 * @throws AtacamaException Throws exception if operation fails
	 */
	public String[] readTextFile(File file) throws AtacamaException {
		Vector lines = new Vector();

		try {
			FileInputStream inFile = new FileInputStream(file);
			DataInputStream inData = new DataInputStream(inFile);

			while (inData.available() > 0) {
				lines.add(inData.readLine());
			}

			inData.close();
		} catch (FileNotFoundException exception) {
			theLogger.logError(
				"Could not read file. File not found: " + file.getPath(),
				exception);
		} catch (IOException exception) {
			theLogger.logError(
				"Error reading file: " + file.getPath(),
				exception);
		}

		if (lines == null || lines.size() < 1) {
			//Empty file
			return new String[0];
		}

		String[] textLines = new String[lines.size()];
		lines.toArray(textLines);
		return textLines;
	}

	/**
	 * Writes a string array to file, one line per string.
	 * 
	 * @param file The file to write to
	 * @param content A string array containing all the text lines to write
	 * @param overwrite True if files should be overwritten
	 * @throws AtacamaException Throws exception if operation fails
	 */
	public void writeTextFile(File file, String[] content, boolean overwrite)
		throws AtacamaException {

		String fileString = "";

		if (content == null || content.length < 1) {
			theLogger.logWarning("String array to write to file is empty.");
		}

		for (int index = 0; index < content.length; index++) {
			fileString += content[index] + "\n";
		}

		writeTextFile(file, fileString, overwrite);
	}

	/**
	 * Writes string into specified file. Writes warning to log if file exist and
	 * overwrite flag is false.
	 * 
	 * @param file The file to write to
	 * @param content A string array containing all the text lines to write
	 * @param overwrite True if files should be overwritten
	 * @throws AtacamaException
	 */
	public void writeTextFile(File file, String content, boolean overwrite)
		throws AtacamaException {

		if (!file.exists() || overwrite) {
			try {
				FileOutputStream outFile = new FileOutputStream(file);
				outFile.write(content.getBytes());
				outFile.close();
			} catch (FileNotFoundException exception) {
				theLogger.logError(
					"Could not find file: "
						+ file.getPath()
						+ " : "
						+ exception.toString());
			} catch (IOException exception) {
				theLogger.logError(
					"Could not write to file: "
						+ file.getPath()
						+ " : "
						+ exception.toString());
			}
		} else {
			theLogger.logWarning(
				"Did not write file because it allready exists: "
					+ file.getPath());
		}
	}

	/**
	 * Compares two text files and sees that their content is identical
	 * 
	 * @param file1 The first file to compare
	 * @param file2 The second file to compare
	 * @return An integer which is -1 if the files have different number of lines,
	 * 0 if the files are identical, or a the number of the first line that doesn't
	 * match.
	 * @throws AtacamaException Throws exception if operation fails
	 */
	public int compareTextFiles(File file1, File file2)
		throws AtacamaException {

		if (!file1.isFile()) {
			theLogger.logError("File is not valid: " + file1.getAbsolutePath());
		}

		if (!file2.isFile()) {
			theLogger.logError("File is not valid: " + file2.getAbsolutePath());
		}

		String[] text1 = readTextFile(file1);
		String[] text2 = readTextFile(file2);

		if (text1.length != text2.length)
			return -1;

		for (int index = 0; index < text1.length; index++) {
			if (!text1[index].equals(text2[index]))
				return (index + 1);
		}

		return 0;
	}

	/**
	 * Checks the file extension of a file. Returns true for match.
	 * Extension is case insenstive and should be without preceding dot.
	 * E.g. "JPG"
	 * 
	 * @param file The file to check extension for
	 * @param extension The extension string
	 * @return True if file has the given extension
	 */
	public boolean isFileType(File file, String extension) {
		return file
			.getPath()
			.substring(file.getPath().lastIndexOf(".") + 1)
			.equalsIgnoreCase(extension);
	}
}