package com.arellano.ramon.atacama.flickr;

import org.jickr.FlickrException;
import org.jickr.PhotoSet;

import com.arellano.ramon.atacama.log.Logger;

/**
 * This class is a subclass of Flickrj Photoset, that implements comparable.
 * It is used for sorting photosets accord to year, month and country accordingly.
 * 
 * @author Ramon Kristian
 *
 */
public class ComparableSet implements Comparable {
	
	private String id = null;
	private String title = null;

	static private Logger theLogger = null;
	
	/**
	 * This is the constuctor. It only fills in the necessary fields from the photoset.
	 * ID and title. The rest will not be used for sorting.
	 * @param photoset
	 */
	public ComparableSet(PhotoSet photoset) {
		super();
		
		this.id = photoset.getID();
		
		try {
			this.title = photoset.getTitle();
		} catch (FlickrException exception) {
			theLogger.logError("Failed to get PhotoSet title sets.", exception);
		}

	}

	/**
	 * This is the implementation of the abstract method of the Comparable
	 * interface. It is used by Collections.sort to sort a List.
	 * 
	 * @param comparableSetObject
	 * @return The result of the comparison
	 */
	public int compareTo(Object comparableSetObject) {
		
		ComparableSet comparableSet = (ComparableSet) comparableSetObject;

		//Sort by type of title
		String comparableTitle1 = getComparableTitle(comparableSet.getTitle());
		String comparableTitle2 = getComparableTitle(this.getTitle());	
		if(comparableTitle1 == null && comparableTitle2 != null) {
			//Titles with year, country and month are sorted highest
			return -1;
		} else if(comparableTitle1 != null && comparableTitle2 == null) {
			return 1;
		} else if(comparableTitle1 == null && comparableTitle2 == null) {
			//Neither title has year, month and date, sort alphabetically
			return this.getTitle().compareTo(comparableSet.getTitle());
		}
		
		//Both titles have year, month and country, sort by year
		String year1 = getYear(comparableTitle1);
		String year2 = getYear(comparableTitle2);
		if(year1.compareTo(year2) != 0) {
			//Years differ, no need for further comparison
			return year1.compareTo(year2); 
		}
		
		//Titles have same year, sort by month
		Integer month1  = getMonth(comparableTitle1);
		Integer month2 = getMonth(comparableTitle2);
		if(month1.compareTo(month2) != 0) {
			//Months differ, no need for further comparison
			return month1.compareTo(month2); 
		}
		
		//Year and month are the same, sort by country alphabetically
		return comparableTitle2.compareTo(comparableTitle1);
	}

	/**
	 * Retrieves the part of the name before the space-dash-space. 
	 * That part should be Country Month and Year.
	 * @param title
	 * @return The part of the title containing country, month and year
	 */
	private String getComparableTitle(String title) {
		int indexOfSeperator = title.indexOf(" - ");
		
		if(indexOfSeperator == -1) {
			return null;
		} else {
			return title.substring(0, title.indexOf(" - "));
		}
	}
	
	/**
	 * Retrieves the year from the title.
	 * 
	 * @param comparableTitle
	 * @return The year
	 */
	private String getYear(String comparableTitle) {
		return comparableTitle.substring(comparableTitle.lastIndexOf(" ") + 1, comparableTitle.length());
	}

	/**
	 * Retrieves the month from the title and maps it to an integer value.
	 * 
	 * @param comparableTitle
	 * @return The integer value of the month
	 */
	private Integer getMonth(String comparableTitle) {
		
		String titleWithoutYear = comparableTitle.substring(0, comparableTitle.lastIndexOf(" "));
		String months = comparableTitle.substring(titleWithoutYear.lastIndexOf(" ") + 1, titleWithoutYear.length()).toLowerCase();
		
		if(months.startsWith("january")) {
			return new Integer(1);
		} if(months.startsWith("february")) {
			return new Integer(2);
		} if(months.startsWith("march")) {
			return new Integer(3);
		} if(months.startsWith("april")) {
			return new Integer(4);
		} if(months.startsWith("may")) {
			return new Integer(5);
		} if(months.startsWith("june")) {
			return new Integer(6);
		} if(months.startsWith("july")) {
			return new Integer(7);
		} if(months.startsWith("august")) {
			return new Integer(8);
		} if(months.startsWith("september")) {
			return new Integer(9);
		} if(months.startsWith("october")) {
			return new Integer(10);
		} if(months.startsWith("november")) {
			return new Integer(11);
		} if(months.startsWith("december")) {
			return new Integer(12);
		} else {
			return new Integer(13);
		}		
	}

	public String getId() {
		return id;
	}

	public String getTitle() {
		return title;
	}
}
