package com.arellano.ramon.atacama.flickr;

import java.io.File;
import java.util.Arrays;
import java.util.Collection;
import java.util.Vector;

import org.jickr.Auth;
import org.jickr.Flickr;
import org.jickr.FlickrException;
import org.jickr.Permission;
import org.jickr.PhotoSet;
import org.jickr.User;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.entity.Album;
import com.arellano.ramon.atacama.entity.Photo;
import com.arellano.ramon.atacama.file.FileManager;
import com.arellano.ramon.atacama.gui.UserDialog;
import com.arellano.ramon.atacama.log.Logger;
import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * This class does all the communication with Flickr. Including authentication
 * and uploading of photos and sets.
 * 
 * @author Ramon Kristian
 * 
 */
public class FlickrUploader {
	// Logger and Config
	static private Config theConfig = null;

	static private Logger theLogger = null;

	static private FileManager theFileManager = null;

	private User flickrUser = null;

	/**
	 * @throws AtacamaException
	 */
	public void initFlickr() throws AtacamaException {
		theConfig = Config.getInstance();
		theLogger = Logger.getInstance();
		theFileManager = FileManager.getInstance();

		theLogger.logTrace("Starting Flickr authentication.");

		try {

			Flickr.setApiKey(theConfig.getString("flickr.apiKey"));
			Flickr.setSharedSecret(theConfig.getString("flickr.secret"));
			theLogger.logTrace("Looking for user with e-mail: " + theConfig.getString("flickr.email"));
			flickrUser = User.findByEmail(theConfig.getString("flickr.email"));
			theLogger.logTrace("Found user with profile: "
					+ flickrUser.getProfileURL());

			Permission perm = Permission.DELETE;
			Auth.setDefaultAuthUser(flickrUser);
			if (Auth.isAuthenticated(flickrUser, perm)) {
				Auth.setAuthContext(flickrUser);
			} else {
				
				String loginURL = Auth.getAuthURL(perm);
				theLogger
						.logTrace("If new machine, go to login URL in a browser: "
								+ loginURL);

				new UserDialog("It seems you need to authenticate Atacama with Flickr. When you press OK we'll send you to the URL below. Authorize it, then come back here." + loginURL);

				try {
					String[] args = { "rundll32", "url.dll,FileProtocolHandler", loginURL };
					Runtime.getRuntime().exec(args);
				} catch (Exception exception) {
					String message = "Failed to open default browser from the Atacama application. ";
					new UserDialog(message);
					theLogger.logError(message, exception);
				}

				new UserDialog("Let's continue");
				
				flickrUser = Auth.authenticate();
			}
			
			theLogger.logTrace("Logged in with permission level: "
					+ Auth.getPermLevel(flickrUser));
			Auth.isAuthenticated(flickrUser, perm);

		} catch (FlickrException exception) {
			theLogger.logError("Failed while getting Flickr context or APIS.",
					exception);
		}
	}

	/**
	 * Retrieves all photosets from Flickr.
	 * 
	 * @return A collection of photosets retrieved from Flickr.
	 * @throws AtacamaException
	 */
	public Collection getPhotosets() throws AtacamaException {
		Collection photosets = null;
		try {
			photosets = flickrUser.getPhotoSets();

		} catch (FlickrException exception) {
			theLogger.logError("Failed to get Flickr sets.", exception);
		}

		return photosets;
	}

	/**
	 * Upload a photoset/album to Flickr. Uploads each photo first, and then
	 * creates an album with the photo IDs returned from Flickr.
	 * 
	 * @param album
	 * @throws AtacamaException
	 */
	public void uploadPhotoset(Album album) throws AtacamaException {

		// Get album Photos
		Vector photos = album.getPhotos();

		// Find primary photo in album
		Photo primaryPhoto = album.getPrimaryPhoto();

		// Upload all photos in reverse order and create vector of the flick photos with ID
		String primaryPhotoId = null;
		String[] flickrPhotoIds = new String[photos.size()];
		for (int photoIndex = photos.size() - 1; photoIndex >= 0; photoIndex--) {
			Photo photo = (Photo) photos.elementAt(photoIndex);

			// Check if photo is JPG
			String photoId = null;
			if (theFileManager
					.isFileType(new File(photo.getPhotoName()), "JPG")) {
				// It's a JPG upload the file
				photoId = uploadPhoto(photo, album);
			} else {
				// Cannot upload non JPG files
				continue;
			}

			// Add photo ID for adding to album later
			flickrPhotoIds[photoIndex] = photoId;

			if (photo.equals(primaryPhoto)) {
				// This is the primary photo, get ID
				primaryPhotoId = photoId;
			}
		}

		if (flickrPhotoIds[0] != null) {
			// There is at least one JPG photo in the album
			// Create Flickr set
			PhotoSet photoSet = null;
			org.jickr.Photo primaryFlickrPhoto = new org.jickr.Photo(
					primaryPhotoId);
			try {
				photoSet = PhotoSet.create(createPhotosetTitle(album), album
						.getComment(), primaryFlickrPhoto);

				org.jickr.Photo[] flickrPhotos = new org.jickr.Photo[flickrPhotoIds.length];
				for (int index = 0; index < flickrPhotoIds.length; index++) {
					flickrPhotos[index] = new org.jickr.Photo(
							flickrPhotoIds[index]);
				}

				photoSet.setPhotos(primaryFlickrPhoto, Arrays
						.asList(flickrPhotos));
			} catch (FlickrException exception) {
				theLogger.logError("Failed to create Flickr set. "
						+ createPhotosetTitle(album), exception);
			}

			theLogger.logTrace("Created Flickr set: "
					+ createPhotosetTitle(album));
		}
	}

	/**
	 * Uses Atacama album name, country, year and month to create a Flickr
	 * photoset title.
	 * 
	 * @param album
	 * @return The title prefixed with country, month and year
	 */
	public String createPhotosetTitle(Album album) {
		String title = album.getCountry() + " " + album.getMonth() + " "
				+ album.getYear() + " - " + album.getAlbumName();

		return title;
	}

	/**
	 * Uploads one photo to Flickr. Uses the name and comment of the Atacama
	 * photo as title and description. And adds the year, month and country of
	 * the album as tags.
	 * 
	 * @param photo
	 * @param album
	 * @return The photo ID from Flickr
	 * @throws AtacamaException
	 */
	public String uploadPhoto(Photo photo, Album album) throws AtacamaException {

		File file = new File(photo.getPhotoFilePath());

		if (!file.exists()) {
			theLogger.logError("Failed to find file to upload: "
					+ photo.getPhotoFilePath());
		}
		
		String tags = album.getCountry() + "," + album.getMonth() + ",\""
				+ album.getAlbumName() + "\"";

		String photoId = null;
		try {
			boolean publicViewable = !(new Boolean(album.getPrivateOnly())); 
			
			// Set title, description, visibility and authentication, and upl
			org.jickr.Photo newFlickrPhoto = org.jickr.Photo
					.create(file, photo.getPhotoName(), photo.getComment(),
							tags, publicViewable, true, true);

			photoId = newFlickrPhoto.getID();
		} catch (FlickrException exception) {
			theLogger.logError("Failed to upload photo."
					+ photo.getPhotoFilePath(), exception);
		}

		theLogger.logTrace("Uploaded photo; Id: " + photoId + "; File: "
				+ photo.getPhotoFilePath());

		return photoId;
	}

	/**
	 * Performs sorting of photosets.
	 * 
	 * @param sortedPhotosetIds
	 * @throws AtacamaException
	 */
	public void orderSets(String[] sortedPhotosetIds) throws AtacamaException {
		try {
			PhotoSet[] sortedPhotoSets = new PhotoSet[sortedPhotosetIds.length];
			for (int index = 0; index < sortedPhotosetIds.length; index++) {
				sortedPhotoSets[index] = PhotoSet
						.findByID(sortedPhotosetIds[index]);
			}
			PhotoSet.orderSets(Arrays.asList(sortedPhotoSets));

		} catch (FlickrException exception) {
			theLogger.logError("Failed to sort photosets.");
		}

	}
}
