package com.arellano.ramon.atacama.flickr;

import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.jickr.FlickrException;
import org.jickr.PhotoSet;

import com.arellano.ramon.atacama.descriptor.DescriptorReader;
import com.arellano.ramon.atacama.entity.Album;
import com.arellano.ramon.atacama.log.Logger;
import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * This class synchronizes between Atacama Albums and Flickr Sets. If an Atacama
 * Album is not found in Flickr, the album and all its photos are uploaded.
 * 
 * @author Ramon Kristian
 * 
 */
public class Synchronizer {

	/**
	 * The logger instance
	 */
	static private Logger theLogger = null;

	/**
	 * The Flickr instance
	 */
	static private FlickrUploader flickrUploader = null;

	/**
	 * Constructor
	 */
	public Synchronizer() {
		theLogger = Logger.getInstance();
	}

	/**
	 * This is the main method which syncrhonizes. It can either check and log
	 * which albums are missing in Flickr, or perform the sync operation
	 * depending on the parameters sent to it. It uses the Flick class for
	 * communication with Flickr.
	 * 
	 * @param logOnly
	 *            If true will only log differences, if false will perform sync
	 * @throws AtacamaException
	 */
	public void sync(boolean logOnly) throws AtacamaException {

		theLogger.logTrace("Starting Flickr synchronization");

		// Get Atacama Albums
		DescriptorReader descriptorReader = new DescriptorReader();
		Vector albums = null;

		albums = descriptorReader.readGallery();

		// Initialize Flick API
		flickrUploader = new FlickrUploader();

		flickrUploader.initFlickr();

		// Get Flickr sets
		Collection photoSets = flickrUploader.getPhotosets();

		Vector newAlbums = compareAlbums(albums, photoSets);

		if (newAlbums.size() > 0) {
			// Log which albums will be uploaded
			theLogger.logTrace("The following albums do not exist in Flickr:");
			for (int albumIndex = 0; albumIndex < newAlbums.size(); albumIndex++) {
				Album newAlbum = (Album) newAlbums.elementAt(albumIndex);
				theLogger.logTrace("Album " + albumIndex + ": "
						+ newAlbum.getAlbumName());

			}

			if (!logOnly) {
				// Upload new albums
				for (int albumIndex = 0; albumIndex < newAlbums.size(); albumIndex++) {
					Album newAlbum = (Album) newAlbums.elementAt(albumIndex);

					theLogger.logTrace("Starting to upload album "
							+ newAlbum.getAlbumName());
					flickrUploader.uploadPhotoset(newAlbum);
				}

				// Order the Flickr sets according to year, month and country
				orderSets();
			}
		} else {
			// No new albums
			theLogger.logTrace("There are no new albums to upload to Flickr.");
		}
	}

	/**
	 * Compares a vector of Atacama albums to a Collection of Flickr Photosets
	 * and returns a Vector of Atacama albums that do not have an equivalent
	 * set. Compares the name, country, year and month of the albums/sets.
	 * 
	 * @param albums
	 *            Atacama album
	 * @param photosets
	 *            Flickr photosets
	 * @return A vector containing all albums that do not exist in Flickr
	 */
	public Vector compareAlbums(Vector albums, Collection photosets) {

		Vector newAlbums = new Vector();

		// Iterate through all Atacama albums and find which are not in Flickr
		for (int albumIndex = 0; albumIndex < albums.size(); albumIndex++) {

			PhotoSet photoset = null;
			Iterator iterator = photosets.iterator();
			boolean foundAlbum = false;
			while (iterator.hasNext()) {
				photoset = (PhotoSet) iterator.next();

				if (isSameAlbum((Album) albums.elementAt(albumIndex), photoset)) {
					// Atacama album was found in Flickr, stop looking
					foundAlbum = true;
					break;
				}
			}

			if (!foundAlbum) {
				// Album was not found, it needs to be added
				newAlbums.add(albums.elementAt(albumIndex));
			}
		}

		return newAlbums;
	}

	/**
	 * Checks if an album is the same as a photoset by comparing name, country,
	 * month and year.
	 * 
	 * @param album
	 * @param photoset
	 * @return true if it's thye same album
	 */
	public boolean isSameAlbum(Album album, PhotoSet photoset) {
		if (album == null || photoset == null) {
			return false;
		} else {
			try {
				return flickrUploader.createPhotosetTitle(album)
						.equals(photoset.getTitle());
			} catch (FlickrException exception) {
				theLogger.logError("Failed to get PhotoSet title sets.", exception);
			}
		
		}
		
		return false;
	}

	/**
	 * Sorts photosets according to year, month and country respectively.
	 * 
	 * @throws AtacamaException
	 */
	private void orderSets() throws AtacamaException {
		Collection photosets = flickrUploader.getPhotosets();

		// Create list of comparable photosets
		Iterator iterator = photosets.iterator();
		List comparableSets = new Vector();
		while (iterator.hasNext()) {
			comparableSets.add(new ComparableSet((PhotoSet) iterator.next()));
		}

		// Sort the sets by year, month and country
		Collections.sort(comparableSets);

		// Enter the IDs of the sorted sets into an array
		String sortedPhotosetIds[] = new String[photosets.size()];
		iterator = comparableSets.iterator();
		int index = 0;
		theLogger.logTrace("Sets in sorted order:");
		while (iterator.hasNext()) {
			ComparableSet comparableSet = (ComparableSet) iterator.next();
			sortedPhotosetIds[index++] = comparableSet.getId();
			theLogger.logTrace(comparableSet.getTitle());
		}

		// Sort the sets in Flickr
		flickrUploader.orderSets(sortedPhotosetIds);
	}
}
