/*
 * Created on 06.jun.04
 *
 */
package com.arellano.ramon.atacama.gui;

import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.Rectangle;
import java.awt.TextArea;
import java.awt.Toolkit;
import java.awt.Window;
import java.io.File;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Vector;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.descriptor.DescriptorGenerator;
import com.arellano.ramon.atacama.descriptor.DescriptorReader;
import com.arellano.ramon.atacama.entity.Album;
import com.arellano.ramon.atacama.file.FileManager;
import com.arellano.ramon.atacama.flickr.Synchronizer;
import com.arellano.ramon.atacama.html.HtmlGenerator;
import com.arellano.ramon.atacama.log.Logger;
import com.arellano.ramon.atacama.photo.PhotoGenerator;
import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * @author ramon.arellano
 * 
 * This class is the controller for the Atacama GUI. It provides several
 * utilities and helps the communication between panels. It also handles most of
 * the file activity needed by panels.
 */
public class Controller {

	// The unique instance of the main Atacama frame
	private MainFrame mainFrame = null;

	// The single instance of the controller
	private static Controller theController = null;

	// Controller, Config, Logger and FileManager
	private static Config theConfig = null;

	private static Logger theLogger = null;

	private static FileManager theFileManager = null;

	/**
	 * The constructor
	 */
	private Controller() {
		// Get the config and file manager
		theConfig = Config.getInstance();
		theFileManager = FileManager.getInstance();
		theLogger = Logger.getInstance();
	}

	/**
	 * Returns the unique instance of this class
	 * 
	 * @return The unique instance of this class.
	 */
	static public Controller getInstance() {
		if (null == theController) {
			theController = new Controller();
		}
		return theController;
	}

	/**
	 * Repaints the main frame to update changes
	 */
	public void repaintMainFrame() {
		mainFrame.show();
	}

	/**
	 * Returns the instance of main frame
	 * 
	 * @return The main Atacama frame
	 */
	public MainFrame getMainFrame() {
		return mainFrame;
	}

	/**
	 * Used by the main frame to set reference to itself
	 * 
	 * @param mainFrame
	 *            The main Atacama frame
	 */
	public void setMainFrame(MainFrame mainFrame) {
		this.mainFrame = mainFrame;
	}

	/**
	 * Returns the single instance of the log text area component
	 * 
	 * @return The log text area component
	 */
	public TextArea getLogArea() {
		return mainFrame.getLogArea();
	}

	/**
	 * Opens a frame for selecting a directory
	 * 
	 * @param message
	 *            The message to be displayed in the dialog
	 * @param startDirectory
	 *            The starting directory for the dialog
	 * @return The chosen directory
	 */
	public File selectDirectory(String message, File startDirectory) {
		File directory = SelectDirectoryDialog.show(mainFrame, message,
				"Select directory", startDirectory.getParentFile());

		return directory;
	}

	/**
	 * Reads a photo descriptor and returns the resulting vector of photos
	 * 
	 * @param descriptorPath
	 *            The path to the descriptor to be read
	 * @return The vector containg all read photos
	 * @throws AtacamaException
	 *             Throws if reading fails
	 */
	public Vector readPhotoDescriptor(String descriptorPath)
			throws AtacamaException {
		// Get the photos for the specified descriptor
		DescriptorReader descReader = new DescriptorReader();

		Vector photos = descReader.readPhotoDescriptor(descriptorPath, true);

		return photos;
	}

	public boolean comparePhotoVectors(Vector oldPhotos, Vector newPhotos)
			throws AtacamaException {
		// Get the photos for the specified descriptor

		DescriptorReader descReader = new DescriptorReader();

		return descReader.comparePhotoVectors(oldPhotos, newPhotos, true);
	}

	/**
	 * Returns the file paths of all descriptors in the photos directory
	 * 
	 * @return The array of string with all descriptor paths
	 */
	public String[] getDescriptorPaths() {
		return theFileManager.findFiles(new File(theConfig
				.getString("common.dir.source.photos")), theConfig
				.getString("descriptor.file.text.photo"), true);
	}

	/**
	 * Writes the photo descriptor to file
	 * 
	 * @param photos
	 *            The vector of photos to write
	 * @param path
	 *            The path to the string to write to
	 */
	public void writePhotoDescriptor(Vector photos, String path) {
		// Save the descriptor for the photos
		DescriptorReader descReader = new DescriptorReader();

		theLogger.logTrace("Writing photo descriptor for directory: " + path);

		try {
			descReader.writePhotoDescriptor(photos, path, theConfig
					.getString("descriptor.file.text.photo"));
		} catch (AtacamaException exception) {
			new UserDialog(
					"Failed to write the photo descriptor to file. See log for details.");
		}

		setStatus("Photo descriptor written to file.");
	}

	/**
	 * Reads a descriptor and returns the resulting vector of photos
	 * 
	 * @param descriptorPath
	 *            The path to the descriptor to read
	 * @param warnIfNotFound
	 *            Set true if a dialog should popup if file is not found
	 * @return The vector of albums read from file
	 */
	public Vector readAlbumDescriptor(String descriptorPath,
			boolean warnIfNotFound) {
		// Get the photos for the specified descriptor
		DescriptorReader descReader = new DescriptorReader();

		Vector albums = new Vector();
		try {
			albums = descReader.readAlbumDescriptor(descriptorPath);
		} catch (AtacamaException exception) {
			if (warnIfNotFound) {
				new UserDialog(
						"Could not read the album descriptor file. "
								+ "Verify that it exists and is valid. See log for details.");
			}
		}

		return albums;
	}

	/**
	 * Writes the album descriptor to file
	 * 
	 * @param albums
	 *            The vector of albums to write
	 * @param filePath
	 *            The path of the file to write
	 */
	public void writeAlbumDescriptor(Vector albums, String filePath) {
		// Save the descriptor for the photos
		DescriptorReader descReader = new DescriptorReader();

		theLogger.logTrace("Writing album descriptor : " + filePath);

		// Write the descriptor to file
		try {
			descReader.writeAlbumDescriptor(albums, new File(filePath)
					.getParentFile().getPath(), new File(filePath).getName());
		} catch (AtacamaException exception) {
			new UserDialog(
					"Failed to write the album descriptor to file. See log for details.");
		}

		// Set status
		theController.setStatus("Album descriptor written to file.");
	}

	/**
	 * Creates a canvas with the thumbnail image of a photo. Returns a standard
	 * image if photo if photo is not JPG.
	 * 
	 * @param photoFilePath
	 *            The path to the image file to create canvas for
	 * @return The thumbnail canvas containing the image
	 */
	public ThumbnailCanvas getThumbnailCanvas(String photoFilePath) {
		ThumbnailCanvas canvas = null;

		// Get image canvas
		if (theFileManager.isFileType(new File(photoFilePath), "JPG")) {
			String thumbFilePath = photoFilePath;

			// Get canvas with resized JPG file
			canvas = new ThumbnailCanvas(thumbFilePath, true);
		} else {
			String thumbFilePath = theConfig.getApplicationDir()
					+ theConfig.getString("html.resource.dir.source")
					+ theConfig.getString("html.resource.image.photo.thumb");

			// Get canvas with non-resized image
			canvas = new ThumbnailCanvas(thumbFilePath, false);
		}

		return canvas;
	}

	/**
	 * Opens a frame for previewing photos
	 * 
	 * @param filePath
	 *            Path to the image file to be previewed
	 */
	public void viewPhoto(String filePath) {
		if (!theFileManager.isFileType(new File(filePath), "JPG")) {
			new UserDialog("Sorry, can only preview JPG images.");
			return;
		}

		if (!new File(filePath).exists()) {
			new UserDialog("The file doesn't exist: " + filePath);
		} else {
			new PhotoPreviewFrame(filePath);
		}
	}

	/**
	 * Let's the user select a file
	 * 
	 * @param title
	 *            The title of the dialog box
	 * @param file
	 *            The starting file to be shown
	 * @return The file the user selected
	 */
	public File selectFile(String title, File file, boolean save) {

		int mode = FileDialog.LOAD;
		if (save) {
			mode = FileDialog.SAVE;
		}

		FileDialog dialog = new FileDialog(mainFrame, title, mode);

		dialog.setDirectory(file.getParentFile().getPath());
		dialog.setFile(file.getName());

		dialog.show();

		if (dialog.getDirectory() == null || dialog.getFile() == null) {
			return null;
		}

		return new File(dialog.getDirectory(), dialog.getFile());
	}

	/**
	 * Saves the properties to file keys sorted alfabetically
	 */
	public void saveProperties() {
		theLogger.logTrace("Saving properties to file: "
				+ theConfig.getPropertiesFilePath());

		try {
			theConfig.storePropertiesFile();
		} catch (AtacamaException exception) {
			// Failed to store properties
			new UserDialog(exception.getMessage());

			return;
		}

		// Sort the string to make it look decent.
		File propertiesFile = new File(theConfig.getPropertiesFilePath());

		String[] propertiesString = null;
		try {
			propertiesString = theFileManager.readTextFile(propertiesFile);
		} catch (AtacamaException exception) {
			new UserDialog("Settings were not writte. "
					+ exception.getMessage());
			return;
		}

		Arrays.sort(propertiesString, String.CASE_INSENSITIVE_ORDER);

		try {
			theFileManager
					.writeTextFile(propertiesFile, propertiesString, true);
		} catch (AtacamaException exception) {
			new UserDialog(
					"Failed to save the properties to file. See log for details.");
		}

		setStatus("Properties saved to file.");
	}

	/**
	 * Generates or validates photo descriptors depending on the validate flag
	 * 
	 * @param validate
	 *            True if no files should be written, only validated
	 */
	public void generateDescriptors(boolean validate) {
		// Empty the log
		mainFrame.clearLog();

		// Generate or validate
		int numDescriptorsOutOfDate = 0;
		try {
			DescriptorGenerator descGenerator = new DescriptorGenerator();
			numDescriptorsOutOfDate = descGenerator.generate(validate);
		} catch (AtacamaException exception) {
			String mode = null;
			if (validate) {
				mode = "validate";
			} else {
				mode = "generate";
			}

			new UserDialog("Could not " + mode + " descriptors."
					+ exception.getMessage());

			return;
		}

		if (validate) {
			new UserDialog(
					"There are "
							+ numDescriptorsOutOfDate
							+ " descriptors that "
							+ "need to be created or updated in your photo directory. Please see "
							+ "log for details.");
		}
	}

	/**
	 * Generate gallery
	 * 
	 * @param generateHtml
	 *            True if HTML should be written
	 * @param generatePhoto
	 *            True if photos should be copied and resized
	 */
	public void generateGallery(boolean generateHtml, boolean generatePhoto) {
		// Empty the log
		mainFrame.clearLog();

		// Read the gallery
		Vector gallery = null;
		try {
			DescriptorReader descReader = new DescriptorReader();
			gallery = descReader.readGallery();
		} catch (AtacamaException exception) {
			new UserDialog("Generation failed. " + exception.getMessage());
			return;
		}
	
		// Generate HTML
		if (generateHtml) {
			try {
				new HtmlGenerator().generate(gallery);
			} catch (AtacamaException exception) {
				new UserDialog("Failed to generate HTML. "
						+ exception.getMessage());
				return;
			}
		}

		// Generate PHOTOS
		if (generatePhoto) {
			try {
				new PhotoGenerator().generate(gallery);
			} catch (AtacamaException exception) {
				new UserDialog("Failed to generate photos. "
						+ exception.getMessage());
				return;
			}
		}
	}

	/**
	 * Returns a string of the specified lenght. It is the original string that
	 * is either truncated or filled with spaces at the end.
	 * 
	 * @param originalString
	 *            The original string
	 * @param length
	 *            The length of the string to be returned
	 * @return The string with desired length
	 */
	public String getSizedString(String originalString, int length) {
		String newString = null;

		if (originalString.length() >= length) {
			newString = originalString.substring(0, length) + "...";
		} else {
			char[] spaceChars = new char[length - originalString.length()];
			Arrays.fill(spaceChars, ' ');
			newString = originalString + new String(spaceChars);
		}

		return newString;
	}

	/**
	 * Sends a message to the status bar
	 * 
	 * @param statusString
	 *            The status message
	 */
	public void setStatus(String statusString) {
		mainFrame.setStatus(getSizedString(statusString, 100));
		repaintMainFrame();
	}

	/**
	 * Opens the javadoc in the default browser
	 */
	public void openJavaDocInDefaultBrowser() {
		try {
			String Url = new File(theConfig.getApplicationDir()
					+ theConfig.getString("gui.file.javadoc"))
					.getCanonicalPath();

			openUrlInDefaultBrowser(Url);
		} catch (Exception exception) {
			String message = "Failed to open JavaDOC in your browser from the Atacama application. ";
			new UserDialog(message);
			theLogger.logError(message, exception);
		}
	}

	/**
	 * Opens the gallery file in the default browser
	 */
	public void openGalleryInDefaultBrowser() {
		try {
			String Url = new File(theConfig
					.getString("common.dir.target.gallery")
					+ theConfig.getString("html.album.index"))
					.getCanonicalPath();

			openUrlInDefaultBrowser(Url);
		} catch (Exception exception) {
			String message = "Failed to open gallery in your browser from the Atacama application. ";
			new UserDialog(message);
			theLogger.logError(message, exception);
		}
	}

	/**
	 * Compares albums to existing Flickr photoset and logs the result
	 */
	public void compareFlickr() {
		try {
			new Synchronizer().sync(true);
		} catch (Exception exception) {
			String message = "Failed to compare Atacama albums to Flickr photosets. See log for details. ";
			new UserDialog(message);
			theLogger.logError(message, exception);
		}
	}

	/**
	 * Compares albums to existing Flickr photoset and uploads albums that do
	 * not exist in Flickr.
	 */
	public void syncFlickr() {
		try {
			new Synchronizer().sync(false);
		} catch (Exception exception) {
			String message = "Failed to upload Atacama albums to Flickr photosets. See log for details. ";
			new UserDialog(message);
			theLogger.logError(message, exception);
		}
	}

	/**
	 * Opens any URL in the default browser
	 * 
	 * @param Url
	 *            The URL
	 */
	public void openUrlInDefaultBrowser(String Url) {
		try {
			String[] args = { "rundll32", "url.dll,FileProtocolHandler", Url };
			Runtime.getRuntime().exec(args);
		} catch (Exception exception) {
			String message = "Failed to open default browser from the Atacama application. ";
			new UserDialog(message);
			theLogger.logError(message, exception);
		}

	}

	/**
	 * Set the bounds of a given frame such that its placed centered in the
	 * user's screen.
	 * 
	 * @param frame
	 *            The frame
	 * @param frameSize
	 *            The size of the frame
	 */
	public void locateCentered(Window frame, Dimension frameSize) {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

		Rectangle frameBounds = new Rectangle(screenSize.width / 2
				- frameSize.width / 2, screenSize.height / 2 - frameSize.height
				/ 2, frameSize.width, frameSize.height);

		frame.setBounds(frameBounds);
	}

	/**
	 * Maxmimizes the screen to window
	 * 
	 * @param frame
	 *            The frame
	 */
	public void locateMaximized(Window frame) {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

		Rectangle frameBounds = new Rectangle(0, 0, screenSize.width,
				screenSize.height - 60);

		frame.setBounds(frameBounds);
	}
}
