/*
 * Created on 05.jun.04
 *
 */
package com.arellano.ramon.atacama.gui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Vector;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.entity.Album;

/**
 * @author ramon.arellano
 *
 * This is the main panel for editing album descriptors, it uses the AlbumScrollPane
 * to hold the information of the albums themselves.
 */
public class EditAlbumPanel extends Panel implements ActionListener {

	//Controller, Config, Logger and FileManager	
	private static Controller theController = null;
	private static Config theConfig = null;

	//Components
	private Label albumFileLabel = null;
	private AlbumScrollPane albumScrollPane = null;

	/**
	 * The Constructor
	 */
	public EditAlbumPanel() {
		//Get the controller, config, logger and file manager
		theController = Controller.getInstance();
		theConfig = Config.getInstance();
	}

	/**
	 * Initialises the edit album descriptor panel with all components
	 */
	public void init() {
		//Set layout for edit album descriptor panel		 
		setLayout(new BorderLayout());

		//Add options panel to edit descriptors panel
		add(initOptionsPanel(), "North");

		//Add scroll pane with all albums from descriptor
		add(initAlbumScrollPane(), "Center");

		//Add buttons panel
		add(initButtonsPanel(), "South");
	}

	/**
	 * Initializes and return the options panel
	 * 
	 * @return The initialized options panel
	 */
	private Panel initOptionsPanel() {
		//Create album directory label for album directory selection panel
		albumFileLabel =
			new Label(theConfig.getString("descriptor.file.text.album"));

		Panel optionsPanel =
			new FileSelectionPanel(
				"Album file: ",
				albumFileLabel,
				Actions.EAP_BUTTON_CHANGE_ALBUM_FILE,
				this);

		return optionsPanel;
	}

	/**
	 * Initializes and return the buttons panel
	 * 
	 * @return The buttons panel
	 */
	private Panel initButtonsPanel() {
		Panel buttonsPanel = new Panel();
		buttonsPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));

		Button addFromPhotosButton =
			new Button(Actions.EAP_BUTTON_ADD_FROM_PHOTOS);
		addFromPhotosButton.setFont(new Font("Arial", Font.BOLD, 12));
		addFromPhotosButton.addActionListener(this);

		Button addNewAlbumButton = new Button(Actions.EAP_BUTTON_ADD_NEW_ALBUM);
		addNewAlbumButton.setFont(new Font("Arial", Font.BOLD, 12));
		addNewAlbumButton.addActionListener(this);

		Button saveDescriptorAsButton =
			new Button(Actions.EAP_BUTTON_SAVE_ALBUM_DESCRIPTOR_AS);
		saveDescriptorAsButton.setFont(new Font("Arial", Font.BOLD, 12));
		saveDescriptorAsButton.addActionListener(this);

		Button saveDescriptorButton =
			new Button(Actions.EAP_BUTTON_SAVE_ALBUM_DESCRIPTOR);
		saveDescriptorButton.setFont(new Font("Arial", Font.BOLD, 12));
		saveDescriptorButton.addActionListener(this);

		buttonsPanel.add(addFromPhotosButton);
		buttonsPanel.add(addNewAlbumButton);
		buttonsPanel.add(saveDescriptorAsButton);
		buttonsPanel.add(saveDescriptorButton);

		return buttonsPanel;
	}

	/**
	 *  Updates info after an action has been performed
	 * @return The initialized album scroll pane
	 */
	public AlbumScrollPane initAlbumScrollPane() {
		if (albumScrollPane == null) {
			albumScrollPane = new AlbumScrollPane();
		}

		albumScrollPane.init(theConfig.getString("descriptor.file.text.album"));

		return albumScrollPane;
	}

	/**
	 * Updates the properties, file label and scroll pane with new album file
	 * 
	 * @param newAlbumFile
	 */
	private void update(File newAlbumFile) {
		//Set properties (update model)
		theConfig.setProperty(
			"descriptor.file.text.album",
			newAlbumFile.getPath());

		//Update view
		albumFileLabel.setText(theController.getSizedString(newAlbumFile.getPath(), 63));

		//Read the new file and update scroll panel
		remove(albumScrollPane);
		add(initAlbumScrollPane(), "Center");
		theController.repaintMainFrame();
	}

	/**
	 * Creates an album vector with the information containted in the 
	 * scroll pane.
	 * 
	 * @return The vector of albums retrieved from the scroll pane
	 */
	private Vector getAlbumsFromScrollPane() {
		//Create new albums vector with info from components
		Vector albums = new Vector();

		for (int index = 0;
			index < albumScrollPane.albumNameFields.size();
			index++) {
			//Get info from component vectors
			TextField albumNameField =
				(TextField) albumScrollPane.albumNameFields.elementAt(index);
			Checkbox publishBox =
				(Checkbox) albumScrollPane.publishBoxes.elementAt(index);
			Checkbox privateOnlyBox =
				(Checkbox) albumScrollPane.privateOnlyBoxes.elementAt(index);
			TextField yearField =
				(TextField) albumScrollPane.yearFields.elementAt(index);
			TextField monthField =
				(TextField) albumScrollPane.monthFields.elementAt(index);
			TextField countryField =
				(TextField) albumScrollPane.countryFields.elementAt(index);
			TextField commentField =
				(TextField) albumScrollPane.commentFields.elementAt(index);

			Album album = new Album();

			//Add info to album
			album.setAlbumName(albumNameField.getText());
			album.setPublish(new Boolean(publishBox.getState()).toString());
			album.setPrivateOnly(new Boolean(privateOnlyBox.getState()).toString());
			album.setYear(yearField.getText());
			album.setMonth(monthField.getText());
			album.setCountry(countryField.getText());
			album.setComment(commentField.getText());

			albums.add(album);
		}

		return albums;
	}

	/**
	 * Receives events from the buttons in the panel
	 * @param event The action event to process
	 */
	public void actionPerformed(ActionEvent event) {
		String action = event.getActionCommand();

		if (action.equals(Actions.EAP_BUTTON_CHANGE_ALBUM_FILE)) {
			//The change file button was pushed
			//Open window to select new file
			File newAlbumFile =
				theController.selectFile(
					"Select album file",
					new File(theConfig.getString("descriptor.file.text.album")),
					false);

			//If file was changed we need to update descriptor scroll pane
			if (newAlbumFile != null) {
				theConfig.setProperty(
					"descriptor.file.text.album",
					newAlbumFile.getPath());
				update(newAlbumFile);
			} else {
				//No file selected
				new UserDialog("No album descriptor file was selected.");
			}
		} else if (action.equals(Actions.EAP_BUTTON_SAVE_ALBUM_DESCRIPTOR)) {
			//The save descriptor button was pushed
			Vector albums = getAlbumsFromScrollPane();

			//Write the descriptor to active descriptor file
			theController.writeAlbumDescriptor(
				albums,
				theConfig.getString("descriptor.file.text.album"));

			update(new File(theConfig.getString("descriptor.file.text.album")));
		} else if (
			action.equals(Actions.EAP_BUTTON_SAVE_ALBUM_DESCRIPTOR_AS)) {
			//The save descriptor button was pushed
			Vector albums = getAlbumsFromScrollPane();

			//Select file to save to
			File newAlbumFile =
				theController.selectFile(
					"Save album descriptor",
					new File(theConfig.getString("descriptor.file.text.album")),
					true);

			//Write the descriptor to file
			theController.writeAlbumDescriptor(albums, newAlbumFile.getPath());

			update(newAlbumFile);
		} else if (action.equals(Actions.EAP_BUTTON_ADD_FROM_PHOTOS)) {
			//The add new albums from published photos button was pressed
			//Add empty row at the top
			albumScrollPane.addAlbumsFromPhotos();

			//Scroll pane to top
			albumScrollPane.setScrollPosition(0, 0);

			//Repaint
			albumScrollPane.repaint();
		} else if (action.equals(Actions.EAP_BUTTON_ADD_NEW_ALBUM)) {
			//The add new album button was pressed
			//Add empty row at the top
			albumScrollPane.addAlbumRowInfo(null, 0);

			//Scroll pane to top
			albumScrollPane.setScrollPosition(0, 0);

			//Repaint
			albumScrollPane.repaint();
		}
	}
}