/*
 * Created on 05.jun.04
 *
 */
package com.arellano.ramon.atacama.gui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Choice;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.Vector;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.entity.Photo;
import com.arellano.ramon.atacama.log.Logger;
import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * @author ramon.arellano
 *
 * The main panel for editing photo descriptors. Uses PhotoScrollPane to manage
 * the photo information.
 */
public class EditPhotoPanel
	extends Panel
	implements ActionListener, ItemListener {

	//Controller, Config, Logger and FileManager	
	private static Controller theController = null;
	private static Logger theLogger = null;
	private static Config theConfig = null;

	//Components
	private Label photoDirLabel = null;
	private Choice descriptorChoice = null;
	private PhotoScrollPane photoScrollPane = null;
	private String currentDescPath = null;
	private Panel messagePanel = null;

	/**
	 * The Constructor
	 */
	public EditPhotoPanel() {
		//Get the controller, config, logger and file manager
		theController = Controller.getInstance();
		theLogger = Logger.getInstance();
		theConfig = Config.getInstance();
	}

	/**
	 * Initialises the edit photo descriptor panel with all components
	 */
	public void init() {

		//Emtpy existing content
		this.removeAll();

		//Set layout for edit photo descriptor panel		 
		setLayout(new BorderLayout());

		//Add options panel to edit descriptors panel
		add(initOptionsPanel(), "North");

		//Add buttons panel
		add(initButtonsPanel(), "South");

		theController.repaintMainFrame();

		//Add scroll pane with all photos from descriptor
		add(initPhotoScrollPane(), "Center");
	}

	/**
	 * Initializes and return the options panel
	 * 
	 * @return The options panel
	 */
	private Panel initOptionsPanel() {
		//Create components for photo directory selection panel

		//Create photo directory label for photo directory selection panel
		photoDirLabel =
			new Label(theConfig.getString("common.dir.source.photos"));

		Panel photoDirPanel =
			new FileSelectionPanel(
				"Photos directory: ",
				photoDirLabel,
				Actions.COMMON_BUTTON_CHANGE_PHOTO_DIR,
				this);

		//Create select label for options panel
		Label selectDescLabel =
			new Label("   Select the photo descriptor to edit: ");
		selectDescLabel.setFont(new Font("Arial", Font.BOLD, 12));

		//Create dropdown box and populate with all photo descriptors for options panel
		descriptorChoice = new Choice();
		descriptorChoice.addItemListener(this);
		addDescriptorsToChoice();
		currentDescPath = descriptorChoice.getSelectedItem();

		//Create dropdown box panel
		Panel descriptorChoicePanel =
			new Panel(new FlowLayout(FlowLayout.LEFT));
		descriptorChoicePanel.add(descriptorChoice);

		//Create new options panel and  add all panels and components
		Panel optionsPanel = new Panel();
		optionsPanel.setLayout(new BorderLayout());
		optionsPanel.add(photoDirPanel, "North");
		optionsPanel.add(selectDescLabel, "Center");
		optionsPanel.add(descriptorChoicePanel, "South");

		return optionsPanel;
	}

	/**
	 * Initializes and return the buttons panel
	 * 
	 * @return The buttons panel
	 */
	private Panel initButtonsPanel() {
		Panel buttonsPanel = new Panel();
		buttonsPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));

		Button validateDescriptorsButton = new Button(Actions.EDP_BUTTON_VALIDATE_DESCRIPTORS);
		validateDescriptorsButton.addActionListener(this);
		validateDescriptorsButton.setFont(new Font("Arial", Font.BOLD, 12));

		Button generateDescriptorsButton = new Button(Actions.EPD_BUTTON_GENERATE_DESCRIPTORS);
		generateDescriptorsButton.addActionListener(this);
		generateDescriptorsButton.setFont(new Font("Arial", Font.BOLD, 12));

		Button refreshButton = new Button(Actions.EDP_BUTTON_REFRESH);
		refreshButton.setFont(new Font("Arial", Font.BOLD, 12));
		refreshButton.addActionListener(this);

		Button saveDescriptorButton =
			new Button(Actions.EDP_BUTTON_SAVE_PHOTO_DESCRIPTOR);
		saveDescriptorButton.setFont(new Font("Arial", Font.BOLD, 12));
		saveDescriptorButton.addActionListener(this);

		buttonsPanel.add(validateDescriptorsButton);
		buttonsPanel.add(generateDescriptorsButton);
		buttonsPanel.add(refreshButton);
		buttonsPanel.add(saveDescriptorButton);

		return buttonsPanel;
	}
	/**
	 * Searches for all photo  descriptors in the photos directory and
	 * adds them to the drop-down list.
	 */
	private void addDescriptorsToChoice() {
		//Get the descriptors
		String[] descriptors = theController.getDescriptorPaths();

		if (descriptors.length == 0) {
			//No descriptors found
			new UserDialog(
				"There are no photo descriptors in the selected photo directory or its subdirectories. "
					+ "Either generate descriptors for these directories or change to another directory.");

			//Return false, meaning there has been no change
			return;
		}

		//Empty the list first
		descriptorChoice.removeAll();

		for (int index = 0; index < descriptors.length; index++) {
			descriptorChoice.add(
				new File(descriptors[index]).getParentFile().getPath());
		}

		return;
	}

	/**
	 *  Updates info after an action has been performed
	 *  @return The initialized photo scroll pane
	 */
	public PhotoScrollPane initPhotoScrollPane() {
		if (photoScrollPane == null) {
			photoScrollPane = new PhotoScrollPane();
		}

		//Reinitialize the scroll pane containing the descriptor info
		remove(photoScrollPane);

		//Set a temporary panel to show a loading message
		if(messagePanel != null)
			remove(messagePanel);
			
		messagePanel = new Panel(new GridBagLayout());
		Label loadingLabel = new Label("Loading photos...");
		messagePanel.add(loadingLabel);
		add(messagePanel, "Center");
		theController.repaintMainFrame();

		//This method will load images and takes time
		photoScrollPane.init(descriptorChoice.getSelectedItem());

		//Remove the temporary panel
		remove(messagePanel);

		return photoScrollPane;
	}

	/**
	 * Saves the descriptor with the current values
	 */
	private void saveDescriptor(boolean confirmBeforeSave) {
		//The save descriptor button was pushed
		//Update the photos vector with info from components
		for (int index = 0;
			index < photoScrollPane.photos.size();
			index++) {
			Photo photo = (Photo) photoScrollPane.photos.elementAt(index);
			Checkbox publishBox =
				(Checkbox) photoScrollPane.publishBoxes.elementAt(index);
			Checkbox primaryBox =
				(Checkbox) photoScrollPane.primaryBoxes.elementAt(index);
			TextField albumField =
				(TextField) photoScrollPane.albumNameFields.elementAt(
					index);
			TextField commentField =
				(TextField) photoScrollPane.commentFields.elementAt(index);

			photo.setPublish(new Boolean(publishBox.getState()).toString());
			photo.setPrimary(new Boolean(primaryBox.getState()).toString());
			photo.setAlbumName(albumField.getText());
			photo.setComment(commentField.getText());
		}

		//Check for changes in photo descriptor
		try {
			if(currentDescPath == null) {
				//No current descriptor, don't save
				return;
			}

			Vector oldPhotos = theController.readPhotoDescriptor(currentDescPath);
			boolean identical = theController.comparePhotoVectors(oldPhotos, photoScrollPane.photos);

			if(identical) {
				//Descriptors are identical, no need to save
				return;
			}

			if(confirmBeforeSave) {
				//Show user confirm dialog
				UserConfirmDialog userConfirmDialog = new UserConfirmDialog("The current photo descriptor has changed.\n\nDo you wish to save it?");

				if(!userConfirmDialog.result) {
					//User does not want to save
					return;
				}
			}

			//Write the descriptor to file
			theController.writePhotoDescriptor(
				photoScrollPane.photos,
				currentDescPath);		
		} catch (AtacamaException exception) {
			theLogger.logWarning("Failed checking for changes in photo descriptor. Did not save descriptor. " + exception.getMessage());
		}
		
	}

	/**
	 * Repaints the panel
	 * @see java.awt.Component#repaint()
	 */
	public void repaint() {
		//Do everything necessary to consider changes done since last
		//time the panel was painted
		addDescriptorsToChoice();
		
		if(currentDescPath != null) {
			descriptorChoice.select(currentDescPath);
		} else {
			currentDescPath = descriptorChoice.getSelectedItem();
		}
		
		add(initPhotoScrollPane(), "Center");
		theController.repaintMainFrame();
	}

	/**
	 * Receives events from the buttons in the panel
	 * @param event The action event
	 */
	public void actionPerformed(ActionEvent event) {
		String action = event.getActionCommand();

		if (action.equals(Actions.COMMON_BUTTON_CHANGE_PHOTO_DIR)) {
			//The change directory button was pushed
			//Open window to select new directory
			File newPhotoDirectory =
				theController.selectDirectory(
					"Select root photo directory",
					new File(theConfig.getString("common.dir.source.photos")));

			//If directory was changed we need to update descriptor choice
			//and descriptor scroll pane
			if (newPhotoDirectory != null) {
				//Setting property
				theConfig.setProperty(
					"common.dir.source.photos",
					newPhotoDirectory.getPath());

				//Update directory label
				photoDirLabel.setText(theController.getSizedString(newPhotoDirectory.getPath(), 63));
				
				//Remove the current (outdated scrollpane), and put up temporary message
				this.remove(photoScrollPane);
				messagePanel = new Panel(new GridBagLayout());
				Label selectLabel = new Label("Select descriptor to edit...");
				messagePanel.add(selectLabel);
				add(messagePanel, "Center");
				theController.repaintMainFrame();
								
				//Update descriptor choice
				addDescriptorsToChoice();

				//Making current descriptor path not valid
				currentDescPath = null;				
			}
		} else if(action.equals(Actions.EDP_BUTTON_VALIDATE_DESCRIPTORS)) {
			theController.setStatus("Validating photo descriptors...");
			//The validate descriptors button has been pushed
			//Validate descriptors
			theController.generateDescriptors(true);
				
			theController.setStatus("Finished validating photo descriptors");
		} else if(action.equals(Actions.EPD_BUTTON_GENERATE_DESCRIPTORS)) {
			theController.setStatus("Generating photo descriptors...");
			//The generate descriptors button has been pushed
			//Generate descriptors
			theController.generateDescriptors(false);			
			repaint();
			theController.setStatus("Finished generating photo descriptors");
		} else if (action.equals(Actions.EDP_BUTTON_REFRESH)) {
			//Refresh button was pushed
			repaint();
		} else if (action.equals(Actions.EDP_BUTTON_SAVE_PHOTO_DESCRIPTOR)) {
			//Save without confirm
			saveDescriptor(false);
		}
	}

	/** 
	 * Receives events from the descriptor choice
	 * and updates the descriptor scroll pane
	 * @param event The item event
	 */
	public void itemStateChanged(ItemEvent event) {
		//Save descriptor
		saveDescriptor(true);

		//Previous descriptor saved, now change current descriptor
		currentDescPath = descriptorChoice.getSelectedItem();
		
		//Event sent my choice
		add(initPhotoScrollPane(), "Center");
		theController.repaintMainFrame();
	}
}
