/*
 * Created on 05.jun.04
 *
 */
package com.arellano.ramon.atacama.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Label;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.MenuShortcut;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.file.FileManager;
import com.arellano.ramon.atacama.log.Logger;
import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * @author ramon.arellano
 *
 * This is the main window of the Atacama GUI. It is also the class to 
 * invoke from the command line in order to start the application.
 * 
 * This class handles menu events which set the active panel to be displayed.
 * There will only exist one instance of this class.
 */
public class MainFrame
	extends Frame
	implements ActionListener, WindowListener {
	//Controller, Config, Logger and FileManager	
	private static Controller theController = null;
	private static Config theConfig = null;
	private static Logger theLogger = null;
	private static FileManager theFileManager = null;

	//The components
	private TextArea logArea = null;
	private Label statusValueLabel = null;

	//The main panels
	private Panel currentPanel = null;
	private EditPhotoPanel editPhotoPanel = null;
	private EditAlbumPanel editAlbumPanel = null;
	private GenerateGalleryPanel generateGalleryPanel = null;

	/**
	 * The constructor
	 */
	public MainFrame() {
		super("Atacama");

		//Get the config and file manager
		theController = Controller.getInstance();

		//Set the controllers instance of the main frame
		theController.setMainFrame(this);

		//The controller will handle window events 
		this.addWindowListener(this);

		//Get the config , logger and file manager
		theConfig = Config.getInstance();
		theFileManager = FileManager.getInstance();
		theLogger = Logger.getInstance();

		//Instantiate log area
		logArea = new TextArea();

		//Set the logArea to be the GUI log component
		theLogger.setGuiLogComponent(logArea);

		//Show the splash screen
//		setEnabled(false);
//		new SplashWindow();

		//Initialise components and panels
		init();
	}

	/**
	 * The main method which starts the GUI application
	 * 
	 * @param args The command line arguments. Not used.
	 */
	public static void main(String[] args) {
		try {
			new MainFrame();
		} catch (RuntimeException exception) {
			theLogger.logError("A runtime excpetion occured.", exception);
		}
	}

	/**
	 * Initialises the Atacama GUI window
	 */
	public void init() {
		//Create frame
		theController.locateMaximized(this);
		
		setBackground(Color.lightGray);
		setLayout(new BorderLayout());

		//Set the menu bar
		setMenuBar(initMenuBar());

		initStatusPanel();

		//The starting panel
		initGenerateGalleryPanel();
		//initEditPhotoPanel();

		show();
	}

	/**
	 * Creates and adds menu objects
	 * @return The menu bar
	 */
	private MenuBar initMenuBar() {
		//Create generate menu
		Menu actionsMenu = new Menu("Actions");
		actionsMenu.setFont(new Font("Arial", Font.BOLD, 12));
		actionsMenu.addActionListener(this);

		actionsMenu.add(new MenuItem(Actions.MENU_EDIT_PHOTO_DESCRIPTORS, new MenuShortcut(KeyEvent.VK_1)));
		actionsMenu.add(new MenuItem(Actions.MENU_EDIT_ALBUM_DESCRIPTOR, new MenuShortcut(KeyEvent.VK_2)));
		actionsMenu.add(new MenuItem(Actions.MENU_GENERATE_GALLERY, new MenuShortcut(KeyEvent.VK_3)));
		actionsMenu.addSeparator();
		actionsMenu.add(new MenuItem(Actions.MENU_EXIT));

		//Create options menu
		Menu optionsMenu = new Menu("Options");
		optionsMenu.setFont(new Font("Arial", Font.BOLD, 12));
		optionsMenu.addActionListener(this);

		optionsMenu.add(new MenuItem(Actions.MENU_SAVE_OPTIONS));
		optionsMenu.add(new MenuItem(Actions.MENU_VIEW_LOG));

		//Create help menu
		Menu helpMenu = new Menu("Help");
		helpMenu.setFont(new Font("Arial", Font.BOLD, 12));
		helpMenu.addActionListener(this);

		helpMenu.add(new MenuItem(Actions.MENU_MANUAL));
		helpMenu.add(new MenuItem(Actions.MENU_VIEW_JAVADOC));
		helpMenu.add(new MenuItem(Actions.MENU_ABOUT));
		
		//Add the menu bar
		MenuBar menuBar = new MenuBar();
		menuBar.add(actionsMenu);
		menuBar.add(optionsMenu);
		menuBar.add(helpMenu);

		return menuBar;
	}

	/**
	 * Initialises the status panel
	 */
	public void initStatusPanel() {
		//Create panel
		Panel statusPanel = new Panel();
		statusPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

		Label statusLabel = new Label("Status:");
		statusLabel.setFont(new Font("Arial", Font.BOLD, 12));
		statusValueLabel =
			new Label(theController.getSizedString("Ready", 100));
		statusValueLabel.setFont(new Font("Arial", Font.PLAIN, 12));
		statusPanel.add(statusLabel);
		statusPanel.add(statusValueLabel);

		add(statusPanel, "South");
	}

	/**
	 * Returns the log area
	 * @return The log area component
	 */
	public TextArea getLogArea() {
		return logArea;
	}

	/**
	 * Clears the log
	 */
	public void clearLog() {
		logArea.setText("");
	}

	/**
	 * Activates the Edit Photo Panel
	 */
	private void initEditPhotoPanel() {
		setTitle("Atacama - Edit Photo Descriptors");

		theController.setStatus(
			"Please wait while initializing photo descriptor window...");

		//Create panel if it hasn't been created earlier
		boolean firstTime = true;
		if (editPhotoPanel == null) {
			editPhotoPanel = new EditPhotoPanel();
		} else {
			firstTime = false;
		}

		//Remove active panel if one exists
		if (currentPanel != null) {
			remove(currentPanel);
		}

		//Add the new panel
		add(editPhotoPanel, "Center");

		if (firstTime) {
			//Do not reinitialize every time
			editPhotoPanel.init();
		}

		//Set the current panel		
		currentPanel = editPhotoPanel;
		theController.setStatus("");

		//Repaint
		show();
	}

	/**
	 * Activates the Edit Album Panel
	 */
	private void initEditAlbumPanel() {
		setTitle("Atacama - Edit Album Descriptor");

		//Create panel if it hasn't been created earlier
		if (editAlbumPanel == null) {
			editAlbumPanel = new EditAlbumPanel();
			editAlbumPanel.init();
		}

		//Remove active panel if one exists
		if (currentPanel != null) {
			remove(currentPanel);
		}

		//Add the new panel
		add(editAlbumPanel, "Center");

		//Set the current panel		
		currentPanel = editAlbumPanel;

		//Repaint
		show();
	}

	/**
	 * Activates the Generate Album Panel
	 */
	private void initGenerateGalleryPanel() {
		setTitle("Atacama - Generate Gallery");

		//Create panel if it hasn't been created earlier
		if (generateGalleryPanel == null) {
			generateGalleryPanel = new GenerateGalleryPanel();
		}

		//Remove active panel if one exists
		if (currentPanel != null) {
			remove(currentPanel);
		}

		//Add the new panel
		add(generateGalleryPanel, "Center");
		generateGalleryPanel.init();

		//Set the current panel		
		currentPanel = generateGalleryPanel;

		//Repaint
		show();
	}

	/**
	 * Displays the user manual
	 */
	public void initManualDialog() {
		File manualFile =
			new File(
				theConfig.getApplicationDir()
					+ theConfig.getString("gui.file.text.manual"));

		String manualText = null;
		try {
			manualText = theFileManager.readTextFileToString(manualFile);
		} catch (AtacamaException exception) {
			new UserDialog("Failed to display user manual. See log for details.");
		}

		//Display the user manual
		new UserDialog(
			"User manual",
			manualText,
			Constants.MANUAL_DIALOG_DIMENSION,
			true);
	}

	/**
	 * Displays the about dialog
	 */
	private void initAboutDialog() {
		File manualFile =
			new File(
				theConfig.getApplicationDir()
					+ theConfig.getString("gui.file.text.about"));

		String aboutText = null;
		try {
			aboutText = theFileManager.readTextFileToString(manualFile);
		} catch (AtacamaException exception) {
			new UserDialog("Failed to display about dialog. See log for details.");
		}

		//Display the user manual
		new UserDialog(
			"About Atacama",
			aboutText,
			Constants.ABOUT_DIALOG_DIMENSION,
			false);
	}

	/**
	 * Sets the status label
	 * 
	 * @param statusString The status string
	 */
	public void setStatus(String statusString) {
		statusValueLabel.setText(statusString);
	}

	/**
	 * Handles all actions performed in this frame
	 * @param event The action event
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent event) {
		String action = event.getActionCommand();

		if (action.equals(Actions.MENU_GENERATE_GALLERY)) {
			initGenerateGalleryPanel();
		} else if (action.equals(Actions.MENU_EDIT_PHOTO_DESCRIPTORS)) {
			initEditPhotoPanel();
		} else if (action.equals(Actions.MENU_EDIT_ALBUM_DESCRIPTOR)) {
			initEditAlbumPanel();
		} else if (action.equals(Actions.MENU_SAVE_OPTIONS)) {
			if (generateGalleryPanel != null) {
				generateGalleryPanel.updateProperties();
			}
			theController.saveProperties();
		} else if (action.equals(Actions.MENU_VIEW_LOG)) {
			LogView.getInstance().openInFrame();
			theController.repaintMainFrame();
			LogView.getInstance().openInFrame();
		} else if (action.equals(Actions.MENU_MANUAL)) {
			initManualDialog();
		} else if (action.equals(Actions.MENU_VIEW_JAVADOC)) {
			theController.openJavaDocInDefaultBrowser();
		} else if (action.equals(Actions.MENU_ABOUT)) {
			initAboutDialog();
		} else if (action.equals(Actions.MENU_EXIT)) {
			System.exit(0);
		}
	}

	//Implementation of  all window listener interface methods
	public void windowClosing(WindowEvent event) {
		System.exit(0);
	}
	public void windowClosed(WindowEvent event) {
	}
	public void windowDeiconified(WindowEvent event) {
	}
	public void windowIconified(WindowEvent event) {
	}
	public void windowActivated(WindowEvent event) {
	}
	public void windowDeactivated(WindowEvent event) {
	}
	public void windowOpened(WindowEvent event) {
	}

}
