/*
 * Created on May 27, 2004
 *
 */
package com.arellano.ramon.atacama.html;

import java.io.File;
import java.math.BigInteger;
import java.util.Vector;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.entity.Album;
import com.arellano.ramon.atacama.entity.Photo;
import com.arellano.ramon.atacama.file.FileManager;
import com.arellano.ramon.atacama.log.Logger;
import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * @author ramon.arellano
 *
 * This is the class that generates all HTML for the gallery
 */
public class HtmlGenerator {

	//Logger, Config and FileManager
	static private Config theConfig = null;
	static private Logger theLogger = null;
	static private FileManager theFileManager = null;

	/**
	 * Constructor
	 */
	public HtmlGenerator() {
		theConfig = Config.getInstance();
		theLogger = Logger.getInstance();
		theFileManager = FileManager.getInstance();
	}

	/**
	 * Genereates all HTML for all albums and album index
	 * @param albums The vector of albums to generate HTML for
	 */
	public void generate(Vector albums) throws AtacamaException {
		//Check that directory were albums will be created exists,
		//if not, create it
		File albumsDir =
			new File(
				theConfig.getString("common.dir.target.gallery")
					+ theConfig.getString("common.dir.target.albums"));

		if (!albumsDir.exists()) {
			albumsDir.mkdirs();
		}

		//Copies static resources such as graphics and css
		copyResources();

		generateAlbumIndex(albums);

		generateAlbums(albums);
	}

	/**
	 * Generates HTML for all albums
	 * @param albums The vector of albums to generate HTML for
	 * @throws AtacamaException Throws when failed
	 */
	private void generateAlbums(Vector albums) throws AtacamaException {
		//For each album create corresponding html in its own directory
		for (int albumIndex = 0; albumIndex < albums.size(); albumIndex++) {
			Album album = (Album) albums.elementAt(albumIndex);

			//Get absolute path where albums will be stored
			File albumHtmlDir = new File(getAlbumHtmlDir(album, false));
			if (albumHtmlDir.exists()) {
				if (!theConfig.getBool("html.overwrite")) {
					//Album exists and overwrite is off, warn that album will not be created
					theLogger.logWarning(
						"HTML for album '"
							+ album.getAlbumName()
							+ "' exists and was not overwritten.");
					continue;
				}
			} else {
				//Album html directory doesn't exist create it
				albumHtmlDir.mkdirs();
			}

			//Warn if album does not contain photos
			if (album.getPhotos() == null || album.getPhotos().size() == 0) {
				theLogger.logWarning(
					"Album '"
						+ album.getAlbumName()
						+ "' does not contain photos. HTML and will not be created.");
				continue;
			}

			theLogger.logTrace(
				"Generating HTML for album '" + album.getAlbumName() + "'.");

			//Generate no-frames photo index
			File noFramesPhotoIndexFile =
				new File(
					getAlbumHtmlDir(album, false)
						+ theConfig.getString("html.url.noframes.prefix")
						+ theConfig.getString(
							"html.resource.template.photo.index"));
			generatePhotoIndex(album, noFramesPhotoIndexFile, false);

			//Generate frames page
			File framesPageFile =
				new File(
					getAlbumHtmlDir(album, false)
						+ theConfig.getString(
							"html.resource.template.frames.page"));
			generateFramesPage(album, framesPageFile);

			//Generate frames photo index
			File framesPhotoIndexFile =
				new File(
					getAlbumHtmlDir(album, false)
						+ theConfig.getString("html.url.frames.prefix")
						+ theConfig.getString(
							"html.resource.template.photo.index"));
			generatePhotoIndex(album, framesPhotoIndexFile, true);

			//Generate photo pages
			generatePhotoPages(album, getAlbumHtmlDir(album, false));
		}
	}

	/**
	 * Generates photo pages for all photos
	 * 
	 * @param album The album to generate photo page for
	 * @param albumHtmlPath The path to the HTML file to write to
	 * @throws AtacamaException Throws when failed
	 */
	private void generatePhotoPages(Album album, String albumHtmlPath)
		throws AtacamaException {
		//Read the html template from file
		String templateName =
			theConfig.getString("html.resource.template.frames.photo.page");
		Document htmlDoc = readHtmlTemplate(templateName);

		//Set CSS link
		setCssLink(htmlDoc, false);

		//Set back to gallery link
		Element backToGalleryLink =
			getElementById(htmlDoc, "a", "backToGallery", templateName);
			backToGalleryLink.setAttribute("href", getBackToGalleriesLink(album));
		backToGalleryLink.setAttribute("target", "_top");

		//Get next link
		Element nextLink =
			getElementById(htmlDoc, "a", "next", templateName);

		//Get previous link
		Element previousLink =
			getElementById(htmlDoc, "a", "previous", templateName);

		//Set the meta content tag to the description of the album
		Element metaDescription =
			(Element) htmlDoc.getElementsByTagName("meta").item(0);
		metaDescription.setAttribute("content", album.getComment());

		//For all photos insert new image and a new comment
		for (int index = 0; index < album.getPhotos().size(); index++) {
			Photo photo = (Photo) album.getPhotos().elementAt(index);

			//Set the URL to the large photo
			Element photoImage =
				getElementById(htmlDoc, "img", "photoImage", templateName);
			String photoUrl =
				"../"
					+ theConfig.getString("photo.dir.photos")
					+ theConfig.getString("photo.dir.target.photo.large")
					+ photo.getPhotoName();
			//Only forward slashes in URL
			photoUrl = photoUrl.replace('\\', '/');

			photoImage.setAttribute("src", photoUrl);
			photoImage.setAttribute("alt", photo.getComment());

			//Set the photo comment
			Element photoComment =
				getElementById(htmlDoc, "p", "photoComment", templateName);
			photoComment.removeChild(photoComment.getFirstChild());
			photoComment.appendChild(
				htmlDoc.createTextNode(photo.getComment()));

			//Insert next link if not last photo
			if(index < album.getPhotos().size() - 1) {
				nextLink.setAttribute("href", getPhotoHtmlName((Photo) album.getPhotos().elementAt(index+1)));
				backToGalleryLink.getParentNode().appendChild(nextLink);
			} else {
				//No next link needed
				nextLink.getParentNode().removeChild(nextLink);				
			}

			//Insert previous link if not first photo
			if(index > 0) {
				previousLink.setAttribute("href", getPhotoHtmlName((Photo) album.getPhotos().elementAt(index-1)));
				backToGalleryLink.getParentNode().insertBefore(previousLink, backToGalleryLink);
				backToGalleryLink.getParentNode().insertBefore(htmlDoc.createTextNode(" "), backToGalleryLink);
			} else {
				//No previous link needed
				previousLink.getParentNode().removeChild(previousLink);				
			}

			//Save file to disk
			File photoHtmlFile =
				new File(albumHtmlPath + getPhotoHtmlName(photo));
			theFileManager.writeXmlFile(htmlDoc, photoHtmlFile);
		}
	}

	/**
	 * Generates the frames page, which displays the single row photo index
	 * and the photo page
	 * 
	 * @param album The album
	 * @param file The HTML file
	 * @throws AtacamaException Throws when failed
	 */
	private void generateFramesPage(Album album, File file)
		throws AtacamaException {
		//Read the html template from file
		String templateName =
			theConfig.getString("html.resource.template.frames.page");
		Document htmlDoc = readHtmlTemplate(templateName);

		//Set HTML document title and CSS link
		setHtmlTitle(album.getAlbumName(), htmlDoc);

		//Set the meta content tag to the description of the album
		Element metaDescription =
			(Element) htmlDoc.getElementsByTagName("meta").item(0);
		if(metaDescription != null) {
			metaDescription.setAttribute("content", album.getComment());
		}

		//Set photo index frame
		Element indexFrame =
			getElementById(htmlDoc, "frame", "photoIndexFrame", templateName);
		indexFrame.setAttribute(
			"src",
			theConfig.getString("html.url.frames.prefix")
				+ theConfig.getString("html.resource.template.photo.index"));

		/* REMOVED after new layout
		//Set photo frame to first photo
		Photo firstPhoto = (Photo) album.getPhotos().elementAt(0);
		Element photoFrame =
			getElementById(htmlDoc, "frame", "photoFrame", templateName);
		if (theFileManager
			.isFileType(new File(firstPhoto.getPhotoName()), "JPG")) {
			//First photo is JPEG, set frame to corresponding photo page
			photoFrame.setAttribute("src", getPhotoHtmlName(firstPhoto));
		} else {
			//First photo is not JPEG, set frame directly to file
			photoFrame.setAttribute(
				"src",
				"../"
					+ theConfig.getString("photo.dir.photos").replace('\\', '/')
					+ theConfig.getString("photo.dir.target.photo.large").replace('\\', '/')
					+ firstPhoto.getPhotoName());
		}
		*/

		//Save file to disk
		theFileManager.writeXmlFile(htmlDoc, file);
	}

	/**
	 * Creates the name of the html page that shows the give photo
	 * 
	 * @param photo The photo
	 * @return The name
	 */
	private String getPhotoHtmlName(Photo photo) {
		String photoName =
			photo.getPhotoName().substring(
				0,
				photo.getPhotoName().lastIndexOf("."));
		return photoName + ".html";
	}

	/**
	 * Generates the both the frames and the no-frames
	 * photo index html file for a specific album
	 * and writes it to the specified album path. The frames flag decides 
	 * the number of columns and the URL of the photo links.
	 * 
	 * @param album The album
	 * @param file The album directory
	 * @param frames True if the photo index should be part of a frame set, false
	 * if it will be a stand-alone page
	 * @throws AtacamaException Throws when failed
	 */
	private void generatePhotoIndex(Album album, File file, boolean frames)
		throws AtacamaException {
		//Read the html template from file
		String templateName =
			theConfig.getString("html.resource.template.photo.index");
		Document htmlDoc = readHtmlTemplate(templateName);

		//Set HTML document title and CSS link
		setHtmlTitle(album.getAlbumName(), htmlDoc);
		setCssLink(htmlDoc, false);

		//Set back to gallery link
		Element backToGalleryLink =
			getElementById(htmlDoc, "a", "backToGallery", templateName);
		backToGalleryLink.setAttribute("href", getBackToGalleriesLink(album));
		backToGalleryLink.setAttribute("target", "_top");
		backToGalleryLink.removeAttribute("id");

		//Set the album name
		Element albumName =
			getElementById(htmlDoc, "h1", "albumName", templateName);
		albumName.removeChild(albumName.getFirstChild());
		albumName.appendChild(htmlDoc.createTextNode(getFullAlbumName(album)));

		//Find the default album title row 
		Element defaultThumbRow =
			getElementById(htmlDoc, "tr", "thumbRow", templateName);

		if (defaultThumbRow == null ) {
			theLogger.logError(
				"Failed to find default thumb rows in html template for noframes photo index.");
		}

		//Clone thumb cell and comment cell nodes
		Element newThumbCell =
			(Element) defaultThumbRow.getElementsByTagName("td").item(0);

		//Remove the id attributes before continuing
		//(XHTML does not allow duplicate id attributes)
		defaultThumbRow.removeAttribute("id");

		//Get table node
		Element tableElement = (Element) defaultThumbRow.getParentNode();

		//Get page node
		Element pageElement = (Element) getElementById(htmlDoc, "div", "photoIndexPage", templateName);
				
		//Different behaviour depending on if we're generating the frames or 
		//no-frames photo index
		int photosPerRow = 0;
		if (frames) {
			//Set number of photos per row to 1 which implies single column
			photosPerRow = 1;
			//Set width of page to narrow
			pageElement.setAttribute("style", theConfig.getString("html.design.frames.page.style"));
		} else {
			photosPerRow =
				theConfig.getInt("html.design.noframes.photos.per.row");
			pageElement.setAttribute("style", theConfig.getString("html.design.noframes.page.style"));
		}

		//For all photos insert new thumb cell and new thumb comment cell
		for (int index = 0; index < album.getPhotos().size(); index++) {
			Photo photo = (Photo) album.getPhotos().elementAt(index);

			//If number of photos per row has been reached (or first pass), create new row
			int currentRowNumber =
				new BigInteger(new Integer(index).toString())
					.mod(new BigInteger(new Integer(photosPerRow).toString()))
					.intValue();
			if (currentRowNumber == 0) {
				Element newThumbRow =
					(Element) defaultThumbRow.cloneNode(false);
				defaultThumbRow.getParentNode().appendChild(newThumbRow);
			}

			//Create thumb URL
			String thumbUrl = "";
			if (theFileManager
				.isFileType(new File(photo.getPhotoName()), "JPG")) {
				//Photo is JPG set thumb url to the resized image
				thumbUrl =
					"../"
						+ theConfig.getString("photo.dir.photos")
						+ theConfig.getString("photo.dir.target.photo.thumb")
						+ photo.getPhotoName();
			} else {
				//Photo is not JPG set thumb url to standard image
				thumbUrl =
					"../../../"
						+ theConfig.getString("html.resource.dir.target")
						+ theConfig.getString("html.resource.image.photo.thumb");
			}
			//Only forward slashes in URL
			thumbUrl = thumbUrl.replace('\\', '/');


			//Create large photo URL
			String photoUrl = "";
			if (!theFileManager.isFileType(
					new File(photo.getPhotoName()),
					"JPG")) {
				//The photo is not JPEG, the link should be directly to the large photo file.
				photoUrl += "../"
					+ theConfig.getString("photo.dir.photos")
					+ theConfig.getString("photo.dir.target.photo.large")
					+ photo.getPhotoName();
			} else if(!frames) {
				//Were creating the photo index for the no-frames page
				//each photo should link to the frames page with correct page to show
				photoUrl += theConfig.getString("html.url.frames.prefix") + ".html?" + getPhotoHtmlName(photo);
			} else {
				photoUrl += getPhotoHtmlName(photo);
			}
			//Only forward slashes in URL
			photoUrl = photoUrl.replace('\\', '/');

			//Set thumb link Url
			Element thumbLink =
				(Element) newThumbCell.getElementsByTagName("a").item(0);
			thumbLink.setAttribute("href", photoUrl);
			if (frames) {
				thumbLink.setAttribute("target", "photo");
			}

			//Set thumb image src url and alt
			Element thumbImage =
				(Element) thumbLink.getElementsByTagName("img").item(0);
			thumbImage.setAttribute("src", thumbUrl);
			//Set the alternative text for the thumb according to props
			if(theConfig.getBool("html.photo.comment.in.thumb.alt")) {
				thumbImage.setAttribute("alt", getTruncComment(photo));
			} else {
				thumbImage.setAttribute("alt", "Image");
			}

			//Set photo comment
			Element newCommentParagraph = (Element) newThumbCell.getElementsByTagName("p").item(0);
			newCommentParagraph.removeChild(newCommentParagraph.getFirstChild());
			newCommentParagraph.appendChild(
				htmlDoc.createTextNode(getTruncComment(photo)));

			//Clone and insert thumb cell in last row
			NodeList rows = tableElement.getElementsByTagName("tr");
			rows.item(rows.getLength() - 1).appendChild(
				newThumbCell.cloneNode(true));

		}

		//Add extra row with back to galleries link at the bottom
		defaultThumbRow.getParentNode().getParentNode().getParentNode().appendChild(
			backToGalleryLink.getParentNode().cloneNode(true));

		//Remove the default cells
		defaultThumbRow.getParentNode().removeChild(defaultThumbRow);

		//Save file to disk
		theFileManager.writeXmlFile(htmlDoc, file);

	}

	/**
	 * Truncates the comment string at a max length and ending it with ...
	 * 
	 * @param photo The photo
	 * @return The truncated string
	 */
	private String getTruncComment(Photo photo) {
		String commentString = photo.getComment();
		int commentMaxLength =
			theConfig.getInt("html.thumb.comment.max.length");

		if (commentString.length() > commentMaxLength) {
			commentString = commentString.substring(0, commentMaxLength);
			commentString =
				commentString.substring(0, commentString.lastIndexOf(" "))
					+ "...";
		}

		return commentString;
	}

	/**
	 * Copies all static html resources to a target path.
	 * Checks overwrite flag before overwriting.
	 * @throws AtacamaException Throws when failed
	 */
	private void copyResources() throws AtacamaException {
		theLogger.logTrace("Copying static HTML resources.");

		File resourceSourceDir =
			new File(
				theConfig.getApplicationDir()
					+ theConfig.getString("html.resource.dir.source"));

		File resourceTargetDir =
			new File(
				theConfig.getString("common.dir.target.gallery")
					+ theConfig.getString("html.resource.dir.target"));

		if (resourceTargetDir.exists()
			&& !theConfig.getBool("html.resource.overwrite")) {

			//Warn that files were not copied
			theLogger.logWarning(
				"HTML resources directory exists. Files were not copied.");
		} else {

			//Copy all html resources to albums directory
			theFileManager.copyDirectory(resourceSourceDir, resourceTargetDir);
		}

	}

	/**
	 * Reads the specified template from file and returns a DOM document
	 * 
	 * @param templateName The name of the template
	 * @return The template as DOM document
	 * @throws AtacamaException Throws when failed
	 */
	private Document readHtmlTemplate(String templateName)
		throws AtacamaException {
		String templatesPath =
			theConfig.getApplicationDir()
				+ theConfig.getString("html.resource.dir.source.templates");

		Document doc =
			theFileManager.readXmlFile(new File(templatesPath, templateName));

		// normalize text representation
		doc.getDocumentElement().normalize();

		return doc;
	}

	/**
	 * Generates the album index from the album descriptor and writes it to file
	 * @param albums The vector of albums
	 * @throws AtacamaException Throws when failed
	 */
	private void generateAlbumIndex(Vector albums) throws AtacamaException {
		theLogger.logTrace("Generating album index.");

		File albumIndexTargetFile =
			new File(
				theConfig.getString("common.dir.target.gallery")
					+ theConfig.getString("html.album.index"));

		//If no overwrite check if file already exists
		if (albumIndexTargetFile.exists()
			&& !theConfig.getBool("html.overwrite")) {

			//Warn that file was not recreated
			theLogger.logWarning(
				"Album index file exists and was not regenerated.");
			return;
		}

		//Read the html template from file
		String templateName =
			theConfig.getString("html.resource.template.album.index");
		Document htmlDoc = readHtmlTemplate(templateName);

		//Set the CCS link with property file value
		setCssLink(htmlDoc, true);

		//Get default rows. These will be duplicated to create the albums page.
		Element defaultGroupRow =
			getElementById(htmlDoc, "div", "groupRow", templateName);
		Element defaultAlbumRow =
			getElementById(htmlDoc, "div", "albumRow", templateName);

		if (defaultAlbumRow == null
			|| defaultGroupRow == null) {
			theLogger.logError(
				"Failed to find default album rows in html template for album index.");
		}

		//Remove the id attributes before continuing
		//(XHTML does not allow duplicate id attributes)
		defaultAlbumRow.removeAttribute("id");
		defaultGroupRow.removeAttribute("id");

		//For each album set the values of the html row, and create new row
		String previousYear = "";
		String previousCountry = "";
		String previousMonth = "";
		for (int index = 0; index < albums.size(); index++) {
			Album album = (Album) albums.elementAt(index);

			//Do not create entry for emtpy albums
			if (album.getPhotos() == null || album.getPhotos().size() == 0) {
				continue;
			}

			//Get album URLs
			String thumbUrl = null;
			if (theFileManager
				.isFileType(
					new File(
						album.getPrimaryPhoto().getPhotoFilePath()),
					"JPG")) {
				//First photo is JPG, use resized album thumb image
				thumbUrl =
					theConfig.getAlbumPhotosDir(album, true)
						+ theConfig.getString("photo.image.album.thumb");
			} else {
				//First photo is not JPG, use standard thumb image
				thumbUrl =
					theConfig.getString("html.resource.dir.target")
						+ theConfig.getString("html.resource.image.album.thumb");
			}
			//Only forward slashes in URL
			thumbUrl = thumbUrl.replace('\\', '/');
			
			String noFramePhotoIndexUrl =
				getAlbumHtmlDir(album, true)
					+ theConfig.getString("html.url.noframes.prefix")
					+ theConfig.getString("html.resource.template.photo.index");
			//Only forward slashes in URL
			noFramePhotoIndexUrl = noFramePhotoIndexUrl.replace('\\', '/');

			try {

				//Create new album title row by cloning default
				Element newAlbumRow = (Element) defaultAlbumRow.cloneNode(true);

				//Set thumb link url
				Element thumbLinkElement =
					(Element) newAlbumRow.getElementsByTagName("a").item(0);
				thumbLinkElement.setAttribute("href", noFramePhotoIndexUrl);
				
				//Set name to be able to link back to this point
				thumbLinkElement.setAttribute("name", createNameToken(album.getAlbumName()).replace("'", ""));

				//Set new album thumb source and alternative text
				Element image =
					(Element) thumbLinkElement.getElementsByTagName(
						"img").item(
						0);
				image.setAttribute("src", thumbUrl);
				//Set the alternative text for the thumb according to props
				if(theConfig.getBool("html.photo.comment.in.thumb.alt")) {
					image.setAttribute("alt", album.getComment());
				} else {
					image.setAttribute("alt", "Image");
				}

				//Set the album title text and link URL
				Node titleLinkNode =
					newAlbumRow.getElementsByTagName("a").item(1);
				titleLinkNode.removeChild(titleLinkNode.getFirstChild());
				titleLinkNode.appendChild(
					htmlDoc.createTextNode(album.getAlbumName()));
				Element titleLinkElement = (Element) titleLinkNode;
				titleLinkElement.setAttribute("href", noFramePhotoIndexUrl);

				//Remove the password protected album link if album is public
				if("false".equals(album.getPrivateOnly())) {
					Node passwordProtectedLink = newAlbumRow.getElementsByTagName("a").item(2);
					passwordProtectedLink.getParentNode().removeChild(passwordProtectedLink);
				}
				
				//Set album comment
				Node newCommentCellNode =
					newAlbumRow.getElementsByTagName("p").item(0);
				newCommentCellNode.removeChild(
					newCommentCellNode.getFirstChild());
				newCommentCellNode.appendChild(
					htmlDoc.createTextNode(album.getComment()));

				//If the year or country has changed since previous album, make a group header
				if (!(album.getCountry().equals(previousCountry)
					&& album.getYear().equals(previousYear)
					&& album.getMonth().equals(previousMonth))) {
					//Create new album group row by cloning default
					Element newGroupRow =
						(Element) defaultGroupRow.cloneNode(true);
					Node newGroupCell =
						newGroupRow.getElementsByTagName("h2").item(0);
					newGroupCell.removeChild(newGroupCell.getFirstChild());
					newGroupCell.appendChild(
						htmlDoc.createTextNode(
							album.getCountry()
								+ " / "
								+ album.getMonth()
								+ " "
								+ album.getYear()));

					//Insert new row to document
					defaultAlbumRow.getParentNode().insertBefore(
						newGroupRow,
						defaultAlbumRow);
				}

				//Insert new row to document
				defaultAlbumRow.getParentNode().insertBefore(
					newAlbumRow,
					defaultAlbumRow);

				//Album has been added. Set previous country and year
				previousYear = album.getYear();
				previousCountry = album.getCountry();
				previousMonth = album.getMonth();
			} catch (NullPointerException exception) {
				theLogger.logError(
					"Failed to generate album index. Most probably an error in the album index template (missing html element or attribute). "
						+ exception.toString());
			}
		}

		//Remove default rows
		defaultAlbumRow.getParentNode().removeChild(defaultAlbumRow);
		defaultGroupRow.getParentNode().removeChild(defaultGroupRow);

		//Save file to disk
		theFileManager.writeXmlFile(htmlDoc, albumIndexTargetFile);

	}

	/**
	 * Retrieves an element from a DOM document describe by its ID
	 * 
	 * @param htmlDoc The DOM document
	 * @param tagName The tag name
	 * @param id The id attribute of the element
	 * @param templateName The name of template
	 * @return The retrieved element
	 * @throws AtacamaException Throws when fails
	 */
	private Element getElementById(
		Document htmlDoc,
		String tagName,
		String id,
		String templateName)
		throws AtacamaException {
		//Find all tags of this kind in the HTML document
		NodeList nodeList = htmlDoc.getElementsByTagName(tagName);

		if (nodeList == null || nodeList.getLength() == 0) {
			theLogger.logError(
				"Expected to find elment with name '"
					+ tagName
					+ "' in template "
					+ templateName
					+ "'");
		}

		for (int index = 0; index < nodeList.getLength(); index++) {
			Element element = (Element) nodeList.item(index);

			if (element.getAttribute("id") != null
				&& element.getAttribute("id").equals(id)) {
				return element;
			}
		}

		theLogger.logError(
			"Expected to find element with id '"
				+ id
				+ "' in template "
				+ templateName
				+ "'");

		//Not found
		return null;
	}

	/**
	 * Set the title of the HTML document
	 * 
	 * @param title The title to set
	 * @param htmlDoc The document 
	 */
	private void setHtmlTitle(String title, Document htmlDoc) {
		Element htmlTitle =
			(Element) htmlDoc.getElementsByTagName("title").item(0);

		htmlTitle.removeChild(htmlTitle.getFirstChild());
		htmlTitle.appendChild(htmlDoc.createTextNode(title));
	}

	/**
	 * Set the CSS link of the HTML document
	 * 
	 * @param htmlDoc The DOM document
	 */
	private void setCssLink(Document htmlDoc, boolean isAlbumIndex) {
		Element htmlLink =
			(Element) htmlDoc.getElementsByTagName("link").item(0);

		String relativeUrl = "";
		if (!isAlbumIndex) {
			relativeUrl += "../../../";
		}

		relativeUrl += theConfig.getString("html.resource.css.url");
		htmlLink.setAttribute("href", relativeUrl);
	}

	/**
	 * Creates the directory path html for a specific album
	 * 
	 * @param album The album
	 * @param relativeToGallery True if relative to gallery root
	 * @return The path
	 */
	private String getAlbumHtmlDir(Album album, boolean relativeToGallery) {
		return theConfig.getAlbumDir(album, relativeToGallery)
			+ theConfig.getString("html.dir.target.html");
	}

	/**
	 * Constructs the full album name to be used in on the html pages
	 * 
	 * @param album The album
	 * @return The full album name
	 */
	private String getFullAlbumName(Album album) {
		return album.getAlbumName()
			+ " - "
			+ album.getCountry()
			+ " "
			+ album.getMonth()
			+ " "
			+ album.getYear();
	}

	/**
	 * Creates the realitve URL to get back to the album index
	 * 
	 * @return The link URL
	 */
	private String getBackToGalleriesLink(Album album) {
		return "../../../" + theConfig.getString("html.album.index") 
			+ "#"
			+ createNameToken(album.getAlbumName().replace("'", ""));
	}
	
	/**
	 * Removes spaces and commas from an album name in order to use
	 * it as a name token attribute in HTML
	 * 
	 * @return The name token
	 */
	private String createNameToken(String name) {
		name = name.replaceAll(" ", "");
		name = name.replaceAll(",", "");
		name = name.replaceAll("-", "");
		
		return name;
	}
}
