/*
 * Created on May 27, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.arellano.ramon.atacama.log;

import java.awt.TextArea;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.GregorianCalendar;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.system.AtacamaException;

/**
 * @author ramon.arellano
 *
 * Logs to standard output or file depending on configuration settings
 */
public class Logger {

	/**
	 * Debug log level. Should only be used for development purposes.
	 */
	public static final int DEBUG = 0;

	/**
	 * Trace log level. Used to trace the running of the application.
	 */
	public static final int TRACE = 1;

	/**
	 * Warning log level. Used when somethin non-critical fails.
	 */
	public static final int WARNING = 2;

	/**
	 * Error log level. Something unexpected has happened and must be
	 * reacted upon.
	 */
	public static final int ERROR = 3;

	//A handle to the GUI component that shows the log
	static public TextArea guiComponentLog = null;

	//A handle to the unique instance.
	static private Logger theInstance = null;

	//The log file
	private FileOutputStream logFile = null;

	//The config instance
	static private Config theConfig = null;

	/**
	 * Constructor
	 */
	protected Logger() {
		theConfig = Config.getInstance();

		if (theConfig.getBool("log.to.file")) {
			//Create path of log file
			String logFilePath =
				theConfig.getApplicationDir()
					+ theConfig.getString("log.dir.target");

			String logFileName = null;
			//Create complete file name
			if (theConfig.getBool("log.filename.date")) {
				logFileName =
					GregorianCalendar.getInstance().getTime().getYear()
						+ "-"
						+ GregorianCalendar.getInstance().getTime().getMonth()
						+ "-"
						+ GregorianCalendar.getInstance().getTime().getDate()
						+ " "
						+ GregorianCalendar.getInstance().getTime().getHours()
						+ "-"
						+ GregorianCalendar.getInstance().getTime().getMinutes()
						+ "-"
						+ GregorianCalendar.getInstance().getTime().getSeconds()
						+ ".log";
			} else {
				logFileName = "atacama.log";
			}

			try {
				logFile =
					new FileOutputStream(new File(logFilePath, logFileName));
			} catch (FileNotFoundException exception) {
				String logLine =
					createLogLine(
						"Could not create log file: "
							+ logFileName
							+ ";"
							+ exception.toString(),
						Logger.ERROR);
				throw new RuntimeException(logLine);
			}
		}
	}

	/**
	 * Retrieves the unique instance of the logger
	 * @return The unique instance of this class.
	 */
	static public Logger getInstance() {
		if (null == theInstance) {
			theInstance = new Logger();
		}
		return theInstance;
	}

	/**
	 * Logs debug messages if configuration allows it
	 * 
	 * @param message The message
	 */
	public void logDebug(String message) {
		if (theConfig.getBool("log.debug")) {
			String logLine = createLogLine(message, DEBUG);
			writeMessage(logLine);
		}
	}

	/**
	 * Logs trace messages if configuration allows it
	 * 
	 * @param message The message
	 */
	public void logTrace(String message) {
		if (theConfig.getBool("log.trace")) {
			String logLine = createLogLine(message, TRACE);
			writeMessage(logLine);
		}
	}

	/**
	 * Logs warning messages if configuration allows it
	 * 
	 * @param message The message
	 */
	public void logWarning(String message) {
		String logLine = createLogLine(message, WARNING);
		writeMessage(logLine);
	}


	/**
	 * Logs error messages and throws AtacamaException
	 * 
	 * @param message The message
	 * @throws AtacamaException This exception is ALWAYS thrown
	 */
	public void logError(String message) {
		logError(message, null);
	}

	/**
	 * Logs error messages and throws AtacamaException
	 * 
	 * @param message The message
	 * @param exception The cause exception
	 * @throws AtacamaException This exception is ALWAYS thrown
	 */
	public void logError(String message, Exception exception) {
		String exceptionMessage = null;
		if (exception != null) {
			exceptionMessage = exception.getMessage();
		}

		String logLine = createLogLine(message + " - Original exception: " + exceptionMessage, ERROR);
		writeMessage(logLine);

		logLine = createLogLine("Printing stack trace", TRACE);

		writeMessage(logLine);
	}

	/**
	 * Logs test messages if configuration allows it
	 * 
	 * @param message The message
	 */
	public void logTest(String message) {
		writeMessage(message + "\n");
	}

	/**
	 * Creates a log line with the input message
	 * 
	 * @param message The message
	 */
	public static String createLogLine(String message, int level) {
		Throwable throwable = new Throwable();
		StackTraceElement stackTraceElement = throwable.getStackTrace()[2];

		String levelString = null;
		switch (level) {
			case 0 :
				levelString = "DEBUG";
				break;
			case 1 :
				levelString = "TRACE";
				break;
			case 2 :
				levelString = "WARNI";
				break;
			case 3 :
				levelString = "ERROR";
				break;
			default :
				levelString = "UNKNO";
		}

		Date date = GregorianCalendar.getInstance().getTime();
		String hours = Integer.toString(date.getHours());
		if (hours.length() == 1)
			hours = "0" + hours;

		String minutes = Integer.toString(date.getMinutes());
		if (minutes.length() == 1)
			minutes = "0" + minutes;

		String seconds = Integer.toString(date.getSeconds());
		if (seconds.length() == 1)
			seconds = "0" + seconds;

		//When running from binary the line number is set to -1.
		//This should not be shown.
		String lineNumber = " line " + stackTraceElement.getLineNumber();
		if(lineNumber.indexOf("-1") != -1) {
			lineNumber = "";
		}

		String logLine =
			hours
				+ ":"
				+ minutes
				+ ":"
				+ seconds
				+ " "
				+ levelString
				+ " "
				+ stackTraceElement.getClassName().substring(
					stackTraceElement.getClassName().lastIndexOf(".") + 1)
				+ "."
				+ stackTraceElement.getMethodName()
				+ "()"
				+ lineNumber
				+ ": "
				+ message
				+ "\n";

		return logLine;
	}

	/**
	 * Writes messages to the configured outputs
	 * 
	 * @param message The message
	 */
	private void writeMessage(String message) {
		if (message == null) {
			message = "NULL";
		}

		//Log to System out
		if (theConfig.getBool("log.to.standard.output")) {
			System.out.print(message);
		}

		//Log to GUI
		if (guiComponentLog != null) {
			guiComponentLog.setText(guiComponentLog.getText() + message);
		}

		//Log to file
		if (theConfig.getBool("log.to.file")) {
			try {
				logFile.write(message.getBytes());
				logFile.flush();
			} catch (IOException exception) {
				String logLine =
					createLogLine(
						"Could not write to log file: " + exception.toString(),
						Logger.ERROR);
				throw new RuntimeException(logLine);
			}
		}
	}

	/**
	 * This method sets the optional GUI component that shows the log
	 * 
	 * @param logArea The text area component
	 */
	public void setGuiLogComponent(TextArea logArea) {
		guiComponentLog = logArea;
	}
}
