/*
 * Created on May 27, 2004
 *
 */
package com.arellano.ramon.atacama.photo;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Panel;
import java.awt.image.BufferedImage;
import java.awt.image.CropImageFilter;
import java.awt.image.FilteredImageSource;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.file.FileManager;
import com.arellano.ramon.atacama.log.Logger;
import com.arellano.ramon.atacama.system.AtacamaException;
import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGEncodeParam;
import com.sun.image.codec.jpeg.JPEGImageEncoder;

/**
 * Resizes jpeg image files on your file system.
 * Uses the com.sun.originalImage.codec.jpeg package shipped
 * by Sun with Java 2 Standard Edition.
 *
 */
public class ImageManipulator extends Panel {

	//Logger, Config and FileManager
	static private Config theConfig = null;
	static private Logger theLogger = null;
	static private FileManager theFileManager = null;

	/**
	 * Constructor
	 */
	public ImageManipulator() {
		theConfig = Config.getInstance();
		theLogger = Logger.getInstance();
		theFileManager = FileManager.getInstance();
	}

	/**
	 * Resizes images and crops it be the square shaped of a specified size
	 * 
	 * @param originalImageFile Original image
	 * @param resizedImageFile Resized image
	 * @param squareSize The width or height in pixels of square
	 * @throws AtacamException Throws when fails
	 */
	public void resizeAndCropSquare(
		File originalImageFile,
		File resizedImageFile,
		int squareSize)
		throws AtacamaException {

		//Check that image file exists
		if (!originalImageFile.exists()) {
			theLogger.logError(
				"The image to be resized doesn't exist: " + originalImageFile);
		}

		//Load the originalImage
		Image originalImage =
			getToolkit().getImage(originalImageFile.getPath());
		loadImage(originalImage);

		Image resizedImage = null;
		int posX = 0;
		int imageWidth = originalImage.getWidth(this);
		int imageHeight = originalImage.getHeight(this);
		int maxHeight = 0;
		int newWidth = 0;

		if (imageWidth > imageHeight) {
			maxHeight = squareSize;

			newWidth =
				Math.round(
					new Float(squareSize).floatValue()
						/ new Float(originalImage.getHeight(this)).floatValue()
						* new Float(originalImage.getWidth(this)).floatValue());

			posX =
				Math.round(
					(new Float(newWidth).floatValue()
						- new Float(maxHeight).floatValue())
						/ new Float(2).floatValue());
		} else {
			maxHeight = squareSize * imageHeight / imageWidth;

			newWidth =
				Math.round(
					new Float(squareSize).floatValue()
						/ new Float(originalImage.getHeight(this)).floatValue()
						* new Float(originalImage.getWidth(this)).floatValue());
		}

		resizedImage = resizeImage(originalImage, maxHeight);

		CropImageFilter cropImageFilter =
			new CropImageFilter(posX, 0, squareSize, squareSize);

		//Apply the filter to the image and make a new image of the cropped area
		Image croppedImage =
			createImage(
				new FilteredImageSource(
					resizedImage.getSource(),
					cropImageFilter));

		writeImageToFile(croppedImage, resizedImageFile);
	}

	/**
	 * Resizes an image file
	 * 
	 * @param originalImageFile The original image file
	 * @param resizedImageFile The resized image file
	 * @throws AtacamException Throws when fails
	 */
	public void resize(
		File originalImageFile,
		File resizedImageFile,
		int maxHeight)
		throws AtacamaException {

		//Check that image file exists
		if (!originalImageFile.exists()) {
			theLogger.logError(
				"The image to be resized doesn't exist: " + originalImageFile);
		}

		//Load the originalImage
		Image originalImage =
			getToolkit().getImage(originalImageFile.getPath());
		loadImage(originalImage);

		//Check if image is actually taller than max height
		if(originalImage.getHeight(this) > maxHeight) {
			//The image is taller, resize
			Image resizedImage = resizeImage(originalImage, maxHeight);
			writeImageToFile(resizedImage, resizedImageFile);
		} else {
			//The image is not taller, do not resize, just write			
			writeImageToFile(originalImage, resizedImageFile);
		}
	}

	/**
	 * Resizes and image to a specific height while retaining aspect
	 * @param originalImage The image to resize
	 */
	public Image resizeImage(Image originalImage, int maxHeight) {

		int newWidth =
			Math.round(
				new Float(maxHeight).floatValue()
					/ new Float(originalImage.getHeight(this)).floatValue()
					* new Float(originalImage.getWidth(this)).floatValue());

		//Resize the originalImage
		Image resizedImage =
			originalImage.getScaledInstance(
				newWidth,
				maxHeight,
				Image.SCALE_SMOOTH);

		return resizedImage;
	}

	/**
	 * Writes an image to file as JPEG encoded file
	 * 
	 * @param image The image to write
	 * @param destinationFile The destination file
	 * @throws AtacamException Throws when fails
	 */
	private void writeImageToFile(Image image, File destinationFile)
		throws AtacamaException {

		//Load it into a BufferedImage
		loadImage(image);

		int imageWidth = image.getWidth(this);
		int imageHeight = image.getHeight(this);
		
		BufferedImage bufferedImage = null;
		try {
			bufferedImage =
				new BufferedImage(
					imageWidth,
					imageHeight,
					BufferedImage.TYPE_INT_RGB);
		} catch(java.lang.IllegalArgumentException exception) {
			exception.printStackTrace();
		}

		Graphics2D imageGraphics = bufferedImage.createGraphics();
		imageGraphics.drawImage(image, 0, 0, this);

		//Use JPEGImageEncoder to write the BufferedImage to a file
		try {
			OutputStream outStream = new FileOutputStream(destinationFile);
			JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(outStream);

			//Set quality of jpeg
			JPEGEncodeParam jpegEncodeParam =
				encoder.getDefaultJPEGEncodeParam(bufferedImage);
			jpegEncodeParam.setQuality(
				theConfig.getFloat("photo.jpeg.quality"),
				true);
			encoder.setJPEGEncodeParam(jpegEncodeParam);

			encoder.encode(bufferedImage);
		} catch (IOException exception) {
			theLogger.logError(
				"Failed to write resized image to file : "
					+ destinationFile.getPath(),
				exception);
		}
	}

	/**
	 * Waits for an image to be rendered
	 * @param image The image
	 */
	private void loadImage(Image image) {
		try {
			MediaTracker tracker = new MediaTracker(this);
			tracker.addImage(image, 0);
			tracker.waitForID(0);
		} catch (Exception exception) {
			theLogger.logWarning(
				"Failed to load the image: " + exception.toString());
		}
	}
}