/*
 * Created on May 27, 2004
 *
 */
package com.arellano.ramon.atacama.system;

import java.util.Vector;

import com.arellano.ramon.atacama.config.Config;
import com.arellano.ramon.atacama.descriptor.DescriptorGenerator;
import com.arellano.ramon.atacama.descriptor.DescriptorReader;
import com.arellano.ramon.atacama.descriptor.DescriptorTester;
import com.arellano.ramon.atacama.entity.EntityTester;
import com.arellano.ramon.atacama.file.FileManager;
import com.arellano.ramon.atacama.file.FileTester;
import com.arellano.ramon.atacama.flickr.Synchronizer;
import com.arellano.ramon.atacama.html.HtmlGenerator;
import com.arellano.ramon.atacama.html.HtmlTester;
import com.arellano.ramon.atacama.log.Logger;
import com.arellano.ramon.atacama.photo.PhotoGenerator;

/**
 * @author ramon.arellano
 *
 * Class for running the application from command-line. Also gives allows
 * testing of the basic functionality of the application.
 */
public class Atacama {

	//The run modes
	private static final int GENERATE_GALLERY = 1;
	private static final int GENERATE_HTML = 3;
	private static final int GENERATE_PHOTOS = 4;
	private static final int GENERATE_DESCRIPTORS = 5;
	private static final int VALIDATE_DESCRIPTORS = 6;
	private static final int SYNC_FLICKR = 7;
	private static final int TEST = 8;

	//The logger, config and file manager
	private static Logger theLogger = null;
	private static Config theConfig = null;
	private static FileManager theFileManager = null;

	//The unique instance of this class
	private static Atacama atacama = null;

	//A vector of albums containing the gallery
	private static Vector gallery = null;

	
	/**
	 * The Constructor
	 * @param propertiesPath The path to the properties file
	 */
	private Atacama(String propertiesPath) {
		if (propertiesPath != null) {
			theConfig = Config.getInstance(propertiesPath);
		} else {
			theConfig = Config.getInstance();
		}
		theLogger = Logger.getInstance();
		theFileManager = FileManager.getInstance();
	}

	/**
	 * The main method of the Atacama application
	 * @param args The command-line arguments
	 */
	public static void main(String[] args) {

		//Must instantiate in order to load properties
		if (args.length == 1) {
			//Use default properties path
			atacama = new Atacama(null);
		} else if (args.length == 2) {
			atacama = new Atacama(args[1]);
		} else {
			//Wrong number of args
			usage();
		}

		try {
			//Run atacama according to run mode argument
			switch (atacama.getRunMode(args[0])) {
				case GENERATE_GALLERY :
					new HtmlGenerator().generate(atacama.getGallery());
					new PhotoGenerator().generate(atacama.getGallery());
					break;
				case GENERATE_HTML :
					new HtmlGenerator().generate(atacama.getGallery());
					break;
				case GENERATE_PHOTOS :
					new PhotoGenerator().generate(atacama.getGallery());
					break;
				case GENERATE_DESCRIPTORS :
					//Call descriptor generator with false validate flag
					//Descriptor will be generated
					new DescriptorGenerator().generate(false);
					break;
				case VALIDATE_DESCRIPTORS :
					//Call descriptor generator with false validate flag
					//Descriptor will be generated
					new DescriptorGenerator().generate(true);
					break;
				case SYNC_FLICKR :
					//Call Flickr class to synchronize against Flickr.com
					new Synchronizer().sync(false);
					break;
				case TEST :
					atacama.test();
					break;
			}
		} catch (AtacamaException exception) {
			theLogger.logWarning(
				"Atacama failed to run in mode: '" + args[0] + "'");
		}
	}

	/**
	 * Runs a series of test of the applications
	 * @throws AtacamaException Throws when something unexpected happens
	 */
	private void test() throws AtacamaException {

		EntityTester entityTester = new EntityTester();
		entityTester.testEntities();

		FileTester fileTester = new FileTester();
		fileTester.testFileSearch();
		fileTester.testDirSearch();
		fileTester.testFileReader();
		fileTester.testFileSubdirSearch();

		DescriptorTester descTester = new DescriptorTester();
		descTester.testPhotoDescriptorReader();
		descTester.testDescriptorCompare();
		descTester.testPassInfoToNewDescriptor();
		//descTester.testDescriptorGenerator();
		descTester.testPhotoDirectoryReader();
		descTester.testAlbumDescriptorReader();
		descTester.testPhotoDescriptorsReader();
		descTester.testDescriptorParser();

		HtmlTester htmlTester = new HtmlTester();
		htmlTester.testGenerate();

		//No test for photos, except actually running generation
		new PhotoGenerator().generate(atacama.getGallery());
	}

	/**
	 * Prints a simple manual to the standard output
	 */
	private static void usage() {
		String usageMessage =
			"Atacama Web Gallery Generator. Written by Ramon Kristian Arellano.\n\n"
				+ "Usage: atacama [run mode] [properties file path (optional)]\n\n"
				+ "Run mode options: \n"
				+ "gg : Generate gallery. Parses descriptors and generates both hmtl and photos.\n"
				+ "gh : Generate HTML\n"
				+ "gp : Generate photos\n"
				+ "gd : generate descriptors\n"
				+ "vd : validate descriptors\n"
				+ "sf : syncrhonize with Flickr\n"
				+ "t  : test\n";

		System.out.println(usageMessage);

		//If usage message is displayed, the program must exit
		System.exit(-1);
	}

	/**
	 * Reads all descriptors from filesystem and 
	 * returns a vector with album objects and corresponding photos
	 * 
	 * @return Vector of albums conataining complete gallery
	 */
	private Vector getGallery() throws AtacamaException {

		if (gallery == null) {
			DescriptorReader descReader = new DescriptorReader();
			gallery = descReader.readGallery();
		}

		return gallery;
	}

	/**
	 * Determines which run mode was given as an argument
	 * @param runModeString The run-mode string
	 * @return Run mode value
	 */
	private int getRunMode(String runModeString) {

		int runModeValue = 0;
		if (runModeString.toLowerCase().equals("gg")) {
			runModeValue = GENERATE_GALLERY;
		} else if (runModeString.toLowerCase().equals("gh")) {
			runModeValue = GENERATE_HTML;
		} else if (runModeString.toLowerCase().equals("gp")) {
			runModeValue = GENERATE_PHOTOS;
		} else if (runModeString.toLowerCase().equals("gd")) {
			runModeValue = GENERATE_DESCRIPTORS;
		} else if (runModeString.toLowerCase().equals("vd")) {
			runModeValue = VALIDATE_DESCRIPTORS;
		} else if (runModeString.toLowerCase().equals("sf")) {
			runModeValue = SYNC_FLICKR;
		} else if (runModeString.toLowerCase().equals("t")) {
			runModeValue = TEST;
		} else {
			//Run mode not recongnized
			usage();
		}

		return runModeValue;
	}
}