/*
 * Photo.java
 *
 *  Copyright (c) 2006, James G. Driscoll
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr;

import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.EnumMap;
import java.util.List;
import java.util.Map;
import java.util.PriorityQueue;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import org.jdom.Document;
import org.jdom.Element;

/**
 *
 * Encapsulates Photo information.  Includes a few static methods to search for photos.
 * Keeps a cache of the backing Images, adjustable in size by the user.  Photo objects
 * are typically created via a find method, via the search method, or, less commonly,
 * by supplying the id (and possibly the secret) in a constructor.
 *
 * @author Jim Driscoll <a href="mailto:jgd@jamesgdriscoll.com">jgd@jamesgdriscoll.com</a>
 */
public class Photo implements Comparable <Photo>{
 
    // Queue to store cache
    private static PriorityQueue<Photo> photoCache = new PriorityQueue<Photo>();
    // Current size of cache
    private static int cachesize = 0;
    // Size, in Megapixels of the cachesize
    private static final int cachesizelimit = 10 * 1024 * 1024;

    // Date image is put in cache
    private Date imageCached = new Date(0);
    // Image associated with this Photo
    private BufferedImage image = null;
    // Size for the image associated with this Photo
    private Size imageSize = null;
    
    // Map of valid sizes for this Photo
    private Map<Size,PhotoSize> sizes = new EnumMap(Size.class);
    
    private String id;
    private String secret;
    private String server;
    private String title;
    private Date postedDate;
    private Date updatedDate;
    private Date takenDate;
    private String takenDateGranularity;
    private URL photopage;
    private License license;
    
    // Permission information
    private boolean pub;
    private boolean friend;
    private boolean family;
    private boolean comment;
    private boolean meta;
    private User permUser = null;
    
    // Boolean to track if we've gotten the extra information associated with a Photo
    private boolean gotInfo = false;
    
    // Boolean to track if we've gotten information about permissions
    private boolean gotPerm = false;
    
    // Boolean to track if this object has been deleted
    private boolean deleted = false;
    
    private String description;
    private int numComments;
    
    // Used to read / write MySql datetime elements
    private final static DateFormat dateformat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    
    Photo(Element photo) {
        if (photo == null) throw new FlickrRuntimeException("Can't construct Photo from null");
        this.server = photo.getAttributeValue("server");
        this.id = photo.getAttributeValue("id");
        this.secret = photo.getAttributeValue("secret");
        this.title = photo.getAttributeValue("title");
    }
    
    /**
     * If you know the ID and the Secret of a photo, you can use it to create a new
     * Photo object.  Typically, you'll get new Photo objects by calling static methods,
     * but this constructor is useful when you know just the photo you want.
     * Note:  This call does not check with Flickr before the object is created.
     * @param id ID of the photo. Must not be null.
     * @param secret Secret string of the photo.  Needed for private photos.
     */
    public Photo(String id, String secret) {
        if (id == null) throw new NullPointerException("ID must not be null");
        this.id = id;
        this.secret = secret;
    }
    
    /**
     * Sometimes, all you know is the PhotoID - if you are authenticated, with
     * READ privlege on the photo in question (or if the Photo is public), this
     * is enough to get the Photo information.  Note:  This call does not check
     * with Flickr before the object is created.
     * @param id ID of the Photo. Must not be null.
     */
    public Photo(String id) {
        this(id,null);
    }
    
    /**
     * Get a photo, provided you have the Photo's id, and secret key.
     * @return The Photo object described by the key and secret.
     */
    public static Photo findByID(String id, String secret) throws FlickrException {
        if (id == null) throw new FlickrRuntimeException("Can't construct Photo from null id");
        if (secret == null) throw new FlickrRuntimeException("Can't construct Photo from null secret");
        Photo photo = new Photo(id,secret);
        photo.getInfo();
        return photo;
    }
    
    /**
     * Get a list of photos that are tagged with the desired keywords.
     * @param tags Comma separated list of tags
     * @throws FlicrException For any error.
     */
    public static List<Photo> findByTags(String tags) throws FlickrException {
        if (tags == null) throw new FlickrRuntimeException("Tags cannot be null");
        return findByTags(tags, true);
    }
    
    /**
     * Get a list of Photos that are tagged with the desired keywords.
     * @param tags Comma separated list of tags
     * @param join True joins the tags with AND, False joins the tags with OR
     * @throws FlickrException For any error.
     */
    public static List<Photo> findByTags(String tags, boolean join) throws FlickrException {
        if (tags == null) throw new FlickrRuntimeException("Tags cannot be null");
        
        Request req = new Request();
        req.setParameter("method","flickr.photos.search");
        req.setParameter("tags",tags);
        String joinStr = join?"all":"any";
        req.setParameter("tag_mode",joinStr);
        return new PhotoList(req);
    }
    
    /**
     * Get a list of Photos associated with the <code>PhotoSearch</code> term.
     * Unauthenticated calls return only public photos.  If authenticated at READ
     * level or above, private photos visible to the authenticated user can be
     * viewed as well.
     * @param search A PhotoSearch object describing the search to perform.
     * @return A list of Photos corresponding to the search term supplied.
     * @throws FlickrExcpetion For any error.
     */
    public static List<Photo> search(PhotoSearch search) throws FlickrException {
        if (search == null) throw new FlickrRuntimeException("Search term cannot be null");
        
        boolean termUsed = false;
        
        Request req = new Request();
        req.setParameter("method","flickr.photos.search");
        if (search.getUser() != null) {
            termUsed = true;
            req.setParameter("user_id",search.getUser().getID());
        }
        if (search.getTags() != null) {
            termUsed = true;
            req.setParameter("tags",search.getTags());
            if (search.getTagMode() != null) {
                if (search.getTagMode() == PhotoSearch.Tagmode.ALL) {
                    req.setParameter("tagmode","all");
                } else { // ANY
                    req.setParameter("tagmode","any");
                }
            } else { // Default is ANY
                req.setParameter("tagmode","any");
            }
        }
        if (search.getSearchText() != null) {
            termUsed = true;
            req.setParameter("text",search.getSearchText());
        }
        if (search.getPrivacy() != null) {
            termUsed = true;
            switch (search.getPrivacy()) {
                case PUBLIC:
                    req.setParameter("privacy_filter","1");
                    break;
                case FRIENDS:
                    req.setParameter("privacy_filter","2");
                    break;
                case FAMILY:
                    req.setParameter("privacy_filter","3");
                    break;
                case FRIENDSANDFAMILY:
                    req.setParameter("privacy_filter","4");
                    break;
                case PRIVATE:
                    req.setParameter("privacy_filter","5");
                    break;
            }
        }
        if (search.getMinUpload() != null) {
            termUsed = true;
            req.setParameter("min_upload_date",""+search.getMinUpload().getTime());
        }
        if (search.getMaxUpload() != null) {
            termUsed = true;
            req.setParameter("max_upload_date",""+search.getMaxUpload().getTime());
        }
        if (search.getMinTaken() != null) {
            termUsed = true;
            req.setParameter("min_taken_date",dateformat.format(search.getMinTaken()));
        }
        if (search.getMaxTaken() != null) {
            termUsed = true;
            req.setParameter("max_taken_date",dateformat.format(search.getMaxTaken()));
        }
        List<License> licenses = search.getLicenses();
        if (licenses != null && licenses.size() > 0) {
            termUsed = true;
            boolean first = true;
            String licenseSearch = "";
            for (License license : licenses) {
                if (first) {
                    first = false;
                } else {
                    licenseSearch += ",";
                }
                licenseSearch += license.getID();
            }
            req.setParameter("license",licenseSearch);
        }
        
        // Order-by
        // date-posted-asc, date-posted-desc, date-taken-asc, date-taken-desc, 
        // interestingness-desc, interestingness-asc, and relevance.
        if (search.getSort() != null) {
            termUsed = true;  // Not sure if this is true or not.  Spec says it is.
            switch (search.getSort()) {
                case POST_ASC:
                    req.setParameter("sort","date-posted-asc");
                    break;
                case POST_DESC:
                    req.setParameter("sort","date-posted-desc");
                    break;
                case TAKEN_ASC:
                    req.setParameter("sort","date-taken-asc");
                    break;
                case TAKEN_DESC:
                    req.setParameter("sort","date-taken-desc");
                    break;
                case INTERESTING_DESC:
                    req.setParameter("sort","interestingness-desc");
                    break;
                case INTERESTING_ASC:
                    req.setParameter("sort","interestingness-asc");
                    break;
                case RELEVANCE:
                    req.setParameter("sort","relevance");
                    break;
            }
        }
        
        if (!termUsed) throw new FlickrRuntimeException("Search object must contain a search condition");
        
        return new PhotoList(req);
    }
    
    /**
     * Gets a list of the most recent photos on Flickr.
     *
     * @return A <code>PhotoList</code> of Flickr's recent photos
     */
    public static List<Photo> getRecentPhotos() throws FlickrException {
        Request req = new Request();
        req.setParameter("method","flickr.photos.getRecent");
        return new PhotoList(req);
    }
    
    /**
     * Gets a list of Photos that Flickr users find interesting, for today.
     *
     * @return A <code>PhotoList</code> of Flickr's interesting photos.
     */
    public static List<Photo> getInteresting() throws FlickrException {
        Request req = new Request();
        req.setParameter("method","flickr.interestingness.getList");
        return new PhotoList(req);
    }
    
    /**
     * Gets a list of Photos that Flickr users find interesting, for today.
     *
     * @param cal - Used to specify what date to fetch interesting photos from.
     * @return A <code>PhotoList</code> of FJickrs interesting photos.
     */
    public static PhotoList getInteresting(Calendar cal) throws FlickrException {
        Request req = new Request();
        req.setParameter("method","flickr.interestingness.getList");
        String date = String.format("%1tY-%1tm-%1td",cal);
        req.setParameter("date",date);
        return new PhotoList(req);
    }
    
    /**
     * Create a new Photo on Flickr.  This photo will be created as a JPG.
     * Convenience method for longer call.  This method requires WRITE level
     * authentication.
     *
     * @param imageFile Image to use as the basis for the Photo.
     * @param title Title of the Photo.  May be null.
     * @return The Photo created.
     * @throws FlickrException on any error.
     */
    public static Photo create(File imageFile, String title) throws FlickrException {
        return create(imageFile, title, null, null, true, true, true);
    }
    
    /**
     *
     * Create a new Photo on Flickr.  This photo will be created as a JPG.  This
     * method requires WRITE level authentication.
     * @param imageFile Image to use as the basis for the Photo. Must be a JPEG formatted file.
     * @param title Title of the Photo.  May be null.
     * @param desc Description of the Photo.  May be null.
     * @param tags Space delimited list of tags for the Photo.
     * @param pub Whether this photo is publicly viewable.
     * @param friend Whether this photo is viewable by people marked as Friends.
     * @param family Whether this photo is viewable by people marked as Family.
     * @return Photo object created.
     * @throws FlickrException on any error.
     */
    public static Photo create(File imageFile, String title, String desc, String tags, boolean pub, boolean friend, boolean family) throws FlickrException {
        if (imageFile == null) throw new NullPointerException("Image cannot be null.");
        if (!imageFile.exists()) throw new IllegalArgumentException("Image File must exist");
        Request req = new Request(Request.FORM, "http://www.flickr.com/services/upload/");
        if (title != null) req.setParameter("title",title);
        if (desc != null) req.setParameter("description",desc);
        if (tags != null) req.setParameter("tags",tags);
        if (pub) {
            req.setParameter("is_public","1");
        } else {
            req.setParameter("is_public","0");
        }
        if (friend) {
            req.setParameter("is_friend","1");
        } else {
            req.setParameter("is_friend","0");
        }
        if (family) {
            req.setParameter("is_family","1");
        } else {
            req.setParameter("is_family","0");
        }
        req.setImage(imageFile);
        Element root = req.getResponse().getRootElement();
        String photoid = root.getChildText("photoid");
        if (photoid == null) throw new FlickrException("Bad XML response");
        return new Photo(photoid);
    }
    
    /**
     * Replaces the Photo on Flickr with the Image provided.  Image must be JPG (sorry).
     * This call requires WRITE level authentication.  This call requires that the
     * authenticated user be a Pro level account.
     * @param imageFile The new image to store for this Photo.
     * @throws FlickrException on any error.
     */
    public void replace(File imageFile) throws FlickrException {
        if (imageFile == null) throw new NullPointerException("Image cannot be null.");
        if (!imageFile.exists()) throw new IllegalArgumentException("Image File must exist");
        Request req = new Request(Request.FORM, "http://www.flickr.com/services/replace/");
        req.setParameter("photo_id",id);
        req.setImage(imageFile);
        req.getResponse();
        Element root = req.getResponse().getRootElement();
        String photoid = root.getChildText("photoid");
        if (photoid == null) throw new FlickrException("Bad XML response");
        return;
    }
    
    /**
     * Delete this photo from Flickr.  Not suprisingly, it requires DELETE level
     * authentication.  The object will be invalid after deletion, and no further
     * actions may be performed on it.
     * @throws FlickrException on any error.
     */
    public void delete() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photos.delete");
        req.setParameter("photo_id",id);
        req.getResponse();
        deleted = true;
    }
    
    /**
     * Get the static Flickr URL where you can find this photo. (i.e., the URL
     * that returns the actual image.) This call
     * returns the default sized photo.  To get the page you see in a browser, use
     * <code>PhotoSize.getPage()</code>.  To get different sizes TODO: ADD CODE SNIP
     *
     * <P><B>Please note:  Flickr requires  that if you display
     * an image from this Photo in a webpage, that you make it a link to the URL returned
     * from <code>Photo.getPhotoPage()</code>.  Please do the right thing.</B></P>
     *
     * @return URL of the default static link for this photo.
     * @see org.jickr.Photo#getPhotoPage
     */
    public URL getStaticURL() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        return getDefaultSize().getStaticURL();
    }
    
    /**
     * Get the Flickr URL of the page where you can find this photo. (i.e., the URL
     * that returns the page where this image is displayed.) This call
     * returns the default sized photo.
     *
     * @return URL of the default page for this photo.
     * @see org.jickr.Photo#getStaticURL
     */
    public URL getPhotoPage() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        if (photopage == null && !gotInfo) getInfo();
        return photopage;
    }
    
    /**
     * Get the title of the photo.
     * @return title - The title of the Photo.
     * @throws FlickrException on an access error to Flickr.
     */
    public String getTitle() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        if (title == null && !gotInfo) getInfo();
        return title;
    }
    
    /**
     * Get the secret key of the photo.
     * @return The secret key of the Photo.
     */
    public String getSecret() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        if (secret == null && !gotInfo) getInfo();
        return secret;
    }
    
    /**
     * Get the id of the photo.  You need this and the secret key to the retreive the Photo.
     * @return The id of the Photo.
     */
    public String getID() {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        return id;
    }
    
    /**
     * Get the description of the photo.
     * @return The Description Text for this Flickr Photo.
     */
    public String getDescription() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        if (description == null && !gotInfo) getInfo();
        return description;
    }
    
    /**
     * Get the license of the photo.
     * @return The license for this Flickr Photo.
     */
    public License getLicense() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        if (license == null && !gotInfo) getInfo();
        return license;
    }
    
    /**
     * Sets the license restrictions of this photo.
     * <P>Requires WRITE authentication and permission.
     * @param license License to use for this Photo.
     * @throws FlickrException on any error.
     */
    public void setLicense(License license) throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photos.licenses.setLicense");
        req.setParameter("photo_id",id);
        req.setParameter("license_id",license.getID());
        req.getResponse();
        this.license = license;
    }
    
    /**
     * Get the number of comments attached to this photo in Flickr.
     * This value is cached - it's only checked once.  If you expect the
     * value to change, use Photo.getComments.size() instead, though that
     * will be more expensive.
     * @see Photo#getComments()
     * @return The total number of comments associated with this <code>Photo</code>
     */
    public int getNumComments() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        if (!gotInfo) getInfo();
        return numComments;
    }
    
    /**
     * Get the date the photo was uploaded to Flickr.
     * @return The date this photo was uploaded.
     */
    public Date getDateUploaded() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        if (!gotInfo) getInfo();
        return postedDate;
    }
    
    /**
     * Get the date the photo was last updated.  This value is cached, and not
     * updated for the life of the object.
     * @return The date this photo was last updated on Flickr.
     */
    public Date getDateUpdated() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        if (!gotInfo) getInfo();
        return updatedDate;
    }
    
    /**
     * Get the date the photo was taken.
     * @return The date this photo was taken.
     */
    public Date getDateTaken() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        if (!gotInfo) getInfo();
        return takenDate;
    }
    
    /**
     * <P>Get the granularity (i.e., the precision) of the date the photo was
     * taken.</P>
     * <P>Currently has three values: 0: granularity to the second, 4: granularity
     * to the month, and 6: granularity to the year.  Later updates to Flickr may
     * return any value between 0 and 10.</P>
     * <P>Currently returned as a String.  This may change to an int at some time
     * in the future, if there's a need.</P>
     * @return Code indicating the precision of the date taken.
     * @throws FlickrException on any error.
     */
    public String getDateTakenGranularity() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        if (!gotInfo) getInfo();
        return takenDateGranularity;
    }
    
    /**
     * Get a BufferedImage of this photo (default size).  This response is
     * cached, as in Photo.getImage(Size).
     * @see Photo#getImage(Size)
     * @return image Image of this photo.
     */
    public BufferedImage getImage() throws IOException, FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        return getImage(getDefaultSize().getSize());
    }
    
    /**
     * Get a BufferedImage of this photo.  This response is cached - the cache
     * is currently limited to a single size per Photo object.
     * @param size - Size of the image to fetch.
     * @return image Image of this photo.
     */
    public BufferedImage getImage(Size size) throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        if (image != null  && imageSize == size) return image;
        // TODO add the ability to cache more than one size at a time
        if (image != null && imageSize != size) dropImage(); // Only cache one size at a time, sorry
        initSizes();
        if (!sizes.containsKey(size)) throw new FlickrException("This photo doesn't have the size: "+size);
        try {
            image = ImageIO.read(sizes.get(size).getStaticURL());
        } catch (IllegalArgumentException iae) {
            // There's a bug in the ImageIO class - it throws Illegal Arg on bad image, instead of IOException
            throw new FlickrException("Bad Image Data: Photo id="+id,iae);
        } catch (IOException ioex) {
            // Since it throws IAE anyway, lets wrap the IO exception too
            throw new FlickrException("Error reading image: Photo id="+id,ioex);
        }
        cachesize = cachesize + (image.getHeight() * image.getWidth());
        while (cachesize > cachesizelimit && photoCache.size() > 0) {
            photoCache.poll().dropImage();
        }
        photoCache.add(this);
        imageCached = new Date();
        return image;
    }
    
    /**
     * Drop the cached Image from this photo.  Will be garbage collected.
     */
    private void dropImage() {
        Logger.global.finest("Dropping Photo From Cache:"+this);
        cachesize = cachesize - (image.getHeight() * image.getWidth());
        Logger.global.finest("CacheSize = "+cachesize);
        image = null;
        imageCached = new Date(0);
    }
    
    /**
     * Get the EXIF data for this photo.  This is returned as a JDOM Document, 
     * since it can vary so very much.  If you have better suggestions for how to
     * handle this call, please tell me at 
     * <a href="mailto:dev@jickr.dev.java.net">dev@jickr.dev.java.net</a>.
     *<p>
     * This call is not cached, and goes out to Flickr every time to check data.
     *
     * @return A Document containing all EXIF data.
     * @throws FlickrException on any error.
     */
    public Document getEXIF() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request();
        req.setParameter("method","flickr.photos.getExif");
        req.setParameter("photo_id",id);
        if (secret != null) req.setParameter("secret",secret);
        return req.getResponse();
    }
    
    /**
     * Get a list of comments for this photo.
     * @return A list of Comment objects for this Photo.
     * @throws FlickrException in the event of any error.
     */
    public List<Comment> getComments() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        List<Comment> commentList = new ArrayList();
        Request req = new Request();
        req.setParameter("method","flickr.photos.comments.getList");
        req.setParameter("photo_id",id);
        
        Document doc = req.getResponse();
        Element root = doc.getRootElement();
        
        List<Element> comments = root.getChild("comments").getChildren("comment");
        for (Element comment: comments) {
            commentList.add(new Comment(Comment.Type.PHOTOCOMMENT,comment));
        }
        return commentList;
    }
    
    /**
     * Get a Map of all valid PhotoSize values for this Photo.
     * @return A Map of all the different Sizes for this Photo.
     * @throws FlickrException on any error.
     */
    public Map<Size,PhotoSize> getSizes() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        this.initSizes();
        return new EnumMap<Size,PhotoSize>(sizes);
    }
    
    /**
     * <P>Adds a comment to this Photo, returns the new comment ID.  Since Flickr
     * doesn't offer a way to search for a comment via ID, the user will
     * have to do a <code>getComments()</code> and search for the comment with
     * <code>Comment.getID()</code> if they want a Comment object representing this
     * added comment.</P>
     *
     * <P>This call requires that the user be authenticated at the WRITE level.</P>
     *
     * @see org.jickr.Photo#getComments()
     * @see org.jickr.Comment#getID()
     * @param commentText Text for the comment to add.
     * @return The comment's ID string
     * @throws FlickrException in the event of a failure.  Note that if an exception is thrown, it's possible (thought unlikely) that a comment was added anyway.
     */
    public String addComment(String commentText) throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        if (commentText == null) throw new NullPointerException("Comment Text may not be null");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photos.comments.addComment");
        req.setParameter("photo_id",id);
        req.setParameter("comment_text",commentText);
        
        Element root = req.getResponse().getRootElement();
        
        try {
            String commentID = root.getChild("comment").getAttributeValue("id");
            // Sigh.  Flickr doesn't let you get a comment via commentID
            // So instead, we'll have just return the ID.
            // The developer can search for it if they wanna.
            return commentID;
        } catch (NullPointerException npe) {
            throw new FlickrException("Oddly formed XML in response",npe);
        }
    }
    
    /**
     *  Add this photo as favorite to the authenticated user.
     *  @throws FlickrException on any error.
     */
    public void addFavorite() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.favorites.add");
        req.setParameter("photo_id",id);
        req.getResponse(); // Empty Response except for status
    }
    
    /**
     * Remove this photo from the authenticated user's list of favorites.
     * @throws FlickrException on any error.
     */
    public void removeFavorite() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.favorites.remove");
        req.setParameter("photo_id",id);
        req.getResponse(); // Empty Response except for status
    }
    
    /**
     * Get a list of all public PhotoSets this Photo belongs to.  This value is
     * not cached - it returns the current list of PhotoSets with every call.
     *
     * @return A list of PhotoSet objects this Photo is associated with.  Returns an empty list if none found.
     * @throws FlickrException on any error.
     */
    public List<PhotoSet> getPublicSets() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request();
        req.setParameter("method","flickr.photos.getAllContexts");
        req.setParameter("photo_id",id);
        Document doc = req.getResponse();
        Element root = doc.getRootElement();
        
        List<Element> sets = root.getChildren("set");
        List<PhotoSet> setlist = new ArrayList<PhotoSet>();
        if (sets.size() == 0) return setlist;
        for (Element set: sets) {
            String setID = set.getAttributeValue("id");
            if (setID == null) throw new FlickrException("Malformed XML - cannot get ID");
            setlist.add(new PhotoSet(setID));
        }
        return setlist;
    }
    
    /**
     * Get a list of all public Groups this Photo belongs to.  This value is
     * not cached - it returns the current list of Groups with every call.
     *
     * @return A list of Group objects this Photo is associated with.  Returns an empty list if none found.
     * @throws FlickrException on any error.
     */
    public List<Group> getPublicGroups() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request();
        req.setParameter("method","flickr.photos.getAllContexts");
        req.setParameter("photo_id",id);
        Document doc = req.getResponse();
        Element root = doc.getRootElement();
        
        List<Element> groups = root.getChildren("pool");
        List<Group> grouplist = new ArrayList<Group>();
        if (groups.size() == 0) return grouplist;
        for (Element group: groups) {
            String groupID = group.getAttributeValue("id");
            if (groupID == null) throw new FlickrException("Malformed XML - cannot get ID");
            grouplist.add(new Group(groupID));
        }
        return grouplist;
    }
    
    
    /**
     * Get a list of tags for this Photo.  This value is not cached - it returns
     * the current list of tags with every call.
     *
     * @return A list of Tag objects associated with this Photo.
     * @throws FlickrException on any error.
     */
    public List<Tag> getTags() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request();
        req.setParameter("method","flickr.photos.getInfo");
        req.setParameter("photo_id",id);
        if (secret != null) req.setParameter("secret",secret);
        Document doc = req.getResponse();
        Element root = doc.getRootElement();
        
        try {
            Element tags = root.getChild("photo").getChild("tags");
            List<Tag> taglist = new ArrayList<Tag>();
            if (tags == null) return taglist;
            List<Element> tagElements = tags.getChildren("tag");
            for (Element tag: tagElements) {
                taglist.add(new Tag(this,tag));
            }
            return taglist;
        } catch (NullPointerException npe) {
            throw new FlickrException("Malformed XML",npe);
        }
    }
    
    /**
     * Adds tags to this Photo.  How the parameter is treated is up to Flickr -
     * for instance, Flickr's documentation says that this is a "comma-delimited list",
     * but spaces seem to delimit new tags as well, such that "New Test Tag" creates
     * 3 new tags, called "New" "Test" and "Tag".
     * @param tags A comma delimited list of tags to add.
     * @throws FlickrException on any error.
     */
    public void addTags(String tags) throws FlickrException {
        if (tags == null) throw new NullPointerException("tags must not be null");
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photos.addTags");
        req.setParameter("photo_id",id);
        req.setParameter("tags",tags);
        req.getResponse();
        return;
    }
    
    /**
     * Replace all tags on the Photo with the specified tags.  Requires WRITE level
     * authenticated access.
     * @param tags A comma delimited list of tags.
     * @throws FlickrException on any error.
     */
    public void setTags(String tags) throws FlickrException {
        if (tags == null) throw new NullPointerException("tags must not be null");
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photos.setTags");
        req.setParameter("photo_id",id);
        req.setParameter("tags",tags);
        req.getResponse();
        return;
    }
    
    /**
     * Get a list of notes for this Photo.  This value is not cached - it returns
     * the current list of notes with every call.
     *
     * @return A list of Note objects associated with this Photo.
     * @throws FlickrException on any error.
     */
    public List<Note> getNotes() throws FlickrException {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request();
        req.setParameter("method","flickr.photos.getInfo");
        req.setParameter("photo_id",id);
        if (secret != null) req.setParameter("secret",secret);
        Element root = req.getResponse().getRootElement();
        
        try {
            Element notes = root.getChild("photo").getChild("notes");
            List<Note> notelist = new ArrayList<Note>();
            if (notes == null) return notelist;
            List<Element> noteElements = notes.getChildren("note");
            for (Element note: noteElements) {
                notelist.add(new Note(note));
            }
            return notelist;
        } catch (NullPointerException npe) {
            throw new FlickrException("Malformed XML",npe);
        }
    }
    
    /**
     * Add a Note to a Photo.  Coordinates for the area are in pixels,
     * based on the 500px image size shown on individual photo pages.
     * @param area Rectangle describing the area covered by the note. Must not be null.
     * @param text Text of the note.  Must not be null.
     * @return The id of the Note created.  You'll have to look it up if you want a Note object.  This is done for performance.
     * @throws FlickrException on any error.
     */
    public String addNote(Rectangle area, String text) throws FlickrException {
        if (area == null) throw new NullPointerException("Area cannot be null");
        if (text == null) throw new NullPointerException("Text cannot be null");
        
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photos.notes.add");
        req.setParameter("photo_id",id);
        req.setParameter("note_x",""+area.x);
        req.setParameter("note_y",""+area.y);
        req.setParameter("note_w",""+area.width);
        req.setParameter("note_h",""+area.height);
        req.setParameter("note_text",text);
        Element root = req.getResponse().getRootElement();
        try {
            String id = root.getChild("note").getAttributeValue("id");
            if (id == null) throw new FlickrException("Malformed XML error - id is not set");
            return id;
        } catch (NullPointerException npe) {
            throw new FlickrException("Malformed XML error",npe);
        }
    }
    
    /**
     * Change the title text and description text associated with a Photo.
     * Requires WRITE level authentication.
     * @param title The new title for the photo. May not be null.
     * @param desc The new description for the photo. May not be null.
     * @throws FlickrException on any error.
     */
    public void updateTitleDesc(String title, String desc) throws FlickrException {
        if (title == null || desc == null) throw new NullPointerException("Text and description cannot be null");
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photos.setMeta");
        req.setParameter("photo_id",id);
        req.setParameter("title",title);
        req.setParameter("description",desc);
        req.getResponse();
        this.title = title;
        this.description = description;
        return;
    }
    
    /**
     * Update the date Flickr lists as the date the Photo was uploaded.
     * Requires WRITE level authentication.
     * @param dateUploaded The new date to list as the date uploaded.  May not be null.
     * @throws FlickrException on any error.
     */
    public void setDateUploaded(Date dateUploaded) throws FlickrException {
        if (dateUploaded == null) throw new NullPointerException("Date Updated cannot be null");
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photos.setDates");
        req.setParameter("photo_id",id);
        req.setParameter("date_posted",""+(dateUploaded.getTime()/1000L));
        req.getResponse();
        this.postedDate = dateUploaded;
        return;
    }
    
    /**
     * Update the date Flickr lists as the date taken for the Photo.
     * Requires WRITE level authentication.
     * @param dateTaken The new date to list as the date uploaded.  May not be null.
     * @throws FlickrException on any error.
     */
    public void setDateTaken(Date dateTaken) throws FlickrException {
        if (dateTaken == null) throw new NullPointerException("Date Taken cannot be null");
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photos.setDates");
        req.setParameter("photo_id",id);
        req.setParameter("date_taken",""+dateformat.format(dateTaken));
        req.getResponse();
        this.takenDate = dateTaken;
        return;
    }
    
    /**
     * Update the date Flickr lists as the date taken for the Photo.
     * Requires WRITE level authentication.  Granularity may be one of: 0: precision to the second,
     * 4: precision to the month, 6: precision to the year.
     * @param granularity The precision of the date taken on Flickr.  May not be null. Must be one of 0, 4 or 6.
     * @throws FlickrException on any error.
     */
    public void setDateTakenGranularity(String granularity) throws FlickrException {
        if (granularity == null) throw new NullPointerException("Granularity cannot be null");
        if (granularity != "0" && granularity != "4" && granularity != "6") throw new IllegalArgumentException("Granularity must be one of 0, 4 or 6");
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photos.setDates");
        req.setParameter("photo_id",id);
        req.setParameter("date_taken_granularity",granularity);
        req.getResponse();
        this.takenDateGranularity = granularity;
        return;
    }
    
    /**
     * <P>Rotate the Photo on Flickr.</P>
     *
     * <P>Requires authentication with WRITE access.</P>
     *
     * <P>NOTE:  This seems to be a <b>very</b> expensive call.</P>
     *
     * @param amount The amount, in degrees, to rotate the photo.  Must be one of 90, 180, 270.
     * @throws FlickrException on any error.
     */
    public void rotate(int amount) throws FlickrException {
        if (amount != 90 && amount != 180 && amount != 270) throw new IllegalArgumentException("Amount must be one of 90, 180 or 270");
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photos.transform.rotate");
        req.setParameter("photo_id",id);
        req.setParameter("degrees",""+amount);
        req.getResponse();
    }
    
    /**
     *  Return whether the photo is publicly visible.
     *  <P>Requires authentication with read access.
     * @return Whether a photo is public.
     * @throws FlickrException on any error.
     */
    public boolean isPublic() throws FlickrException {
        if (gotPerm && permUser == Auth.getAuthContext()) {
            return pub;
        }
        getPerm();
        return pub;
    }
    
    /**
     * Returns whether the photo is visible to people marked as
     * friends of the authenticated user.
     * <P>Requires authentication with READ access.
     * @return Whether a photo is visible to people marked as friends.
     * @throws FlickrException on any error.
     */
    public boolean isFriend() throws FlickrException {
        if (gotPerm && permUser == Auth.getAuthContext()) {
            return friend;
        }
        getPerm();
        return friend;
        
    }
    
    /**
     * Returns whether the photo is visible to people marked as
     * family of the authenticated user.
     * <P>Requires authentication with READ access.
     * @return Whether a photo is visible to people marked as family.
     * @throws FlickrException on any error.
     */
    public boolean isFamily() throws FlickrException {
        if (gotPerm && permUser == Auth.getAuthContext()) {
            return family;
        }
        getPerm();
        return family;
        
    }
    
    /**
     * Returns whether someone with the authenticated user's permissions
     * can comment on the photo.
     * <P>Requires authentication with READ access.
     * @return Whether a photo can have comments added to it by the authenticated user.
     * @throws FlickrException on any error.
     */
    public boolean canComment() throws FlickrException {
        if (gotPerm && permUser == Auth.getAuthContext()) {
            return comment;
        }
        getPerm();
        return comment;
        
    }
    
    /**
     * Returns whether someone with the authenticated user's permissions
     * can add metadata to the photo.  Metadata includes Notes and Tags.
     * <P>Requires authentication with READ access.
     * @return Whether a photo can have metadata added to it by the authenticated user.
     * @throws FlickrException on any error.
     */
    public boolean canAddMeta() throws FlickrException {
        if (gotPerm && permUser == Auth.getAuthContext()) {
            return meta;
        }
        getPerm();
        return meta;
    }
    
    /**
     * Set permissions on a Photo.
     * <P>Requires authentication with WRITE access.
     * @param pub Is the public allowed to view the photo?
     * @param friends Are friends allowed to view the photo?
     * @param family Are family allowed to view the photo?
     * @param comment Who can comment on the photo?
     * @param addmeta Who can add Notes and Tags to a photo?
     * @throws FlickrException on any error.
     */
    public void setPerms(boolean pub, boolean friends, boolean family,
            Allowed comment, Allowed addmeta) throws FlickrException {
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photos.setPerms");
        req.setParameter("photo_id",id);
        req.setParameter("is_public",pub?"1":"0");
        req.setParameter("is_friend",friend?"1":"0");
        req.setParameter("is_family",family?"1":"0");
        String c = null;
        switch (comment) {
            case NOBODY: c = "0";
            case FRIENDSANDFAMILY: c = "1";
            case CONTACTS: c = "2";
            case EVERYONE: c = "3";
        }
        req.setParameter("perm_comment",c);
        switch (addmeta) {
            case NOBODY: c = "0";
            case FRIENDSANDFAMILY: c = "1";
            case CONTACTS: c = "2";
            case EVERYONE: c = "3";
        }
        req.setParameter("perm_addmeta",c);
        req.getResponse();
    }
    
    /**
     * Load all permission data for the photo.
     */
    private void getPerm() throws FlickrException {
        Request req = new Request();
        req.setParameter("method","flickr.photos.getPerms");
        req.setParameter("photo_id",id);
        Element root = req.getResponse().getRootElement();
        Element perms = root.getChild("perms");
        try {
            this.pub = (perms.getAttributeValue("ispublic").equals("1"));
            this.friend = (perms.getAttributeValue("isfriend").equals("1"));
            this.family = (perms.getAttributeValue("isfamily").equals("1"));
            this.comment = (perms.getAttributeValue("permcomment").equals("1"));
            this.meta = (perms.getAttributeValue("permaddmeta").equals("1"));
            this.permUser = Auth.getAuthContext();
        } catch (NullPointerException npe) {
            throw new FlickrException("Malformed XML Error",npe);
        }
        
    }
    
    private void getInfo() throws FlickrException {
        Request req = new Request();
        req.setParameter("method","flickr.photos.getInfo");
        req.setParameter("photo_id",id);
        if (secret != null) req.setParameter("secret",secret);
        Document doc = req.getResponse();
        Element root = doc.getRootElement();
        
        try {
            this.id = root.getChild("photo").getAttributeValue("id");
            this.secret = root.getChild("photo").getAttributeValue("secret");
            this.server = root.getChild("photo").getAttributeValue("server");
            this.license = License.findByID(root.getChild("photo").getAttributeValue("license"));
            String posted = root.getChild("photo").getChild("dates").getAttributeValue("posted");
            long postedTime = Long.parseLong(posted) * 1000L;
            this.postedDate = new Date(postedTime);
            String updated = root.getChild("photo").getChild("dates").getAttributeValue("lastupdate");
            long updatedTime = Long.parseLong(updated) * 1000L;
            this.updatedDate = new Date(updatedTime);
            this.takenDate = dateformat.parse(root.getChild("photo").getChild("dates").getAttributeValue("taken"));
            this.takenDateGranularity = root.getChild("photo").getChild("dates").getAttributeValue("takengranularity");
            this.title = root.getChild("photo").getChildText("title");
            this.description = root.getChild("photo").getChildText("description");
            this.numComments = Integer.parseInt(root.getChild("photo").getChildText("comments"));
            this.photopage = new URL(root.getChild("photo").getChild("urls").getChildText("url"));
        } catch (NullPointerException npe) {
            throw new FlickrException("Malformed XML error",npe);
        } catch (ParseException pe) {
            throw new FlickrException("Malformed XML error",pe);
        } catch (MalformedURLException mue) {
            throw new FlickrException("Malformed URL error",mue);
        }
        
        gotInfo = true;
    }
    
    /**
     * Get the String representation of the URL for this photo.
     * @return String representing this Photo.  Using the URL for this purpose.
     */
    public String toString() {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        try {
            return getDefaultSize().URL.toString();
        } catch (FlickrException ex) {
            return "Error getting String for photo:"+id;
        }
    }
    
    /**
     * Determine of a Photo is equal to another.
     * @return Whether a Photo object represents the same photo
     */
    public boolean equals(Object o) {
        if (!(o instanceof Photo)) return false;
        if (o == null) return false;
        Photo p = (Photo)o;
        return this.getID().equals(p.getID());
    }
    
    /**
     * Get the hashcode for this object.
     */
    public int hashCode() {
        return this.getID().hashCode();
    }
    
    /**
     * Compares a Photo's image.  Uses the Date the Photo's image object was
     * created to compare.
     * (Note: Not the date of the photo, but the creatation date and time of the
     * Java Object).  Return is the same as for java.util.Date.
     *
     * TODO:
     * Yes, this doesn't actually do what you'd expect.  I need to fix this at some
     * point to return something a little more rational, or hide it.  In particular,
     * equals uses a different equality metric, so I've got to deal with that at some point.
     *
     * @return comparison <1 if Photo newer than supplied photo, 0 if the same, 1 if older.
     */
    public int compareTo(Photo p) {
        if (deleted) throw new IllegalStateException("No actions may performed on a deleted Photo");
        // TODO fix this to match equals!  Right now, may give problems
        return this.imageCached.compareTo(p.imageCached);
    }
    
    private void initSizes() throws FlickrException {
        // Only run once.
        if (sizes.size() != 0 ) return;
        
        Request req = new Request();
        req.setParameter("method","flickr.photos.getSizes");
        req.setParameter("photo_id",id);
        
        Document doc = req.getResponse();
        Element root = doc.getRootElement();
        
        List<Element> photosizes = root.getChild("sizes").getChildren("size");
        
        for (Element photosize : photosizes)  {
            PhotoSize ps = new PhotoSize(photosize);
            sizes.put(ps.getSize(),ps);
        }
    }
    
    /**
     * Gets the default PhotoSize for this item.  Will return Medium sized photos,
     * marching downward until we find something.
     * @return The default PhotoSize.
     */
    private PhotoSize getDefaultSize() throws FlickrException {
        initSizes();
        if (sizes.containsKey(Size.MEDIUM)) {
            return sizes.get(Size.MEDIUM);
        } else if (sizes.containsKey(Size.SMALL)) {
            return sizes.get(Size.SMALL);
        } else if (sizes.containsKey(Size.SQUARE)) {
            return sizes.get(Size.SQUARE);
        } else if (sizes.containsKey(Size.THUMB)) {
            return sizes.get(Size.THUMB);
        } else if (sizes.containsKey(Size.LARGE)) {
            return sizes.get(Size.LARGE);
        } else if (sizes.containsKey(Size.ORIGINAL)) {
            // Now we're desparate.  Wonder if this will ever happen?
            return sizes.get(Size.ORIGINAL);
        } else {
            throw new FlickrRuntimeException("Can't find a Default PhotoSize");
        }
    }
       
    /**
     * List of possibly valid Photo sizes.  Actual sizes available may be smaller.
     */
    public enum Size {
        /**
         * Square small photo -   Typically 75x75 square. JPG format.
         */
        SQUARE,
        /**
         * Thumbnail size photo -  Typically 100x67 (Landscape). JPG format.
         */
        THUMB,
        /**
         * Small photo - 240x160 (Landscape). JPG format.
         */
        SMALL,
        /**
         * Medium photo - 500x333 (Landscape).  Typically the default sized photo. JPG format.
         */
        MEDIUM,
        /**
         * Large photo -  typically 1024x683.  JPG format - largest guarenteed JPG.
         */
        LARGE,
        /**
         * Original photo, same as uploaded. In the original size, in the orginal format (GIF, PNG, JPG, TIF).
         */
        ORIGINAL }
    
    /**
     * List of possible permissions allowed for a Photo.  Used to set permissions in <code>Photo.setPerms()</code>
     * @see Photo#setPerms(boolean,boolean,boolean,Allowed,Allowed)
     */
    public enum Allowed {
        /**
         * Allow noone to set this data.
         */
        NOBODY,
        /**
         * Allow friends and family to set this data.
         */
        FRIENDSANDFAMILY,
        /**
         * Allow contacts to set this data.
         */
        CONTACTS,
        /**
         * Allow the public to set this data.
         */
        EVERYONE }
    
    /**
     * PhotoSize encapsulates information for a certain size of a photo.  It's
     * intended that many of these will be associated with a single instance of Photo.
     *
     * @see org.jickr.Photo
     * @author driscoll
     */
    public class PhotoSize {
        
        private Size size ;
        private String sizeString;
        private int width;
        private int height;
        private URL URL;
        private URL pageURL;
        
        /**
         * Creates a new instance of PhotoSize.
         *
         * @throws FlickrRuntimeException - if it receives a size that it doesn't
         * recognize. If this happens, it means Flickr changed it's formats, and it's
         * time to update the API.
         */
        PhotoSize(Element sizeElement) throws FlickrRuntimeException {
            try {
                sizeString = sizeElement.getAttributeValue("label");
                if (sizeString.equals("Thumbnail")) {
                    size = Size.THUMB;
                } else if (sizeString.equals("Square")) {
                    size = Size.SQUARE;
                } else if (sizeString.equals("Small")) {
                    size = Size.SMALL;
                } else if (sizeString.equals("Medium")) {
                    size = Size.MEDIUM;
                } else if (sizeString.equals("Large")) {
                    size = Size.LARGE;
                } else if (sizeString.equals("Original")) {
                    size = Size.ORIGINAL;
                } else {
                    throw new FlickrRuntimeException("Unknown Size recieved: "+sizeString);
                }
                this.width = Integer.parseInt(sizeElement.getAttributeValue("width"));
                this.height = Integer.parseInt(sizeElement.getAttributeValue("height"));
                try {
                    this.URL = new URL(sizeElement.getAttributeValue("source"));
                    this.pageURL = new URL(sizeElement.getAttributeValue("url"));
                } catch (MalformedURLException ex) {
                    // This would be suprising, but might as well throw it.
                    throw new FlickrRuntimeException("Malformed URL Exception",ex);
                }
            } catch (NullPointerException npe) {
                throw new FlickrRuntimeException("Malformed XML recieved from Flickr",npe);
            }
        }
        
        /**
         * Gets the size of the Photo.
         * @return The size of the Photo in this object.
         */
        public Size getSize() {
            return size;
        }
        
        /**
         * Gets the width of the image associated with the PhotoSize.
         * @return The width in pixels.
         */
        public int getWidth() {
            return width;
        }
        
        /**
         * Gets the height of the image associated with the PhotoSize.
         * @return The height in pixels.
         */
        public int getHeight() {
            return height;
        }
        
        /**
         * Gets the static URL of the image for this PhotoSize.
         * @return The URL of the static reference for the image of this PhotoSize.
         */
        public URL getStaticURL() {
            return URL;
        }
        
        /**
         * Gets the URL of the Page for this PhotoSize at Flickr.  This is the
         * page that you would want to call up in a browser, for instance, or send
         * in an email.  Note this is different than the page returned by
         * <code>Photo.getPhotoPage()</code>
         *
         * @see Photo#getPhotoPage()
         * @return The URL of the page containing the Photo of this size.
         */
        public URL getPage() {
            return pageURL;
        }
        
    }
}