/*
 * Request.java
 *
 *  Copyright (c) 2006, James G. Driscoll 
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;
import java.util.logging.Logger;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

/**
 * A class for encapsulating a request to Flickr.  This is an implementation class, not
 * intended for general use.
 * @author Jim Driscoll <a href="mailto:jgd@jamesgdriscoll.com">jgd@jamesgdriscoll.com</a>
 */
class Request {
    
    /**
     * Use to signal a GET request.  Used for retrieving data from Flickr.
     */
    static final int GET = 0;
    /**
     * Use to signal a POST request. Used for changing data on Flickr.
     */
    static final int POST = 1;
    /**
     * Use to signal a FORM request.  Used to send files.
     */
    static final int FORM = 2;
    
    private TreeMap<String,String> parameters = new TreeMap<String,String>();
    private static boolean signed = false;
    private int type;
    private File image = null;
    
    private String base = Flickr.getBase();
    
    /**
     * Creates a new Authenticated request, type GET.
     * @param user The user to authenticate the call as.
     */
    Request() throws FlickrException {
        this(GET);
    }
    
    /**
     * Creates a new instance of request with the specified method.
     * Adds authentication from the thread's AuthContext.
     * @param type - one of GET or POST
     */
    Request(int type) throws FlickrException {
        if (type != GET && type != POST) throw new IllegalArgumentException("type not allowed");
        parameters.put("api_key",Flickr.getApiKey());
        User user = Auth.getAuthContext();
        if (user != null) parameters.put("auth_token",Auth.getToken(user));
    }
    
    /**
     * Create a request with an unusual URL as the base.  Used for authentication.
     * A shorthand way to generate a signed call.  NOTE: No token is put on this
     * Request - the only constructor where this is the case...
     * @param customBase - An unusual URL to use for the request.  Used for Authentication.
     */
    Request(String customBase) throws FlickrException {
        this.type = GET;
        base = customBase;
        parameters.put("api_key",Flickr.getApiKey());
    }
    
    /** Creates a new instance of request with a custom call base.
     *  Used for authentication pre-calls.  A shorthand way to generate a signed call.
     * @param type - one of GET, POST or FORM
     * @param customBase - An unusual URL to use for the request.  Used for Authentication and Upload.
     */
    Request(int type, String customBase) throws FlickrException {
        this.type = type;
        base = customBase;
        parameters.put("api_key",Flickr.getApiKey());
        User user = Auth.getAuthContext();
        if (user != null) parameters.put("auth_token",Auth.getToken(user));
    }
    
    void setParameter(String parameter, String value) {
        parameters.put(parameter, value);
    }
    
    void setImage(File image) {
        this.image = image;
    }
    
    /**
     * Return a Map of the parameters for this request.  This map is a copy -
     * to set parameters, use the setParameter method.
     * @return A copy of all the parameters set for this request.
     */
    Map<String,String> getParameters() {
        return new TreeMap<String,String>(parameters);
    }
    
    /**
     * Get the signature for this request, based on the current list of parameters.
     * @return signature of this request.
     */
    private String getSig(String params) {
        String signString = Flickr.getSharedSecret() + params;
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            return hexStringFromBytes(md.digest(signString.getBytes()));
        } catch (NoSuchAlgorithmException ex) {
            // There's no fixin' this, so let's just give up
            throw new Error("Missing MD5 Algorithm",ex);
        }
    }
    
    // Found this algorithm on the web, adapted it to java
    private String hexStringFromBytes(byte[] b) {
        char[] hexChars ={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
        String hex = "";
        int big;
        int little = 0;
        int i;
        // MSB maps to idx 0
        for (i = 0; i < b.length; i++) {
            big = ((int)b[i] & 0x000000FF) / 16;
            little = ((int)b[i] & 0x000000FF) % 16;
            hex = hex + hexChars[big] + hexChars[little];
        }
        return(hex);
    }
    
    /**
     * Get the URL for this request.
     * @return The URL in string form.
     */
    String getURL() {
        // The Request URL we'll actually use for the Flickr request
        String requestURL = base;
        if (type == GET) {
            requestURL += "?"+getParams();
        }
        return requestURL;
    }
    
    // Used for GET and POST requests
    private String getParams() {
       String requestParam = "";
        // string to compute the signature
        String paramString = "";
        boolean first = true;
        for (Map.Entry<String,String>entry : parameters.entrySet()) {
            paramString += entry.getKey()+entry.getValue();
            try {
                if (first) {
                    first = false;
                } else {
                    requestParam += "&";
                }
                requestParam += URLEncoder.encode(entry.getKey(),"UTF-8")+"="+
                        URLEncoder.encode(entry.getValue(),"UTF-8");
            } catch (UnsupportedEncodingException ex) {
                ex.printStackTrace();
                // No Fixing this, really
                throw new Error("Unsupported Encoding Exception",ex);
            }
        }
        // Now compute and add the signature, if appropriate
        if (Flickr.getSharedSecret() != null) {
            requestParam += "&api_sig="+getSig(paramString);
        }
        return requestParam;
    }
    
    /**
     * Get a signed map of parameter strings.  Does not URLEncode the strings,
     * since that is not required for FORM based requests.
     */
    private Map<String, String> getParamMap() throws IOException {
        Map<String, String> requestMap = new HashMap<String, String>();
        
        // string to compute the signature
        String paramString = "";
        boolean first = true;
        for (Map.Entry<String,String>entry : parameters.entrySet()) {
            String key = entry.getKey();
            String val = entry.getValue();
            requestMap.put(key,val);
            paramString += key+val;
        }
        // Now compute and add the signature, if appropriate
        if (Flickr.getSharedSecret() != null) {
            requestMap.put("api_sig",getSig(paramString));
        }
        return requestMap;
    }
    
    private InputStream getConnectionResponse() throws IOException {
        String urlString = getURL();
        URL url = new URL(urlString);
        HttpURLConnection con = (HttpURLConnection) url.openConnection();
        InputStream in = null;
        OutputStream out;
        String param;
        byte[] buff;
        switch (type) {
            case GET:
                con.setRequestMethod("GET");
                con.setDoOutput(false);
                con.setDoInput(true);
                con.connect();
                in = con.getInputStream();
                break;
            case POST:
                con.setRequestMethod("POST");
                con.setDoOutput(true);
                con.setDoInput(true);
                con.connect();
                out = con.getOutputStream();
                param = getParams();
                buff = param.getBytes("UTF8");
                out.write(buff);
                out.flush();
                out.close();
                in = con.getInputStream();
                break;
            case FORM:
                con.setRequestMethod("POST");
                con.setDoOutput(true);
                con.setDoInput(true);
                con.setUseCaches(false);
                con.setDefaultUseCaches(false);
                String boundary = MultipartFormOutputStream.createBoundary();
                // The following request properties were recommended by the MPF class
                // TODO: Revisit this set, I think some aren't necessary
                con.setRequestProperty("Accept", "*/*");
                con.setRequestProperty("Content-Type", MultipartFormOutputStream.getContentType(boundary));
                con.setRequestProperty("Connection", "Keep-Alive");
                con.setRequestProperty("Cache-Control", "no-cache");
                con.connect();
                MultipartFormOutputStream mpout = new MultipartFormOutputStream(con.getOutputStream(), boundary);
                // First, write out the parameters
                Map<String,String> paramMap = getParamMap();
                for (Map.Entry<String,String>entry : paramMap.entrySet()) {
                    mpout.writeField(entry.getKey(),entry.getValue());
                }
                // Now write the photo
                mpout.writeFile("photo","image/jpeg",image);
                mpout.flush();
                mpout.close();
                // All done, get the InputStream
                in = con.getInputStream();
                break;
        }
        
        return in;
    }
    
    /**
     * Utility method to get a JDOM Document corresponding to the Flickr Response.
     * Screens for errors, throws FlickrExceptions if any failures.  Attempts to
     * set return codes in the FlickrException in the event of errors.  Notably,
     * "Not Found" errors are return code 1.   See Flickr documentation for a complete list
     *  of error codes.  Note:  End users usually won't need
     * to use this method, but may wish to in order to get functionality not provided
     * by the API.
     *
     * @return doc JDOM Document returned by FlJickr
     * @throws FlickrException In the event of any error.
     */
    Document getResponse() throws FlickrException {
        
        SAXBuilder sb = new SAXBuilder();
        XMLOutputter out = new XMLOutputter();
        Document doc;
        Element root;
        InputStream in = null;
        
        try {
            
            in = getConnectionResponse();
            doc = sb.build(in);
            root = doc.getRootElement();
            
            if (!root.getAttributeValue("stat").equals("ok")) {
                Element err = root.getChild("err");
                String codeString = err.getAttributeValue("code");
                int code = 0;
                if (codeString != null) {
                    try {
                        code = Integer.parseInt(codeString);
                    } catch (NumberFormatException nfe) {
                        // I'm stumped, we'll have to carry on
                        Logger.global.severe("Warning: Unexpected Return Code Returned, continuing: "+codeString);
                    }
                }
                FlickrException fe = new FlickrException("Error: "+err.getAttributeValue("msg")
                +" (Code: "+err.getAttributeValue("code")+")",code);
                throw fe;
            }
            return doc;
        } catch (IOException ex) {
            throw new FlickrException("IO Error: "+ex.getMessage(),ex);
        } catch (JDOMException ex) {
            throw new FlickrException("Parse Error: "+ex.getMessage(),ex);
        } finally {
            try {
                in.close();
            } catch (Exception ex) {
                // And silently ignore, since we're just trying to see if it works.
                // If it doesn't work, that's good too.
            }
        }
    }
    
    public String toString() {
        return getURL();
    }
}
