/*
 * Tag.java
 *
 *  Copyright (c) 2006, James G. Driscoll 
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr;

import java.util.ArrayList;
import java.util.List;
import org.jdom.Element;

/**
 * Encapsulates the information about a Tag associated with a Photo.
 *
 * @see Photo#getTags()
 *
 * @author Jim Driscoll <a href="mailto:jgd@jamesgdriscoll.com">jgd@jamesgdriscoll.com</a>
 */
public class Tag {
    
    private Photo photo;
    private String id;
    private User author;
    private String rawText;
    private String text;
    
    private boolean deleted = false;
    
    /**
     * Creates a new instance of Tag.
     */
    Tag(Photo photo, Element e) {
        if (photo == null) throw new NullPointerException("Photo cannot be null");
        if (e == null) throw new NullPointerException("Element must not be null");
        this.photo = photo;
        id = e.getAttributeValue("id");
        rawText = e.getAttributeValue("raw");
        author = new User(e.getAttributeValue("author"));
        text = e.getText();
    }
    
    /**
     * <P>Get a list of all tags related to the tag supplied, as determined 
     * by Flickr.</P>
     * <P>For instance, a lookup of all tags related to "cat" will return such
     * things as "pet" and "kitty".</P>
     *
     * @param search String of the tag to search.
     * @return A list of String representation of tags, related to the search term by Flickr.
     * @throws FlickrException on any error.
     */
    public static List<String> getRelated(String search) throws FlickrException {
        Request req = new Request();
        req.setParameter("method","flickr.tags.getRelated");
        req.setParameter("tag",search);
        
        Element root = req.getResponse().getRootElement();
        Element tags = root.getChild("tags");
        List<String> taglist = new ArrayList<String>();
        if (tags == null) return taglist;
        List<Element> tagElements = tags.getChildren("tag");
        for (Element tag: tagElements) {
            taglist.add(tag.getText());
        }
        return taglist;
    }
    
    /**
     * Gets the unique ID of this tag.
     * @return The unique ID of this tag.
     */
    public String getID() {
        if (deleted) throw new IllegalStateException("Tag has already been deleted");
        return id;
    }
    
    /**
     * Gets the Author of this tag.
     * @return The User which is the author of this tag.
     */
    public User getAuthor() {
        if (deleted) throw new IllegalStateException("Tag has already been deleted");
        return author;
    }
    
    /**
     * Gets the text of this tag as the user entered it.  Typically, you would
     * instead use the value returned by Tag.getText() instead.
     * @see Tag#getText()
     * @return The raw text of this tag.
     */
    public String getRawText() {
        if (deleted) throw new IllegalStateException("Tag has already been deleted");
        return rawText;
    }
    
    /**
     * Gets the text of this tag as used by Flickr.  This means it will be
     * sanitized, with spaces and other marks removed.  The exact mapping is
     * not publicly defined by Flickr.
     * @return The text of this tag.
     */
    public String getText() {
        if (deleted) throw new IllegalStateException("Tag has already been deleted");
        return text;
    }
    
    /**
     * Get the Photo that this Tag is attached to.
     * @return The Photo that has this tag.
     */
    public Photo getParentPhoto() {
        if (deleted) throw new IllegalStateException("Tag has already been deleted");
        return photo;
    }
    
    /**
     * Deletes the Tag.  The Tag will no longer be a valid object, and should be discarded.
     * @throws FlickrException in the event of any error.
     */
    public void remove() throws FlickrException {
        if (deleted) throw new IllegalStateException("Tag has already been deleted");
        Request req = new Request(Request.POST);
        req.setParameter("method","flickr.photos.removeTag");
        req.setParameter("tag_id",id);
        req.getResponse();
        deleted = true;
    }
    
    /**
     * Check if this Tag is equal to the object supplied.  Uses id for comparison.
     * @param o Object to compare.
     * @return If the object to equal to this object.
     */
    public boolean equals(Object o) {
        if (o == null) return false;
        if (! (o instanceof Tag)) return false;
        return this.getID().equals(((Tag)o).getID());
    }
    
    /**
     * Return the HashCode of this Tag, based on ID.
     * @return int value representing a fairly unique hashcode.
     */
    public int hashCode() {
        return this.getID().hashCode();
    }
    
}