/*
 * User.java
 *
 *  Copyright (c) 2006, James G. Driscoll 
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.jickr;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.jdom.Document;
import org.jdom.Element;

/**
 * Information on a User of Flickr.
 *
 *
 * @author Jim Driscoll <a href="mailto:jgd@jamesgdriscoll.com">jgd@jamesgdriscoll.com</a>
 */
public class User {
    
    private enum Lookup { USER, EMAIL }
    
    // Flickr User Name
    private String username;
    // Real Name of User
    private String realname;
    // Flickr unique Identifier for user
    private String id;
    // Freeform text location of user
    private String location;
    private URL photosurl;
    private URL profileurl;
    private int photocount;
    private boolean pro;
    
    private boolean checked = false;
    
    /**
     * Creates a new instance of User with the given ID.  Note: Will not query
     * Flickr before creating object.  This is done for performance reasons, but
     * it does lead to odd behavior - like getting a message that the ID is invalid
     * the next time you use it.  To create an object with validation, use 
     * <code>User.findByID(String)</code>
     *
     * @see User#findByID(String)
     */
   public User(String id) {
        this.id = id;
    }
    
    /**
     * Creates a new instance of User.
     */
    private User(Lookup type, String search) throws FlickrException {
        Request req = new Request();
        switch (type) {
            case USER:
                req.setParameter("method","flickr.people.findByUsername");
                req.setParameter("username",search);
                break;
            case EMAIL:
                req.setParameter("method","flickr.people.findByEmail");
                req.setParameter("find_email",search);
                break;
        }
        Element root = req.getResponse().getRootElement();
        try {
            id = root.getChild("user").getAttributeValue("nsid");
        } catch (NullPointerException npe) {
            throw new FlickrException("Malformed XML",npe);
        }
    }
    
    /**
     *  Return a new User object that corresponds to the stated username.
     *
     * @param username Username of the User. Must not be null.
     * @throws FlickrException for almost anything, including not found errors
     */
    public static User findByUsername(String username) throws FlickrException {
        if (username == null) throw new NullPointerException("Username must not be null");
        return new User(Lookup.USER, username);
    }
    
    /**
     * Return a new User object that corresponds to the stated Email Address.
     *
     * @param emailAddr Email address <code>&qt;user@domain.com&qt;</code>
     * @return user A User object that corresponds to that email address. Must not be null.
     * @throws FlickrException for almost anything, including not found errors
     */
    public static User findByEmail(String emailAddr) throws FlickrException {
        if (emailAddr == null) throw new NullPointerException("emailAddr must not be null");
        return new User(Lookup.EMAIL, emailAddr);
    }
    
    /**
     * Return a new User object that corresponds to the stated user ID.  A user ID
     * is a Flickr specific unique identifier.
     * <P>Note: this call creates a new User object without querying Flickr, so
     * there is no guarentee that this object is valid.  Validity will be checked
     * once other calls are made to request data.  This was done for performance
     * reasons in other parts of the API.</P>
     *
     * @param id Flickr specific identifier.  Must not be null.
     * @return User object that corresponds to that email address.
     */
    public static User findByID(String id) {
        if (id == null) throw new NullPointerException("ID may not be null");
        return new User(id);
    }
    
    /**
     * Return a new User object that corresponds to the stated URL.
     *
     * @param userURL Flickr URL for this user.
     * @return User object that corresponds to the stated URL.
     */
    public static User findByURL(String userURL) throws FlickrException {
        if (userURL == null) throw new NullPointerException("userURL may not be null");
        Request req = new Request();
        req.setParameter("method","flickr.urls.lookupUser");
        req.setParameter("url",userURL.toString());
        Element root = req.getResponse().getRootElement();
        try {
            String userID = root.getChild("user").getAttributeValue("id");
            return new User(userID);
        } catch (NullPointerException npe) {
            throw new FlickrException("Malformed XML Error",npe);
        }
    }
    
    /**
     * Get the User Name
     *
     *
     * @return username Flickr user name
     */
    public String getUserName() throws FlickrException {
        if (!checked) checkInfo();
        return username;
    }
    
    /**
     * Get the real name.
     *
     *
     * @return realname The user's real name, as listed in Flickr.
     */
    public String getRealName() throws FlickrException {
        if (!checked) checkInfo();
        return realname;
    }
    
    /**
     * Get the location listed for this user in Flickr.
     *
     *
     * @return location Location of this user, as listed in Flickr.
     */
    public String getLocation() throws FlickrException {
        if (!checked) checkInfo();
        return location;
    }
    
    /**
     * Gets the URL for photos for this user.
     * @return URL of the user's photos
     */
    public URL getPhotosURL() throws FlickrException {
        if (!checked) checkInfo();
        return photosurl;
    }
    
    /**
     * Gets the URL of the user profile.
     * @return URL of the user profile
     */
    public URL getProfileURL() throws FlickrException {
        if (!checked) checkInfo();
        return profileurl;
    }
    
    /**
     * Gets the number of photos the user owns.
     * @return photocount - count of photos the user owns
     *
     */
    public int getPhotoCount() throws FlickrException {
        if (!checked) checkInfo();
        return photocount;
    }
    
    /**
     * Gets whether the User is registered with a Pro account.
     * @return If the user has a Pro account.
     * @throws FlickrException on any error.
     */
    public boolean isPro() throws FlickrException {
        if (!checked) checkInfo();
        return pro;
    }
    
    /**
     *  Gets the value of User's ID, the flickr unique identifier used in most of their URLs.
     *
     * @return id - Unique identifier used in FJickrURLs.
     */
    public String getID() {
        // Since this is always set, no check necessary
        return id;
    }
    
    /**
     * Get a list of users who are listed as contacts for this User.
     *
     * @return userlist A list of FlickrUsers.
     */
    public List<User> getPublicContacts() throws FlickrException {
        
        ArrayList<User> userlist = new ArrayList();
        
        Request req = new Request();
        req.setParameter("method","flickr.contacts.getPublicList");
        req.setParameter("user_id",id);
        
        Document doc = req.getResponse();
        Element root = doc.getRootElement();
        
        List<Element> contacts = root.getChild("contacts").getChildren("contact");
        
        for (Element contact: contacts)  {
            userlist.add(new User(contact.getAttributeValue("nsid")));
        }
        
        return userlist;
    }
    
    /**
     * Get a list of users that are contacts for the authenticated user.  Requires
     * READ privlege.
     * @return userlist A list of users who are contacts for the authenticated user.
     * @throws FlickrException on any error, including not being authenticated.
     */
    public static List<User> getContacts() throws FlickrException {
        //TODO - add contact filter
        User user = Auth.getAuthContext();
        if (!Auth.isAuthenticated(user,Permission.READ)) throw new FlickrRuntimeException("Needs READ permission");
        
        ArrayList<User> userlist = new ArrayList();
        
        Request req = new Request();
        req.setParameter("method","flickr.contacts.getList");
        Document doc = req.getResponse();
        Element root = doc.getRootElement();
        
        List<Element> contacts = root.getChild("contacts").getChildren("contact");
        for (Element contact: contacts) {
            userlist.add(new User(contact.getAttributeValue("nsid")));
        }
        return userlist;
    }
    
    /**
     * Get a list of all public photos for this user.
     * @return photolist A list of all public photos  belonging to this user.
     * @throws FlickrException in the event of any error.
     */
    public List<Photo> getPublicPhotos() throws FlickrException {
        Request req = new Request();
        req.setParameter("method","flickr.people.getPublicPhotos");
        req.setParameter("user_id",id);
        return new PhotoList(req);
    }
    
    /**
     * Get a list of all public favorite photos for this user.
     * @return photolist A list of all public favorite photos for this user.
     * @throws FlickrException in the event of any error.
     */
    public List<Photo> getPublicFavoritePhotos() throws FlickrException {
        Request req = new Request();
        req.setParameter("method","flickr.favorites.getPublicList");
        req.setParameter("user_id",id);
        return new PhotoList(req);
    }
    
    /**
     * Get a list of all favorite photos for this user.  This call requires
     * READ Permissions.
     * @return photolist A list of all favorite photos for this user.
     * @throws FlickrException in the event of any error.
     */
    public List<Photo> getFavoritePhotos() throws FlickrException {
        Request req = new Request();
        req.setParameter("method","flickr.favorites.getList");
        req.setParameter("user_id",id);
        return new PhotoList(req);
    }
    
    /**
     * Get a list of all public photos for this user's contacts.
     * @return photolist A list of 10 publicly viewable photos for this user's contacts.
     * @throws FlickrException in the event of any error.
     */
    public List<Photo> getPublicContactPhotos() throws FlickrException {
        return getPublicContactPhotos(false, 10);
    }
    
    /**
     * Get a list of all public photos for this user's contacts.
     * @return photolist A list of all public photos for this user's contacts.
     * @param justFriends Return only friends' photos, or all contacts' photos.
     * @param count How many photos to return - default is 10, max is 50.
     * @throws FlickrException in the event of any error.
     */
    public List<Photo> getPublicContactPhotos(boolean justFriends, int count) throws FlickrException {
        Request req = new Request();
        req.setParameter("method","flickr.photos.getContactsPublicPhotos");
        req.setParameter("user_id",id);
        req.setParameter("count",count+"");
        if (justFriends) req.setParameter("just_friends","1");
        return new PhotoList(req);
    }
    
    /**
     * Get a list of the PhotoSets for this user.
     * @return photosetlist A list of all PhotoSets for this user
     * @throws FlickrException in the event of any error
     */
    public List<PhotoSet> getPhotoSets() throws FlickrException {
        
        Request req = new Request();
        req.setParameter("method","flickr.photosets.getList");
        req.setParameter("user_id",id);
        
        Document doc = req.getResponse();
        Element root = doc.getRootElement();
        List tempList = new ArrayList();
        
        List<Element> photosets = root.getChild("photosets").getChildren("photoset");
        for (Element photoset : photosets)  {
            tempList.add(new PhotoSet(photoset));
        }
        return tempList;
    }
    
    /**
     * Get a list of all the tags associated with this User's Photos.
     * @return A list of tags in String form.
     * @throws FlickrException on any error.
     */
    public List<String> getTags() throws FlickrException {
        Request req = new Request();
        req.setParameter("method","flickr.tags.getListUser");
        req.setParameter("user_id",id);
        
        Document doc = req.getResponse();
        Element root = doc.getRootElement();
        
        try {
            Element tags = root.getChild("who").getChild("tags");
            List taglist = new ArrayList<String>();
            if (tags == null) return taglist;
            List<Element> tagElements = tags.getChildren("tag");
            for (Element tag: tagElements) {
                taglist.add(tag.getText());
            }
            return taglist;
        } catch (NullPointerException npe) {
            throw new FlickrException("Malformed XML",npe);
        }
    }
    
    /**
     * Get a list of the popular tags associated with this User's Photos, and how
     * many times they are used.
     * @param count The number of popular tags to return.  Must be greater than 0.
     * @return An unordered Map of tags in String form, with # of uses in Integer form.
     * @throws FlickrException on any error.
     */
    public Map<String,Integer> getPopularTags(int count) throws FlickrException {
        if (count <= 0) throw new IllegalArgumentException("Count must be > 0");
        Request req = new Request();
        req.setParameter("method","flickr.tags.getListUserPopular");
        req.setParameter("user_id",id);
        req.setParameter("count",count+"");
        
        Document doc = req.getResponse();
        Element root = doc.getRootElement();
        
        try {
            Element tags = root.getChild("who").getChild("tags");
            Map<String,Integer> tagmap = new HashMap<String,Integer>();
            if (tags == null) return tagmap;
            List<Element> tagElements = tags.getChildren("tag");
            for (Element tag: tagElements) {
                String tagtext = tag.getText();
                String countstring = tag.getAttributeValue("count");
                Integer tagcount = Integer.valueOf(countstring);
                tagmap.put(tagtext,tagcount);
            }
            return tagmap;
        } catch (NullPointerException npe) {
            throw new FlickrException("Malformed XML",npe);
        } catch (NumberFormatException nfe) {
            throw new FlickrException("Malformed XML",nfe);
        }
    }
    
    private void checkInfo() throws FlickrException {
        Request req = new Request();
        req.setParameter("method","flickr.people.getInfo");
        req.setParameter("user_id",id);
        
        Element root = req.getResponse().getRootElement();
        
        try {
            username = root.getChild("person").getChildText("username");
            realname = root.getChild("person").getChildText("realname");
            location = root.getChild("person").getChildText("location");
            photocount = Integer.parseInt(root.getChild("person").getChild("photos").getChildText("count"));
            this.pro = (root.getChild("person").getAttributeValue("ispro").equals("1"));
            photosurl = new URL(root.getChild("person").getChild("photosurl").getText());
            profileurl = new URL(root.getChild("person").getChild("profileurl").getText());
        } catch (MalformedURLException ex) {
            throw new FlickrException("Malformed URL: ",ex);
        } catch (NullPointerException npe) {
            throw new FlickrException("Malformed XML",npe);
        } catch (NumberFormatException nfe) {
            throw new FlickrException("Malformed XML",nfe);
        }
        checked = true;
    }
    
    /**
     * Return a String rendition of this object.  Use the User ID for this purpose.
     * @return A String representation of this object.
     */
    public String toString() {
        return id;
    }
    
    /**
     * Check for equality of two User objects.  Uses User ID interally.
     *
     * @param o - Object to compare.
     * @return Whether the object is equal to this object.
     */
    public boolean equals(Object o) {
        try {
            if (!(o instanceof User)) return false;
            User u = (User)o;
            if (u == null) return false;
            return this.getID().equals(u.getID());
        } catch (ClassCastException cce) {
            return false;
        }
    }
    
    /**
     * Create a hashcode for the User object, based on ID.
     * @return A hashcode for this object.
     */
    public int hashCode() {
        return this.getID().hashCode();
    }
}
